(*  title     : An encoding of ITL in Isabelle/HOL
    Authors   : Antonio Cau     <cau.researcher at gmail.com>
                Ben Moszkowski
                David Smallwood <drs at dmu.ac.uk>
    Maintainer: Antonio Cau     <cau.researcher at gmail.com>        
    License   : BSD
*)

theory Theorems
 imports
   ITL
begin
(* sledgehammer_params [minimize=true,preplay_timeout=10,timeout=60,verbose=true,
                    provers="cvc4 z3 e spass vampire " ]
*)

section \<open>ITL theorems\<close>

text \<open>We give the proofs of a list of ITL theorems. These proofs and theorems were from 
\cite{Ben:ImpReaPro}.\<close>

subsection \<open>Propositional reasoning\<close>

text \<open>This is a list of propositional logic theorems used in the proofs of the ITL theorems.\<close>

lemma itl_prop:
  "\<turnstile> (f \<equiv>\<^sub>i f) \<equiv>\<^sub>i true\<^sub>i "
  "\<turnstile> (\<not>\<^sub>i true\<^sub>i) \<equiv>\<^sub>i false\<^sub>i"
  "\<turnstile> (\<not>\<^sub>i false\<^sub>i) \<equiv>\<^sub>i true\<^sub>i"
  "\<turnstile> (\<not>\<^sub>i \<not>\<^sub>i f) \<equiv>\<^sub>i f"
  "\<turnstile> (\<not>\<^sub>i f \<equiv>\<^sub>i f) \<equiv>\<^sub>i false\<^sub>i"
  "\<turnstile> (f \<equiv>\<^sub>i \<not>\<^sub>i f) \<equiv>\<^sub>i false\<^sub>i"
  "\<turnstile> (\<not>\<^sub>i (f \<equiv>\<^sub>i g)) \<equiv>\<^sub>i (f \<equiv>\<^sub>i \<not>\<^sub>i g)"
  "\<turnstile> (true\<^sub>i \<equiv>\<^sub>i f) \<equiv>\<^sub>i f"
  "\<turnstile> (f \<equiv>\<^sub>i true\<^sub>i) \<equiv>\<^sub>i f"
  "\<turnstile> (true\<^sub>i \<supset>\<^sub>i f) \<equiv>\<^sub>i f"
  "\<turnstile> (false\<^sub>i \<supset>\<^sub>i f) \<equiv>\<^sub>i true\<^sub>i"
  "\<turnstile> (f \<supset>\<^sub>i true\<^sub>i) \<equiv>\<^sub>i true\<^sub>i"
  "\<turnstile> (f \<supset>\<^sub>i f) \<equiv>\<^sub>i true\<^sub>i"
  "\<turnstile> (f \<supset>\<^sub>i  false\<^sub>i) \<equiv>\<^sub>i \<not>\<^sub>i f"
  "\<turnstile> (f \<supset>\<^sub>i \<not>\<^sub>i f) \<equiv>\<^sub>i \<not>\<^sub>i f"
  "\<turnstile> (f \<and>\<^sub>i true\<^sub>i) \<equiv>\<^sub>i f"
  "\<turnstile> (true\<^sub>i \<and>\<^sub>i f) \<equiv>\<^sub>i f"
  "\<turnstile> (f \<and>\<^sub>i false\<^sub>i) \<equiv>\<^sub>i false\<^sub>i"
  "\<turnstile> (false\<^sub>i \<and>\<^sub>i f) \<equiv>\<^sub>i false\<^sub>i"
  "\<turnstile> (f \<and>\<^sub>i f) \<equiv>\<^sub>i f"
  "\<turnstile> (f \<and>\<^sub>i \<not>\<^sub>i f) \<equiv>\<^sub>i false\<^sub>i"
  "\<turnstile> (\<not>\<^sub>i f \<and>\<^sub>i f) \<equiv>\<^sub>i false\<^sub>i"
  "\<turnstile> (f \<or>\<^sub>i true\<^sub>i) \<equiv>\<^sub>i true\<^sub>i"
  "\<turnstile> (true\<^sub>i \<or>\<^sub>i f) \<equiv>\<^sub>i true\<^sub>i"
  "\<turnstile> (f \<or>\<^sub>i false\<^sub>i) \<equiv>\<^sub>i f"
  "\<turnstile> (false\<^sub>i \<or>\<^sub>i f) \<equiv>\<^sub>i f"
  "\<turnstile> (f \<or>\<^sub>i f) \<equiv>\<^sub>i f"
  "\<turnstile> (f \<or>\<^sub>i \<not>\<^sub>i f) \<equiv>\<^sub>i true\<^sub>i"
  "\<turnstile> (\<not>\<^sub>i f \<or>\<^sub>i f) \<equiv>\<^sub>i true\<^sub>i"
  "(\<turnstile> f \<equiv>\<^sub>i f1) = (\<turnstile> f1 \<equiv>\<^sub>i f)"
  "(\<turnstile> f \<equiv>\<^sub>i f1) = ( (\<turnstile> f \<supset>\<^sub>i f1) \<and>  (\<turnstile> f1 \<supset>\<^sub>i f) )"
  "(\<turnstile> f  \<supset>\<^sub>i ( f1 \<and>\<^sub>i f2)) = ( (\<turnstile> f \<supset>\<^sub>i f1) \<and> (\<turnstile> f \<supset>\<^sub>i f2))" 
  "(\<turnstile> f \<equiv>\<^sub>i f1) = (\<turnstile> \<not>\<^sub>i f \<equiv>\<^sub>i \<not>\<^sub>if1)"
  "(\<turnstile> \<not>\<^sub>i ( f \<or>\<^sub>i f1)) = (\<turnstile> \<not>\<^sub>i f \<and>\<^sub>i \<not>\<^sub>i f1)" 
  "(\<turnstile>  ( f \<supset>\<^sub>i f1)) = (\<turnstile>  ( \<not>\<^sub>if \<or>\<^sub>i f1) )" 
by auto

lemma prop01:
 assumes "\<turnstile> f \<equiv>\<^sub>i g"
 shows   "\<turnstile> \<not>\<^sub>i f \<equiv>\<^sub>i \<not>\<^sub>i g"
using assms itl_prop(33) by blast

lemma prop02:
 assumes "\<turnstile> f \<supset>\<^sub>i g"
         "\<turnstile> g \<supset>\<^sub>i h "
 shows   "\<turnstile> f \<supset>\<^sub>i h"
using assms(1) assms(2) by auto

lemma prop03:
 assumes "\<turnstile> f \<equiv>\<^sub>i g"
         "\<turnstile> g \<equiv>\<^sub>i h "
 shows   "\<turnstile> f \<equiv>\<^sub>i h"
using assms(1) assms(2) by auto

lemma prop04:
 " \<turnstile> \<not>\<^sub>i ( f \<and>\<^sub>i g \<and>\<^sub>i \<not>\<^sub>i h) \<equiv>\<^sub>i (f \<supset>\<^sub>i (g \<supset>\<^sub>i h))"
by auto

lemma prop05:
 assumes "\<turnstile> f \<equiv>\<^sub>i g"
 shows   "\<turnstile> h \<and>\<^sub>i f \<equiv>\<^sub>i h \<and>\<^sub>i g"
using assms by auto

lemma prop06:
 assumes "\<turnstile> f \<equiv>\<^sub>i g"
 shows   "\<turnstile> f \<and>\<^sub>i h \<equiv>\<^sub>i g \<and>\<^sub>i h"
using assms by auto

lemma prop07:
 assumes "\<turnstile> f \<equiv>\<^sub>i f1 "
         "\<turnstile> g \<equiv>\<^sub>i g1"
 shows   "\<turnstile> (if\<^sub>i  (init w)  then f  else g) \<equiv>\<^sub>i (if\<^sub>i  (init w)  then f1  else g1)"
using assms(1)  assms(2) by simp

lemma prop08:
 assumes "\<turnstile> f \<and>\<^sub>i g \<supset>\<^sub>i h"
 shows   "\<turnstile> f \<and>\<^sub>i g \<and>\<^sub>i f1 \<supset>\<^sub>i h "
using assms by auto

lemma prop09:
 assumes "\<turnstile> f \<and>\<^sub>i g \<and>\<^sub>i h \<supset>\<^sub>i  f1 "
         "\<turnstile> f \<and>\<^sub>i h \<and>\<^sub>i g \<supset>\<^sub>i \<not>\<^sub>i f1 "
 shows   "\<turnstile> \<not>\<^sub>i (f \<and>\<^sub>i h \<and>\<^sub>i g) "
using assms(1)  assms(2) by auto

lemma prop10:
 assumes "\<turnstile> f \<and>\<^sub>i f1 \<supset>\<^sub>i h"
 shows   "\<turnstile> f \<and>\<^sub>i g \<and>\<^sub>i f1 \<supset>\<^sub>i h "
using assms by auto

lemma prop11:
 "\<turnstile> (if\<^sub>i  g  then  f  else f1) \<supset>\<^sub>i (( g \<supset>\<^sub>i f) \<and>\<^sub>i (\<not>\<^sub>ig \<supset>\<^sub>i f1)) "
by simp 

lemma prop12:
 assumes "\<turnstile> f \<supset>\<^sub>i g"
 shows   " \<turnstile> h \<and>\<^sub>i f \<supset>\<^sub>i h \<and>\<^sub>i g"
using assms by auto

lemma prop13:
 assumes "\<turnstile> f \<supset>\<^sub>i \<not>\<^sub>i g \<or>\<^sub>i h"
 shows   "\<turnstile> g \<and>\<^sub>i f \<supset>\<^sub>i h"
using assms by auto

lemma prop14:
 assumes "\<turnstile> f \<equiv>\<^sub>i g \<or>\<^sub>i h"
         "\<turnstile> h \<supset>\<^sub>i h1"
 shows   "\<turnstile> f \<supset>\<^sub>i g  \<or>\<^sub>i h1"
using assms(1) assms(2) by auto 

lemma prop15:
 assumes "\<turnstile> f \<equiv>\<^sub>i g"
         "\<turnstile> h \<supset>\<^sub>i g"
 shows   "\<turnstile> h \<supset>\<^sub>i f"
using assms(1) assms(2) by auto

lemma prop16:
 assumes "\<turnstile> f \<supset>\<^sub>i g \<or>\<^sub>i h"
         "\<turnstile> g \<supset>\<^sub>i h1 \<or>\<^sub>i h"
 shows   "\<turnstile> f \<supset>\<^sub>i h1 \<or>\<^sub>i h"
using assms(1) assms(2) by auto

lemma prop17:
 assumes "\<turnstile> f \<supset>\<^sub>i g"
         "\<turnstile> f1 \<supset>\<^sub>i g"
  shows  "\<turnstile> f \<or>\<^sub>i f1  \<supset>\<^sub>i g"
using assms(1) assms(2) by auto


lemma prop18:
 assumes "\<turnstile> g \<and>\<^sub>i h \<supset>\<^sub>i h1"
         "\<turnstile> f \<equiv>\<^sub>i g"
 shows   "\<turnstile> f \<and>\<^sub>i h \<supset>\<^sub>i h1"
using assms(1) assms(2) by auto

lemma prop19:
  assumes "\<turnstile> f \<equiv>\<^sub>i g \<or>\<^sub>i h "
  shows   "\<turnstile> h \<supset>\<^sub>i f"
using assms by auto  


lemma prop20:
 assumes "\<turnstile> f \<equiv>\<^sub>i g \<or>\<^sub>i h"
 shows   "\<turnstile> \<not>\<^sub>i f \<equiv>\<^sub>i \<not>\<^sub>ig \<and>\<^sub>i \<not>\<^sub>i h"
using assms by auto

lemma prop21:
 assumes "\<turnstile> f \<equiv>\<^sub>i h"
         "\<turnstile> f \<equiv>\<^sub>i h1"
 shows   "\<turnstile> h1 \<equiv>\<^sub>i h"
using assms(1) assms(2) by auto

lemma prop22:
 assumes "\<turnstile> f \<equiv>\<^sub>i h"
         "\<turnstile> g \<equiv>\<^sub>i g1"
 shows   "\<turnstile> f \<and>\<^sub>i g \<supset>\<^sub>i h \<and>\<^sub>i g1"
using assms(1) assms(2) by auto

lemma prop23:
 assumes "\<turnstile> f \<equiv>\<^sub>i g \<or>\<^sub>i h"
 shows   "\<turnstile> f \<and>\<^sub>i f1 \<equiv>\<^sub>i (g \<and>\<^sub>i f1) \<or>\<^sub>i (h \<and>\<^sub>i f1)"
 using assms by auto

lemma prop24:
 assumes "\<turnstile> f \<equiv>\<^sub>i g \<or>\<^sub>i (h \<and>\<^sub>i h1)"
         "\<turnstile> g \<equiv>\<^sub>i g1"
         "\<turnstile> h \<equiv>\<^sub>i h2 \<and>\<^sub>i h3"
         "\<turnstile> h3 \<and>\<^sub>i h1 \<equiv>\<^sub>i h4" 
 shows   "\<turnstile> f \<equiv>\<^sub>i g1 \<or>\<^sub>i (h2 \<and>\<^sub>i h4)"
using assms(1) assms(2) assms(3) assms(4) by auto

lemma prop25:
 assumes "\<turnstile> f \<supset>\<^sub>i false\<^sub>i"
 shows "\<turnstile> g \<or>\<^sub>i f \<equiv>\<^sub>i g "
using assms by auto

lemma prop26:
 assumes "\<turnstile> f \<supset>\<^sub>i g"
 shows "\<turnstile> f \<supset>\<^sub>i h \<or>\<^sub>i g "
using assms by auto

lemma prop27:
 assumes "\<turnstile> f \<supset>\<^sub>i g"
 shows "\<turnstile> \<not>\<^sub>i g \<supset>\<^sub>i \<not>\<^sub>i f"
using assms by auto

lemma prop28:
 assumes "\<turnstile> f \<equiv>\<^sub>i g \<or>\<^sub>i h "
         "\<turnstile> h \<equiv>\<^sub>i h1"
 shows "\<turnstile> f \<equiv>\<^sub>i g \<or>\<^sub>i h1 "
using assms(1) assms(2) by auto

lemma prop29:
 assumes "\<turnstile> f \<supset>\<^sub>i g \<or>\<^sub>i h "
 shows   "\<turnstile> f \<and>\<^sub>i \<not>\<^sub>i g \<supset>\<^sub>i h"
using assms by auto

lemma prop30:
 assumes "\<turnstile> f0 \<supset>\<^sub>i g"
         "\<turnstile> f1 \<supset>\<^sub>i g"
 shows "\<turnstile> f0 \<or>\<^sub>i f1 \<supset>\<^sub>i g "
using assms(1) assms(2) by auto

lemma prop31:
 "\<turnstile> (f \<supset>\<^sub>i (g \<equiv>\<^sub>i h)) \<equiv>\<^sub>i ( (f \<and>\<^sub>i g) \<equiv>\<^sub>i (f \<and>\<^sub>i h)) "
by auto

lemma prop32:
 assumes "\<turnstile> f \<and>\<^sub>i g \<supset>\<^sub>i h"
 shows   "\<turnstile> g \<supset>\<^sub>i (f \<supset>\<^sub>i h)"
using assms by auto

lemma prop33:
 assumes "\<turnstile> f0 \<and>\<^sub>i g \<supset>\<^sub>i h"
         "\<turnstile> f1 \<and>\<^sub>i g \<supset>\<^sub>i h"
 shows   "\<turnstile> (f0 \<or>\<^sub>i f1) \<and>\<^sub>i g \<supset>\<^sub>i h"
using assms(1) assms(2) by auto

lemma prop34:
 assumes "\<turnstile> f \<and>\<^sub>i g \<supset>\<^sub>i h"
         "\<turnstile> f"
 shows   "\<turnstile> g \<supset>\<^sub>i h"
using assms(1) assms(2) by auto  

lemma prop35:
 assumes "\<turnstile> f \<supset>\<^sub>i g \<or>\<^sub>i h"
         "\<turnstile> h \<supset>\<^sub>i h1"
 shows   "\<turnstile> f \<supset>\<^sub>i g \<or>\<^sub>i h1"
using assms(1) assms(2) by auto 

lemma prop36:
 assumes "\<turnstile> f \<and>\<^sub>i g \<supset>\<^sub>i h"
 shows   "\<turnstile> f \<supset>\<^sub>i (g \<supset>\<^sub>i h)"
using assms by auto

lemma prop37:
 assumes "\<turnstile> f1 \<supset>\<^sub>i f"
         "\<turnstile> f \<and>\<^sub>i f1 \<and>\<^sub>i g \<supset>\<^sub>i h"
 shows   " \<turnstile> f1 \<and>\<^sub>i g \<supset>\<^sub>i h"
using assms(1) assms(2) by auto 

lemma prop38:
 assumes "\<turnstile> f \<supset>\<^sub>i g"
 shows   "\<turnstile> f \<equiv>\<^sub>i f \<and>\<^sub>i g"
using assms by auto 

lemma prop39:
 assumes "\<turnstile> f \<equiv>\<^sub>i f1"
         "\<turnstile> g \<equiv>\<^sub>i g1"
 shows   "\<turnstile> (f \<supset>\<^sub>i g) \<equiv>\<^sub>i (f1 \<supset>\<^sub>i g1)"
using assms(1) assms(2) by auto 

lemma prop40:
 assumes "\<turnstile> f \<equiv>\<^sub>i f1"
         "\<turnstile> g \<equiv>\<^sub>i g1"
         "\<turnstile> h \<equiv>\<^sub>i h1"
 shows   "\<turnstile> (f \<equiv>\<^sub>i g \<and>\<^sub>i h) \<equiv>\<^sub>i (f1 \<equiv>\<^sub>i g1 \<and>\<^sub>i h1) "
using assms(1) assms(2) assms(3) by auto 

subsection \<open>State formulas\<close>

text \<open>
The \<open>init\<close> operator denotes state formulas, i.e., ITL formula that only constrain
the first state of an interval. 
\<close>

lemma Initprop :
  "\<turnstile> ((init f) \<and>\<^sub>i (init g)) \<equiv>\<^sub>i init(f \<and>\<^sub>i g)"
  "\<turnstile> (\<not>\<^sub>i (init f)) \<equiv>\<^sub>i init ( \<not>\<^sub>i f )"
  "\<turnstile> ((init f) \<or>\<^sub>i (init g)) \<equiv>\<^sub>i init (f \<or>\<^sub>i g)" 
  "\<turnstile> init true\<^sub>i" 
by auto 

lemma Finprop :
 "\<turnstile> (true\<^sub>i;(f \<and>\<^sub>i empty)) \<and>\<^sub>i (true\<^sub>i;(g \<and>\<^sub>i empty)) \<equiv>\<^sub>i (true\<^sub>i;(f \<and>\<^sub>i g \<and>\<^sub>i empty)) "
 "\<turnstile> (true\<^sub>i;(f \<and>\<^sub>i empty)) \<or>\<^sub>i (true\<^sub>i;(g \<and>\<^sub>i empty)) \<equiv>\<^sub>i (true\<^sub>i;((f \<or>\<^sub>i g) \<and>\<^sub>i empty)) "
 "\<turnstile> (true\<^sub>i;(true\<^sub>i \<and>\<^sub>i empty))"
 "\<turnstile> \<not>\<^sub>i (true\<^sub>i;(f \<and>\<^sub>i empty)) \<equiv>\<^sub>i (true\<^sub>i;(\<not>\<^sub>if \<and>\<^sub>i empty))"
apply simp_all 
apply auto[1]
apply auto[2]
using dual_order.order_iff_strict by fastforce


subsection \<open>Basic Theorems\<close>

lemma BiChopImpChop :
    " \<turnstile> bi (f \<supset>\<^sub>i f1) \<supset>\<^sub>i f;g \<supset>\<^sub>i f1;g "
 proof -
 have  1: "\<turnstile> g \<supset>\<^sub>i g" by auto
 hence 2: "\<turnstile> \<box> ( g \<supset>\<^sub>i g) " by (rule BoxGen)
 have  3: "\<turnstile> bi ( f \<supset>\<^sub>i f1) \<and>\<^sub>i \<box>(g \<supset>\<^sub>i g) \<supset>\<^sub>i f;g \<supset>\<^sub>i f1;g " by (rule BiBoxChopImpChop)
 from 2 3 show ?thesis  by auto
qed 

lemma AndChopA:
    " \<turnstile> (f \<and>\<^sub>i f1);g \<supset>\<^sub>i f;g "
proof -
 have  1: "\<turnstile> f \<and>\<^sub>i f1 \<supset>\<^sub>i f" by auto
 hence 2: "\<turnstile> bi (f \<and>\<^sub>i f1 \<supset>\<^sub>i f)" by (rule BiGen)
 have  3: "\<turnstile> bi (f \<and>\<^sub>i f1 \<supset>\<^sub>i f) \<supset>\<^sub>i (f  \<and>\<^sub>i f1);g \<supset>\<^sub>i f;g" by (rule BiChopImpChop)
 from 2 3 show ?thesis by auto
qed

lemma AndChopB:
    "\<turnstile> (f \<and>\<^sub>i f1);g \<supset>\<^sub>i f1;g "
proof -
 have  1: "\<turnstile> f \<and>\<^sub>i f1 \<supset>\<^sub>i f1" by auto
 hence 2: "\<turnstile>  bi (f \<and>\<^sub>i f1 \<supset>\<^sub>i f1)" by (rule BiGen)
 have  3: "\<turnstile>  bi (f \<and>\<^sub>i f1 \<supset>\<^sub>i f1) \<supset>\<^sub>i (f \<and>\<^sub>i f1);g \<supset>\<^sub>i f1;g" by (rule BiChopImpChop)
 from 2 3 show ?thesis by auto
qed

lemma NextChop:
  "\<turnstile> (\<circle> f);g  \<equiv>\<^sub>i \<circle>(f;g)"
proof -
 have 1: "\<turnstile> skip;(f;g)  \<equiv>\<^sub>i (skip;f);g" by (rule ChopAssoc)
 show ?thesis by (metis "1" itl_prop(30) next_d_def)
qed

lemma BoxChopImpChop :
    " \<turnstile>  \<box> (g \<supset>\<^sub>i g1) \<supset>\<^sub>i f;g \<supset>\<^sub>i f;g1 "
 proof -
 have  1: "\<turnstile> g \<supset>\<^sub>i g" by auto
 hence 2: "\<turnstile> bi ( g \<supset>\<^sub>i g) " by (rule BiGen)
 have  3: "\<turnstile> bi ( f \<supset>\<^sub>i f) \<and>\<^sub>i \<box>(g \<supset>\<^sub>i g1) \<supset>\<^sub>i f;g \<supset>\<^sub>i f;g1 " by (rule BiBoxChopImpChop)
 from 2 3 show ?thesis  by auto
qed 

lemma LeftChopImpChop:
 assumes "\<turnstile> f\<supset>\<^sub>i f1" 
 shows   "\<turnstile> f;g \<supset>\<^sub>i f1;g"
proof -
 have  1: "\<turnstile> f\<supset>\<^sub>i f1" using assms by auto
 hence 2: "\<turnstile> bi (f \<supset>\<^sub>i f1)" by (rule BiGen)
 have  3: "\<turnstile> bi (f \<supset>\<^sub>i f1) \<supset>\<^sub>i f;g \<supset>\<^sub>i f1;g" by (rule BiChopImpChop)
 from 2 3 show ?thesis using MP by blast
qed

lemma RightChopImpChop:
 assumes "\<turnstile> g \<supset>\<^sub>i g1" 
 shows   "\<turnstile> f;g \<supset>\<^sub>i f;g1"
proof -
 have  1: "\<turnstile> g \<supset>\<^sub>i g1" using assms by auto
 hence 2: "\<turnstile> \<box> (g \<supset>\<^sub>i g1)" by (rule BoxGen)
 have  3: "\<turnstile> \<box> (g \<supset>\<^sub>i g1) \<supset>\<^sub>i f;g \<supset>\<^sub>i f;g1" by (rule BoxChopImpChop)
 from 2 3 show ?thesis using MP by blast
qed
 
lemma RightChopEqvChop:
 assumes "\<turnstile> g \<equiv>\<^sub>i g1"
 shows   "\<turnstile> f;g \<equiv>\<^sub>i f;g1"
proof -
 have 1: "\<turnstile> g \<equiv>\<^sub>i g1" using assms by auto
 have 2: "(\<turnstile> g \<supset>\<^sub>i g1) \<Longrightarrow> (\<turnstile> f;g \<supset>\<^sub>i f;g1)" by (rule RightChopImpChop)
 have 3: "(\<turnstile> g1 \<supset>\<^sub>i g) \<Longrightarrow> (\<turnstile> f;g1 \<supset>\<^sub>i f;g)" by (rule RightChopImpChop)
 from 1 2 3 show ?thesis using itl_prop(31) by blast
qed

lemma ChopOrEqv:
 " \<turnstile> f;(g \<or>\<^sub>i g1) \<equiv>\<^sub>i f;g \<or>\<^sub>i f;g1 "
proof -
 have  1: "\<turnstile> g \<supset>\<^sub>i g \<or>\<^sub>i g1" by auto
 hence 2: "\<turnstile> f;g \<supset>\<^sub>i f;(g \<or>\<^sub>i g1)" by (rule RightChopImpChop)
 have  3: "\<turnstile> g1 \<supset>\<^sub>i g \<or>\<^sub>i g1" by auto 
 hence 4: "\<turnstile> f;g1 \<supset>\<^sub>i f;(g \<or>\<^sub>i g1)" by (rule RightChopImpChop)
 from 2 4 show ?thesis by auto
qed

lemma OrChopEqv:
 " \<turnstile> (f \<or>\<^sub>i f1);g \<equiv>\<^sub>i f;g \<or>\<^sub>i f1;g "
proof -
 have  1: "\<turnstile> f \<supset>\<^sub>i f \<or>\<^sub>i f1" by auto
 hence 2: "\<turnstile> f;g \<supset>\<^sub>i (f\<or>\<^sub>i f1);g" by (rule LeftChopImpChop)
 have  3: "\<turnstile> f1 \<supset>\<^sub>i f \<or>\<^sub>i f1" by auto 
 hence 4: "\<turnstile> f1;g \<supset>\<^sub>i (f\<or>\<^sub>i f1);g" by (rule LeftChopImpChop)
 from 2 4 show ?thesis by auto
qed

lemma OrChopImpRule:
 assumes "\<turnstile> f \<supset>\<^sub>i f1 \<or>\<^sub>i f2"
 shows   " \<turnstile> f;g \<supset>\<^sub>i (f1;g) \<or>\<^sub>i (f2;g)"
proof -
 have  1: "\<turnstile> f \<supset>\<^sub>i f1 \<or>\<^sub>i f2" using assms by auto
 hence 2: "\<turnstile> f;g \<supset>\<^sub>i (f1 \<or>\<^sub>i f2);g" by (rule LeftChopImpChop)
 have  3: "\<turnstile> (f1 \<or>\<^sub>i f2); g \<equiv>\<^sub>i f1;g \<or>\<^sub>i f2;g" by (rule OrChopEqv) 
 from 2 3 show ?thesis by auto
qed

lemma  LeftChopEqvChop:
 assumes "\<turnstile> f \<equiv>\<^sub>i f1"
 shows   " \<turnstile> f;g \<equiv>\<^sub>i f1;g"
proof -
 have  1: "\<turnstile> f \<equiv>\<^sub>i f1" using assms by auto
 hence 2: "\<turnstile> f \<supset>\<^sub>i f1" by auto
 hence 3: "\<turnstile> f;g \<supset>\<^sub>i f1;g" by (rule LeftChopImpChop)
 from  1 have "\<turnstile> f1 \<supset>\<^sub>i f" by auto
 hence 4: "\<turnstile> f1;g \<supset>\<^sub>i f;g" by (rule LeftChopImpChop)
 from 3 4 show ?thesis by auto
qed


lemma OrChopEqvRule:
 assumes "\<turnstile> f \<equiv>\<^sub>i f1 \<or>\<^sub>i f2"
 shows   "\<turnstile> f;g \<equiv>\<^sub>i (f1;g) \<or>\<^sub>i (f2;g)"
proof -
 have  1: "\<turnstile> f \<equiv>\<^sub>i f1 \<or>\<^sub>i f2" using assms by auto
 hence 2: "\<turnstile> f;g \<equiv>\<^sub>i (f1 \<or>\<^sub>i f2);g" by (rule LeftChopEqvChop) 
 have  3: "\<turnstile> (f1 \<or>\<^sub>i f2);g \<equiv>\<^sub>i f1;g \<or>\<^sub>i f2;g" by (rule OrChopEqv)
 from 2 3 show ?thesis by auto
qed

lemma NextImpNext:
 assumes "\<turnstile> f \<supset>\<^sub>i g"
 shows   "\<turnstile> \<circle> f \<supset>\<^sub>i \<circle> g"
proof -
 have  1: "\<turnstile> f \<supset>\<^sub>i g" using assms by auto
 hence 2: "\<turnstile> \<box> (f \<supset>\<^sub>i g)" by (rule BoxGen)
 have  3: "\<turnstile> \<box> (f \<supset>\<^sub>i g) \<supset>\<^sub>i (skip;f) \<supset>\<^sub>i (skip;g)" by (rule BoxChopImpChop)
 have  4: "\<turnstile>(skip;f) \<supset>\<^sub>i (skip;g)" by (metis 2 3 MP)
 from 4 show ?thesis by (metis next_d_def)
qed

lemma ChopOrImpRule:
 assumes " \<turnstile> g \<supset>\<^sub>i g1 \<or>\<^sub>i g2"
 shows   " \<turnstile> f;g \<supset>\<^sub>i (f;g1) \<or>\<^sub>i (f;g2)"
proof -
 have  1: "\<turnstile> g \<supset>\<^sub>i g1 \<or>\<^sub>i g2" using assms by auto
 hence 2: "\<turnstile> f;g \<supset>\<^sub>i f;(g1 \<or>\<^sub>i g2)" by (rule RightChopImpChop)
 have  3: "\<turnstile> f;(g1 \<or>\<^sub>i g2) \<equiv>\<^sub>i f;g1 \<or>\<^sub>i f;g2" by (rule ChopOrEqv)
 from 2 3 show ?thesis by auto
qed

lemma NextImpDist:
 "\<turnstile> \<circle> (f \<supset>\<^sub>i g) \<supset>\<^sub>i \<circle> f \<supset>\<^sub>i \<circle> g"
proof -
 have  1: "\<turnstile> \<not>\<^sub>i (f \<supset>\<^sub>i g) \<equiv>\<^sub>i f \<and>\<^sub>i \<not>\<^sub>i g" by auto
 hence 2: "\<turnstile> skip;\<not>\<^sub>i (f \<supset>\<^sub>i g) \<equiv>\<^sub>i skip;(f \<and>\<^sub>i \<not>\<^sub>i g)" by (rule RightChopEqvChop)
 have  3: "\<turnstile> f \<supset>\<^sub>i g \<or>\<^sub>i (f \<and>\<^sub>i \<not>\<^sub>i g)" by auto
 hence 4: "\<turnstile> skip;f \<supset>\<^sub>i (skip;g) \<or>\<^sub>i (skip;(f \<and>\<^sub>i \<not>\<^sub>i g))" by (rule ChopOrImpRule)
 hence 5: "\<turnstile> \<not>\<^sub>i (skip;(f \<and>\<^sub>i \<not>\<^sub>i g)) \<supset>\<^sub>i (skip;f) \<supset>\<^sub>i (skip;g)" by auto
 have  6: "\<turnstile> \<not>\<^sub>i (skip;\<not>\<^sub>i(f \<supset>\<^sub>i g)) \<supset>\<^sub>i (skip;f) \<supset>\<^sub>i (skip;g)" by auto
 hence 7: "\<turnstile> \<not>\<^sub>i (\<circle>\<not>\<^sub>i(f \<supset>\<^sub>i g)) \<supset>\<^sub>i (\<circle> f) \<supset>\<^sub>i (\<circle> g)" by (simp add: next_d_def)
 have  8: "\<turnstile> \<circle>(f \<supset>\<^sub>i g) \<supset>\<^sub>i \<not>\<^sub>i (\<circle>\<not>\<^sub>i(f \<supset>\<^sub>i g))" by (rule NextImpNotNextNot)
 from 7 8 show ?thesis by auto
qed

lemma ChopImpDiamond:
 "\<turnstile> f;g  \<supset>\<^sub>i \<diamond> g"
proof -
 have  1: "\<turnstile> f \<supset>\<^sub>i true\<^sub>i" by auto
 hence 2: "\<turnstile> f;g \<supset>\<^sub>i true\<^sub>i;g" by (rule LeftChopImpChop)
 from 2 show ?thesis by auto
qed

lemma NowImpDiamond:
 "\<turnstile> f \<supset>\<^sub>i \<diamond> f"
proof -
 have  1: "\<turnstile> empty;f \<equiv>\<^sub>i f" by (rule EmptyChop)
 have  2: "\<turnstile> empty \<supset>\<^sub>i true\<^sub>i" by auto
 hence 3: "\<turnstile> empty;f \<supset>\<^sub>i true\<^sub>i;f" by (rule LeftChopImpChop)
 have  4: " \<turnstile> f \<supset>\<^sub>i true\<^sub>i;f" using "1" "3" by auto
 from 4 show ?thesis by auto
qed

lemma BoxElim:
 "\<turnstile> \<box> f \<supset>\<^sub>i f "
proof -
  have  1: "\<turnstile> \<not>\<^sub>i f \<supset>\<^sub>i \<diamond> \<not>\<^sub>i f" by (rule NowImpDiamond)
  hence 2: "\<turnstile> \<not>\<^sub>i (\<diamond> \<not>\<^sub>i f) \<supset>\<^sub>i f " by auto
  from 2 show ?thesis by (metis always_d_def)
qed


lemma NextDiamondImpDiamond:
 "\<turnstile> \<circle> (\<diamond> f) \<supset>\<^sub>i \<diamond> f"
proof -
 have  1: "\<turnstile> skip;(true\<^sub>i;f) \<equiv>\<^sub>i (skip;true\<^sub>i);f" by (rule ChopAssoc)
 hence 2: "\<turnstile> (skip;true\<^sub>i);f \<equiv>\<^sub>i skip;(true\<^sub>i;f)" by auto 
 hence 3: "\<turnstile> (skip;true\<^sub>i);f \<equiv>\<^sub>i \<circle>(\<diamond>f)" by (simp add: next_d_def)
 have  4: "\<turnstile> (skip;true\<^sub>i);f \<supset>\<^sub>i \<diamond> f" by (rule ChopImpDiamond)
 from 3 4 show ?thesis by auto
qed

lemma BoxImpNowAndWeakNext:
 " \<turnstile> \<box> f \<supset>\<^sub>i (f \<and>\<^sub>i wnext ( \<box> f) )"
proof -
  have  1: "\<turnstile> \<not>\<^sub>i f \<supset>\<^sub>i \<diamond> \<not>\<^sub>i f " by (rule NowImpDiamond)
  hence 2: "\<turnstile> \<not>\<^sub>i (\<diamond> \<not>\<^sub>i f) \<supset>\<^sub>i f" by auto
  hence 3: "\<turnstile> \<box> f \<supset>\<^sub>i f" by (metis always_d_def)
  have  4: "\<turnstile> \<circle> ( \<diamond> \<not>\<^sub>i f) \<supset>\<^sub>i \<diamond> ( \<not>\<^sub>i f ) " by (rule NextDiamondImpDiamond)
  have  5: "\<turnstile> \<not>\<^sub>i \<not>\<^sub>i (\<diamond> \<not>\<^sub>i f) \<supset>\<^sub>i \<diamond>( \<not>\<^sub>i f ) " by auto
  hence 6: "\<turnstile> \<circle> ( \<not>\<^sub>i \<not>\<^sub>i (\<diamond> \<not>\<^sub>i f) )  \<supset>\<^sub>i \<circle> (\<diamond>( \<not>\<^sub>i f )) " by (rule NextImpNext)
  have  7: "\<turnstile> \<circle> ( \<not>\<^sub>i \<not>\<^sub>i (\<diamond> \<not>\<^sub>i f) )  \<supset>\<^sub>i \<diamond> ( \<not>\<^sub>i f )" using "4" "6" by auto  
  hence 8: "\<turnstile> \<circle> ( \<not>\<^sub>i( \<box> f)) \<supset>\<^sub>i \<diamond> ( \<not>\<^sub>i f ) " by (simp add: always_d_def)
  hence 9: "\<turnstile> \<not>\<^sub>i (\<diamond> ( \<not>\<^sub>i f ))  \<supset>\<^sub>i \<not>\<^sub>i ( \<circle> ( \<not>\<^sub>i( \<box> f)))" by auto
  hence 10:"\<turnstile> \<box>f \<supset>\<^sub>i wnext ( \<box> f )" by (simp add: always_d_def wnext_d_def) 
  from 3 10 show ?thesis using itl_prop(32) by blast
qed

lemma BoxImpBoxRule:
 assumes "\<turnstile> f \<supset>\<^sub>i g"
 shows   "\<turnstile> \<box> f\<supset>\<^sub>i \<box> g" 
proof - 
 have  1: "\<turnstile> f \<supset>\<^sub>i g" using assms by auto
 hence 2: "\<turnstile> \<not>\<^sub>i g \<supset>\<^sub>i  \<not>\<^sub>i f" by auto
 hence 3: "\<turnstile> \<box>(\<not>\<^sub>i g \<supset>\<^sub>i  \<not>\<^sub>i f)" by (rule BoxGen)
 have  4: "\<turnstile>  \<box>(\<not>\<^sub>i g \<supset>\<^sub>i  \<not>\<^sub>i f) \<supset>\<^sub>i (true\<^sub>i;\<not>\<^sub>ig) \<supset>\<^sub>i (true\<^sub>i;\<not>\<^sub>if)" by (rule BoxChopImpChop)
 have  5: "\<turnstile> (true\<^sub>i;\<not>\<^sub>ig) \<supset>\<^sub>i (true\<^sub>i;\<not>\<^sub>if)" using "3" "4" MP by auto
 hence 6: "\<turnstile> \<diamond> \<not>\<^sub>ig  \<supset>\<^sub>i \<diamond>\<not>\<^sub>if" by (simp add: sometimes_d_def)
 hence 7: "\<turnstile>  \<not>\<^sub>i (\<diamond>\<not>\<^sub>if ) \<supset>\<^sub>i \<not>\<^sub>i( \<diamond> \<not>\<^sub>ig)" by auto
 from 7 show ?thesis by (simp add: always_d_def)
qed

lemma BoxImpDist:
 " \<turnstile>  \<box>(f \<supset>\<^sub>i g) \<supset>\<^sub>i \<box> f \<supset>\<^sub>i \<box> g"
proof -
 have  1: "\<turnstile> (f \<supset>\<^sub>i g) \<supset>\<^sub>i (\<not>\<^sub>i  g \<supset>\<^sub>i \<not>\<^sub>i  f) " by auto
 hence 2: "\<turnstile> \<box>(f \<supset>\<^sub>i g) \<supset>\<^sub>i \<box>(\<not>\<^sub>i  g \<supset>\<^sub>i \<not>\<^sub>i  f) " by (rule BoxImpBoxRule)
 have  3: "\<turnstile> \<box>(\<not>\<^sub>i  g \<supset>\<^sub>i \<not>\<^sub>i  f) \<supset>\<^sub>i  (true\<^sub>i; \<not>\<^sub>i  g) \<supset>\<^sub>i (true\<^sub>i; \<not>\<^sub>i  f)" by (rule BoxChopImpChop)
 have  4: "\<turnstile> \<box>(f \<supset>\<^sub>i g) \<supset>\<^sub>i   (true\<^sub>i; \<not>\<^sub>i  g) \<supset>\<^sub>i (true\<^sub>i; \<not>\<^sub>i  f)" using "2" "3" prop02 by blast   
 hence 5: "\<turnstile> \<box>(f \<supset>\<^sub>i g) \<supset>\<^sub>i   \<diamond>\<not>\<^sub>i  g \<supset>\<^sub>i \<diamond>\<not>\<^sub>i  f " by (simp add: sometimes_d_def)
 hence 6: "\<turnstile> \<box>(f \<supset>\<^sub>i g) \<supset>\<^sub>i   \<not>\<^sub>i( \<diamond>\<not>\<^sub>i  f) \<supset>\<^sub>i \<not>\<^sub>i( \<diamond>\<not>\<^sub>i  g) " by auto
 from 6 show ?thesis by (simp add: always_d_def)
qed

lemma DiamondEmpty:
 " \<turnstile>   \<diamond> empty "
proof -
 have 1: "\<turnstile> true\<^sub>i " by auto
 have 2: "\<turnstile> true\<^sub>i;  empty  \<equiv>\<^sub>i true\<^sub>i " by (rule ChopEmpty)
 have 3: "\<turnstile> true\<^sub>i;  empty " using "1" "2" by auto
 from 3 show ?thesis by (simp add: sometimes_d_def)
qed

lemma NextEqvNext:
 assumes "\<turnstile>   f\<equiv>\<^sub>i g"
 shows   " \<turnstile> \<circle> f \<equiv>\<^sub>i \<circle> g"
proof -
 have  1: "\<turnstile> f\<equiv>\<^sub>i g" using assms by auto
 hence 2: "\<turnstile> skip;f \<equiv>\<^sub>i skip;g"  by (rule RightChopEqvChop) 
 from 1 show ?thesis by (simp add: next_d_def)
qed
 
lemma NextAndNextImpNextRule:
 assumes "\<turnstile> (f \<and>\<^sub>i g) \<supset>\<^sub>i h " 
 shows   "\<turnstile> (\<circle> f \<and>\<^sub>i \<circle> g) \<supset>\<^sub>i \<circle> h "
using assms by auto

lemma NextAndNextEqvNextRule:
 assumes "\<turnstile> f \<and>\<^sub>i g \<equiv>\<^sub>i h "
 shows   "\<turnstile> \<circle> f \<and>\<^sub>i \<circle> g \<equiv>\<^sub>i \<circle> h "
using assms by auto

lemma WeakNextEqvWeakNext:
 assumes "\<turnstile> f\<equiv>\<^sub>i g "
 shows   "\<turnstile> wnext  f \<equiv>\<^sub>i wnext  g "
using assms by auto

lemma DiamondImpDiamond:
 assumes "\<turnstile> f\<supset>\<^sub>i g "
 shows   "\<turnstile> \<diamond> f \<supset>\<^sub>i \<diamond> g "
using assms by auto

lemma DiamondEqvDiamond:
 assumes "\<turnstile> f\<equiv>\<^sub>i g " 
 shows   "\<turnstile> \<diamond> f \<equiv>\<^sub>i \<diamond> g "
using assms by auto

lemma BoxEqvBox:
 assumes "\<turnstile> f\<equiv>\<^sub>i g "
 shows   "\<turnstile> \<box> f \<equiv>\<^sub>i \<box> g "
using assms by auto

lemma BoxAndBoxImpBoxRule:
 assumes "\<turnstile> f \<and>\<^sub>i g \<supset>\<^sub>i h "
 shows   "\<turnstile> \<box> f \<and>\<^sub>i \<box> g \<supset>\<^sub>i \<box> h "
using assms by auto

lemma BoxAndBoxEqvBoxRule:
 assumes "\<turnstile> f \<and>\<^sub>i g \<equiv>\<^sub>i h "
 shows   "\<turnstile> \<box> f \<and>\<^sub>i \<box> g \<equiv>\<^sub>i \<box> h " 
using assms by auto

lemma ImpBoxRule:
 assumes "\<turnstile>   f \<supset>\<^sub>i g "
 shows   "\<turnstile> \<box> f \<supset>\<^sub>i \<box> g "
using assms by auto

lemma BoxIntro:
 assumes "\<turnstile>  f\<supset>\<^sub>i g" 
         "\<turnstile>  more  \<and>\<^sub>i f \<supset>\<^sub>i\<circle> f "
 shows   "\<turnstile> f\<supset>\<^sub>i\<box> g "
proof -
 have   1: "\<turnstile>  more  \<and>\<^sub>i f \<supset>\<^sub>i\<circle> f " using assms by auto
 hence  2: "\<turnstile> f\<supset>\<^sub>i (empty \<or>\<^sub>i \<circle>f)" by auto
 hence  3: "\<turnstile> f \<supset>\<^sub>i wnext f" by auto
 hence  4: "\<turnstile> \<box>(f \<supset>\<^sub>i wnext f)" by (rule BoxGen)
 have   5: "\<turnstile> (\<box> (f \<supset>\<^sub>i wnext f)) \<and>\<^sub>i f \<supset>\<^sub>i \<box> f  " by (rule BoxInduct)
 hence  6: "\<turnstile> (\<box> (f \<supset>\<^sub>i wnext f)) \<supset>\<^sub>i (f \<supset>\<^sub>i \<box>f)"  using prop36 by blast 
 have   7: "\<turnstile> f \<supset>\<^sub>i \<box>f " using "4" "6" MP by blast
 have   8: "\<turnstile> \<box>f \<supset>\<^sub>i f" by (rule BoxElim)
 have   9: "\<turnstile> f \<equiv>\<^sub>i \<box> f" using "7" "8" itl_prop(31) by blast
 have  10: "\<turnstile> f \<supset>\<^sub>i g" using assms by auto
 hence 11: "\<turnstile> \<box>f \<supset>\<^sub>i \<box> g" by (rule ImpBoxRule)
 from 7 9 11 show ?thesis  using prop02 by blast 
qed

lemma NextLoop:
 assumes "\<turnstile> f\<supset>\<^sub>i \<circle> f" 
 shows   "\<turnstile> \<not>\<^sub>i  f "
proof -
 have   1: "\<turnstile> f\<supset>\<^sub>i \<circle> f " using assms by auto
 hence  2: "\<turnstile> f \<supset>\<^sub>i (more \<and>\<^sub>i wnext f)" by auto
 hence  3: "\<turnstile> f \<supset>\<^sub>i wnext f " by auto
 hence  4: "\<turnstile> \<box>(f \<supset>\<^sub>i wnext f)" by (rule BoxGen)
 have   5: "\<turnstile> \<box> (f \<supset>\<^sub>i wnext f) \<and>\<^sub>i f \<supset>\<^sub>i \<box> f  " by (rule BoxInduct)
 hence  6: "\<turnstile> \<box> (f \<supset>\<^sub>i wnext f) \<supset>\<^sub>i (f \<supset>\<^sub>i \<box>f)" using prop36 by blast 
 have   7: "\<turnstile> f \<supset>\<^sub>i \<box>f " using "4" "6" MP by blast
 have   8: "\<turnstile> \<box>f \<supset>\<^sub>i f" by (rule BoxElim)
 have   9: "\<turnstile> f \<equiv>\<^sub>i \<box> f" using "7" "8" itl_prop(31) by blast
 have  10: "\<turnstile> f \<supset>\<^sub>i more" using "2" by auto
 hence 11: "\<turnstile> \<box> f \<supset>\<^sub>i \<box> more" by (rule ImpBoxRule)
 have  12: "\<turnstile> \<not>\<^sub>i(\<box> more) " by auto
 from 7 9 11 12 show ?thesis by (metis not_d_def prop02) 
qed

lemma WnextEqvEmptyOrNext:
 "\<turnstile> wnext f \<equiv>\<^sub>i empty \<or>\<^sub>i \<circle> f"
by auto

lemma NotEmptyAndNext:
 "\<turnstile> \<not>\<^sub>i(empty \<and>\<^sub>i \<circle> f)"
by auto 

lemma BoxEqvAndWnextBox:
 "\<turnstile> \<box> f \<equiv>\<^sub>i f \<and>\<^sub>i wnext ( \<box> f)"
proof -
 have 1: "\<turnstile> \<box> f \<supset>\<^sub>i f \<and>\<^sub>i wnext ( \<box> f)" 
      using BoxImpNowAndWeakNext by blast
 have 2: "\<turnstile> f \<and>\<^sub>i wnext ( \<box> f) \<supset>\<^sub>i f" 
      by simp
 have 3: "\<turnstile> more \<and>\<^sub>i (f \<and>\<^sub>i wnext ( \<box> f) ) \<supset>\<^sub>i \<circle> (f \<and>\<^sub>i wnext ( \<box> f) )" 
      by (metis "1" NextImpNext WnextEqvEmptyOrNext empty_d_def prop10 prop13 prop14)
 have 4: "\<turnstile> f \<and>\<^sub>i wnext ( \<box> f) \<supset>\<^sub>i \<box> f" 
      using "2" "3" BoxIntro by blast
 from 1 4 show ?thesis using itl_prop(31) by blast
qed

lemma BoxEqvAndEmptyOrNextBox:
 "\<turnstile> \<box>f \<equiv>\<^sub>i f \<and>\<^sub>i (empty  \<or>\<^sub>i  \<circle>(\<box> f))"
using BoxEqvAndWnextBox WnextEqvEmptyOrNext using prop03 prop05 by blast

lemma BoxEqvBoxBox:
 "\<turnstile> \<box>f \<equiv>\<^sub>i \<box> (\<box> f)"
by auto

lemma BoxBoxImpBox:
 "\<turnstile> \<box>(\<box>h) \<supset>\<^sub>i \<box> h"
using BoxEqvBoxBox itl_prop(31) by blast

lemma BoxImpBoxBox:
 "\<turnstile> \<box> h  \<supset>\<^sub>i  \<box>(\<box>h)"
by simp

lemma DiamondIntro:
 assumes "\<turnstile> (f \<and>\<^sub>i \<not>\<^sub>i  g) \<supset>\<^sub>i \<circle> f "
 shows   "\<turnstile> f\<supset>\<^sub>i\<diamond> g "
proof -
 have   1: "\<turnstile> f \<and>\<^sub>i \<not>\<^sub>i  g \<supset>\<^sub>i \<circle> f " 
        using assms by auto
 hence  2: "\<turnstile> f \<and>\<^sub>i \<not>\<^sub>i  g \<and>\<^sub>i (\<box> \<not>\<^sub>i g)\<supset>\<^sub>i (\<circle> f) \<and>\<^sub>i (\<box> \<not>\<^sub>i g)" 
        by auto
 have   3: "\<turnstile> (\<box> \<not>\<^sub>i g) \<supset>\<^sub>i \<not>\<^sub>i  g" 
        by (rule BoxElim)
 hence  4: "\<turnstile> \<box> \<not>\<^sub>i g \<equiv>\<^sub>i (\<box> \<not>\<^sub>i g) \<and>\<^sub>i \<not>\<^sub>i  g" 
        using BoxImpBoxBox BoxBoxImpBox itl_prop(31) itl_prop(32) prop02 prop26 prop29 by blast 
 have   5: "\<turnstile> f \<and>\<^sub>i (\<box> \<not>\<^sub>i g)\<supset>\<^sub>i \<circle> f \<and>\<^sub>i \<box> \<not>\<^sub>i g " 
        using "2" "4"  by auto
 have   6: "\<turnstile> \<box> \<not>\<^sub>i g \<equiv>\<^sub>i (\<not>\<^sub>i g) \<and>\<^sub>i wnext(\<box> \<not>\<^sub>i g)  "  
        using BoxEqvAndWnextBox by blast 
 have   7: "\<turnstile> \<circle> f \<and>\<^sub>i \<box> \<not>\<^sub>i g \<supset>\<^sub>i  \<circle> f \<and>\<^sub>i wnext(\<box> \<not>\<^sub>i g) " 
        using "6" by auto
 have   8: "\<turnstile> f \<and>\<^sub>i (\<box> \<not>\<^sub>i g)\<supset>\<^sub>i \<circle> f \<and>\<^sub>i wnext(\<box> \<not>\<^sub>i g) " 
        using "5" "7" by auto
 hence  9: "\<turnstile> f \<and>\<^sub>i (\<box> \<not>\<^sub>i g)\<supset>\<^sub>i more \<and>\<^sub>i wnext f \<and>\<^sub>i wnext(\<box> \<not>\<^sub>i g)" 
        by auto
 hence 10: "\<turnstile> f \<and>\<^sub>i (\<box> \<not>\<^sub>i g)\<supset>\<^sub>iwnext f \<and>\<^sub>i wnext(\<box> \<not>\<^sub>i g) " 
        by auto
 hence 11: "\<turnstile> f \<and>\<^sub>i (\<box> \<not>\<^sub>i g)\<supset>\<^sub>i wnext (f \<and>\<^sub>i \<box> \<not>\<^sub>i g) " 
        by auto
 hence 12: "\<turnstile> \<box>(f \<and>\<^sub>i (\<box> \<not>\<^sub>i g)\<supset>\<^sub>i wnext (f \<and>\<^sub>i \<box> \<not>\<^sub>i g)) " 
        by (rule BoxGen)
 have  13: "\<turnstile> \<box>(f \<and>\<^sub>i (\<box> \<not>\<^sub>i g)\<supset>\<^sub>i wnext (f \<and>\<^sub>i \<box> \<not>\<^sub>i g)) \<and>\<^sub>i f \<and>\<^sub>i (\<box> \<not>\<^sub>i g) \<supset>\<^sub>i \<box>(f \<and>\<^sub>i (\<box> \<not>\<^sub>i g)) " 
        by (rule BoxInduct)
 hence 14: "\<turnstile> \<box>(f \<and>\<^sub>i (\<box> \<not>\<^sub>i g)\<supset>\<^sub>i wnext (f \<and>\<^sub>i \<box> \<not>\<^sub>i g)) \<supset>\<^sub>i ((f \<and>\<^sub>i (\<box> \<not>\<^sub>i g)) \<supset>\<^sub>i \<box>(f \<and>\<^sub>i (\<box> \<not>\<^sub>i g))) " 
        using prop36 by blast 
 have  15: "\<turnstile> ((f \<and>\<^sub>i (\<box> \<not>\<^sub>i g)) \<supset>\<^sub>i \<box>(f \<and>\<^sub>i (\<box> \<not>\<^sub>i g))) " 
        using "12" "14" MP by blast
 have  16: "\<turnstile> \<box>(f \<and>\<^sub>i (\<box> \<not>\<^sub>i g)) \<supset>\<^sub>i (f \<and>\<^sub>i (\<box> \<not>\<^sub>i g)) " 
        by (rule BoxElim)
 have  17: "\<turnstile> \<box>(f \<and>\<^sub>i (\<box> \<not>\<^sub>i g)) \<equiv>\<^sub>i (f \<and>\<^sub>i (\<box> \<not>\<^sub>i g))" 
        using "16" "15" itl_prop(31) by blast
 have  18: "\<turnstile> (f \<and>\<^sub>i (\<box> \<not>\<^sub>i g)) \<supset>\<^sub>i more" 
        using "9" by auto
 hence 19: "\<turnstile> \<box>(f \<and>\<^sub>i (\<box> \<not>\<^sub>i g)) \<supset>\<^sub>i \<box> more" 
        by (rule ImpBoxRule)
 have  20: "\<turnstile> \<not>\<^sub>i(\<box> more) " 
        by auto
 have  21: "\<turnstile> \<not>\<^sub>i(f \<and>\<^sub>i (\<box> \<not>\<^sub>i g))" 
        using "17" "19" "20" by auto
 hence 22: "\<turnstile> \<not>\<^sub>i f \<or>\<^sub>i \<not>\<^sub>i (\<box> \<not>\<^sub>i g) " 
        by auto
 have  23: "\<turnstile> \<not>\<^sub>i (\<box> \<not>\<^sub>i g) \<equiv>\<^sub>i \<diamond> g" 
        by auto
 from 22 23 show ?thesis by auto
qed
 
(* changed \<circle>f to \<circle>(f \<and>\<^sub>i \<not>\<^sub>i g) *)
lemma DiamondIntroB:
 assumes " \<turnstile> (f \<and>\<^sub>i \<not>\<^sub>i  g) \<supset>\<^sub>i \<circle> (f \<and>\<^sub>i \<not>\<^sub>i g) "
 shows   "\<turnstile> f\<supset>\<^sub>i\<diamond> g "
proof -
 have  1: "\<turnstile> (f \<and>\<^sub>i \<not>\<^sub>i  g) \<supset>\<^sub>i \<circle> (f \<and>\<^sub>i \<not>\<^sub>ig) " using assms by auto
 hence 2: "\<turnstile> \<not>\<^sub>i(f \<and>\<^sub>i \<not>\<^sub>i g) " by (rule NextLoop)
 hence 3: "\<turnstile> f \<supset>\<^sub>i g" by auto
 have  4: "\<turnstile> g \<supset>\<^sub>i \<diamond> g" by (rule NowImpDiamond)
 from 3 4 show ?thesis by auto
qed

lemma NextContra :
 assumes "\<turnstile> (f\<and>\<^sub>i \<not>\<^sub>i  g)\<supset>\<^sub>i (\<circle> f \<and>\<^sub>i \<not>\<^sub>i( \<circle> g)) "
 shows   "\<turnstile> f \<supset>\<^sub>i g "
proof -
 have  1: "\<turnstile> (f\<and>\<^sub>i \<not>\<^sub>i  g)\<supset>\<^sub>i (\<circle> f \<and>\<^sub>i \<not>\<^sub>i( \<circle> g)) " using assms by auto
 hence 2: "\<turnstile> \<not>\<^sub>i( f \<supset>\<^sub>i g) \<supset>\<^sub>i \<circle> ( \<not>\<^sub>i(f \<supset>\<^sub>i g)) " by auto
 hence 3: "\<turnstile> \<not>\<^sub>i \<not>\<^sub>i( f \<supset>\<^sub>i g) " by (rule NextLoop)
 from 3 show ?thesis by auto
qed

lemma DiamondDiamondEqvDiamond:
 " \<turnstile> \<diamond>(\<diamond> f) \<equiv>\<^sub>i \<diamond> f"
proof -
 have  1: "\<turnstile> true\<^sub>i;true\<^sub>i \<equiv>\<^sub>i true\<^sub>i" by auto
 hence 2: "\<turnstile> (true\<^sub>i;true\<^sub>i);f \<equiv>\<^sub>i true\<^sub>i;f " using LeftChopEqvChop by blast
 have  3: "\<turnstile> (true\<^sub>i;true\<^sub>i);f \<equiv>\<^sub>i true\<^sub>i;(true\<^sub>i;f) " using ChopAssoc itl_prop(30) by blast
 from 2 3 show ?thesis by auto
qed

lemma WeakNextDiamondInduct:
 assumes "\<turnstile> wnext  (\<diamond> f) \<supset>\<^sub>i f "
 shows   "\<turnstile> f "
proof -
 have  1: "\<turnstile> wnext  (\<diamond> f) \<supset>\<^sub>i f " using assms by blast
 hence 2: "\<turnstile> \<not>\<^sub>i f \<supset>\<^sub>i \<not>\<^sub>i( wnext  (\<diamond> f))" using prop27 by blast  
 hence 3: "\<turnstile> \<not>\<^sub>i f \<supset>\<^sub>i \<circle>( \<not>\<^sub>i (\<diamond> f)) " by auto
 have  4: "\<turnstile> f \<supset>\<^sub>i \<diamond> f" by (rule NowImpDiamond)
 hence 5: "\<turnstile> \<not>\<^sub>i( \<diamond> f) \<supset>\<^sub>i \<not>\<^sub>i f  " by auto
 have  6: "\<turnstile> \<not>\<^sub>if \<supset>\<^sub>i \<circle>( \<not>\<^sub>if)" using "3" "5" using NextImpNext prop02 by blast  
 hence 7: "\<turnstile> \<not>\<^sub>i\<not>\<^sub>i f" by (rule NextLoop)
 from 7 show ?thesis by auto
qed

lemma EmptyNextInducta:
 assumes "\<turnstile> empty  \<supset>\<^sub>i f " 
         "\<turnstile> \<circle> f \<supset>\<^sub>i f " 
 shows "\<turnstile> f "
proof -
 have  1: "\<turnstile> empty  \<supset>\<^sub>i f " using assms by auto
 have  2: "\<turnstile> \<circle> f \<supset>\<^sub>i f " using assms by blast
 have  3: "\<turnstile> (empty \<or>\<^sub>i \<circle> f) \<supset>\<^sub>i f  " using "1" "2"  prop17 by blast 
 have  4: "\<turnstile> wnext f \<equiv>\<^sub>i (empty \<or>\<^sub>i \<circle> f)" by  (rule  WnextEqvEmptyOrNext)
 hence 5: "\<turnstile> wnext f \<supset>\<^sub>i f" using "3" using itl_prop(31) prop02 by blast 
 hence 6: "\<turnstile> \<not>\<^sub>if \<supset>\<^sub>i \<not>\<^sub>i (wnext f)" by auto
 hence 7: "\<turnstile> \<not>\<^sub>if \<supset>\<^sub>i \<circle>(\<not>\<^sub>i f) " by auto
 hence 8: "\<turnstile> \<not>\<^sub>i \<not>\<^sub>i f " by (rule NextLoop)
 from 8 show ?thesis by auto
qed

lemma EmptyNextInductb:
 assumes "\<turnstile> empty  \<and>\<^sub>i f \<supset>\<^sub>i g " 
         "\<turnstile> \<circle>(f\<supset>\<^sub>i g) \<and>\<^sub>i f \<supset>\<^sub>i g "
 shows   "\<turnstile> f\<supset>\<^sub>i g "
proof -
 have  1: "\<turnstile> empty  \<and>\<^sub>i f \<supset>\<^sub>i g " using assms by auto
 have  2: "\<turnstile> \<circle>(f\<supset>\<^sub>i g) \<and>\<^sub>i f \<supset>\<^sub>i g " using assms by blast
 have  3: "\<turnstile> (empty \<or>\<^sub>i \<circle>(f\<supset>\<^sub>i g)) \<and>\<^sub>i f \<supset>\<^sub>i g " using "1" "2" prop33 by blast   
 hence 4: "\<turnstile> wnext (f\<supset>\<^sub>i g)  \<and>\<^sub>i f \<supset>\<^sub>i g "  using prop36 by auto 
 hence 5: "\<turnstile> wnext (f\<supset>\<^sub>i g) \<supset>\<^sub>i ( f \<supset>\<^sub>i g) "  using prop36 by blast 
 hence 6: "\<turnstile> \<not>\<^sub>i ( f \<supset>\<^sub>i g) \<supset>\<^sub>i \<not>\<^sub>i ( wnext (f\<supset>\<^sub>i g)) " using prop27 by blast 
 hence 7: "\<turnstile> \<not>\<^sub>i ( f \<supset>\<^sub>i g) \<supset>\<^sub>i \<circle> ( \<not>\<^sub>i(f\<supset>\<^sub>i g)) " by simp
 hence 8: "\<turnstile> \<not>\<^sub>i \<not>\<^sub>i ( f \<supset>\<^sub>i g)" by (rule NextLoop)
 from 8 show ?thesis by auto
qed


lemma FinImpFin:
 assumes "\<turnstile> f \<supset>\<^sub>i g "
 shows   "\<turnstile>  fin  f \<supset>\<^sub>i  fin  g "
using ImpBoxRule assms by auto

lemma FinEqvFin:
 assumes "\<turnstile> f \<equiv>\<^sub>i g "
 shows   "\<turnstile>  fin  f \<equiv>\<^sub>i  fin  g "
using FinImpFin assms itl_prop(31) by blast

lemma FinAndFinImpFinRule:
 assumes "\<turnstile> f \<and>\<^sub>i g \<supset>\<^sub>i h "
 shows   "\<turnstile>  fin  f \<and>\<^sub>i  fin  g \<supset>\<^sub>i  fin  h "
proof -
  have "\<turnstile> f \<and>\<^sub>i g \<supset>\<^sub>i h " using assms by auto 
  then show ?thesis by simp  
qed

lemma FinAndFinEqvFinRule:
 assumes "\<turnstile> f \<and>\<^sub>i g \<equiv>\<^sub>i h "
 shows   "\<turnstile>  fin  f \<and>\<^sub>i  fin  g \<equiv>\<^sub>i  fin  h "
by (meson FinAndFinImpFinRule FinImpFin assms itl_prop(31) itl_prop(32))

(* dodgy
lemma HaltImpHalt:
 assumes " \<turnstile>   f \<supset>\<^sub>i g "
 shows "\<turnstile>  halt  f \<supset>\<^sub>i  halt  g "
proof -
 have 1: "\<turnstile>   f \<supset>\<^sub>i g " using assms by auto
 hence 2: " \<turnstile> (empty\<equiv>\<^sub>if) \<supset>\<^sub>i (empty\<equiv>\<^sub>ig)" 
*)


lemma HaltEqvHalt:
 assumes "\<turnstile> f \<equiv>\<^sub>i g "
 shows   "\<turnstile>  halt  f \<equiv>\<^sub>i  halt  g "
proof -
 have  1: "\<turnstile> f \<equiv>\<^sub>i g" using assms by auto
 hence 2: "\<turnstile> (empty \<equiv>\<^sub>i f)  \<equiv>\<^sub>i (empty \<equiv>\<^sub>i g) " by auto
 hence 3: "\<turnstile> \<box>(empty \<equiv>\<^sub>i f)  \<equiv>\<^sub>i \<box> (empty \<equiv>\<^sub>i g)" by (rule BoxEqvBox)
 from 3 show ?thesis by (simp add: halt_d_def)
qed

lemma BiImpDiImpDi:
  "\<turnstile> bi (f \<supset>\<^sub>i g) \<supset>\<^sub>i  di  f \<supset>\<^sub>i  di  g "
proof -
 have 1: "\<turnstile> bi (f\<supset>\<^sub>i g)  \<supset>\<^sub>i (f; true\<^sub>i) \<supset>\<^sub>i (g; true\<^sub>i) "  by (rule BiChopImpChop)
 from 1 show ?thesis   by (simp add: di_d_def)
qed

lemma DiImpDi:
 assumes "\<turnstile> f\<supset>\<^sub>i g "
 shows   "\<turnstile>  di  f \<supset>\<^sub>i  di  g "
proof -
 have  1: "\<turnstile> f\<supset>\<^sub>i g " using assms by auto
 hence 2: "\<turnstile> f; true\<^sub>i \<supset>\<^sub>i g; true\<^sub>i " by (rule LeftChopImpChop)
 from 2 show ?thesis by (simp add: di_d_def)
qed

lemma BiImpBiRule:
 assumes "\<turnstile> f\<supset>\<^sub>i g"
 shows   "\<turnstile> bi  f \<supset>\<^sub>i bi  g "
proof -
 have  1: "\<turnstile> f\<supset>\<^sub>i g" using assms by auto
 hence 2: "\<turnstile> \<not>\<^sub>i  g \<supset>\<^sub>i \<not>\<^sub>i  f " by auto
 hence 3: "\<turnstile>  di \<not>\<^sub>i  g \<supset>\<^sub>i  di \<not>\<^sub>i  f " by (rule DiImpDi)
 hence 4: "\<turnstile> \<not>\<^sub>i ( di \<not>\<^sub>i  f) \<supset>\<^sub>i \<not>\<^sub>i ( di \<not>\<^sub>i  g) " by auto
 from 4 show ?thesis by (simp add: bi_d_def)
qed

lemma DiEqvDi:
 assumes "\<turnstile> f\<equiv>\<^sub>i g "
 shows   "\<turnstile>  di  f \<equiv>\<^sub>i  di  g "
proof -
 have  1: "\<turnstile> f \<equiv>\<^sub>i g " using assms by auto
 hence 2: "\<turnstile> f; true\<^sub>i \<equiv>\<^sub>i g; true\<^sub>i " by (rule LeftChopEqvChop)
 from 2 show ?thesis by (simp add: di_d_def)
qed

lemma BiEqvBi:
 assumes "\<turnstile> f \<equiv>\<^sub>i g "
 shows   "\<turnstile>  bi f \<equiv>\<^sub>i  bi  g "
proof -
 have  1: "\<turnstile> f \<equiv>\<^sub>i g " using assms by auto
 hence 2: "\<turnstile> \<not>\<^sub>i  f \<equiv>\<^sub>i \<not>\<^sub>i  g " by auto
 hence 3: "\<turnstile>  di \<not>\<^sub>i  f \<equiv>\<^sub>i  di \<not>\<^sub>i  g " by (rule DiEqvDi)
 hence 4: "\<turnstile> \<not>\<^sub>i  (di \<not>\<^sub>i  f) \<equiv>\<^sub>i \<not>\<^sub>i ( di \<not>\<^sub>i  g) " by auto
 from 4 show ?thesis by (simp add: bi_d_def)
qed

lemma LeftChopChopImpChopRule:
 assumes "\<turnstile> (f; g) \<supset>\<^sub>i g "
 shows   "\<turnstile> (f; g); h \<supset>\<^sub>i (g; h) "
proof -
 have  1: "\<turnstile> (f; g) \<supset>\<^sub>i g " using assms by blast
 hence 2: "\<turnstile> (f; g); h \<supset>\<^sub>i g; h " by (rule LeftChopImpChop)
 have  3: "\<turnstile> f; (g; h) \<equiv>\<^sub>i (f; g); h   " by (rule ChopAssoc) 
 from 2 3 show ?thesis by auto
qed

lemma AndChopCommute :
 "\<turnstile> (f \<and>\<^sub>i f1); g \<equiv>\<^sub>i (f1 \<and>\<^sub>i f); g "
proof -
 have 1: "\<turnstile> f \<and>\<^sub>i f1 \<equiv>\<^sub>i f1 \<and>\<^sub>i f  " by auto
 from 1 show ?thesis by (rule LeftChopEqvChop)
qed

lemma BiAndChopImport:
 "\<turnstile>  bi  f \<and>\<^sub>i (f1; g) \<supset>\<^sub>i (f \<and>\<^sub>i f1); g "
proof -
 have  1: "\<turnstile> f \<supset>\<^sub>i (f1 \<supset>\<^sub>i f\<and>\<^sub>i f1) " by auto
 hence 2: "\<turnstile> bi  f \<supset>\<^sub>i bi  (f1 \<supset>\<^sub>i f\<and>\<^sub>i f1) " by (rule BiImpBiRule)
 have  3: "\<turnstile> bi  (f1 \<supset>\<^sub>i (f \<and>\<^sub>i f1)) \<supset>\<^sub>i   f1; g \<supset>\<^sub>i (f \<and>\<^sub>i f1); g " by (rule BiChopImpChop)
 from 1 3 show ?thesis using MP by auto
qed

lemma StateAndChopImport:
 "\<turnstile> (init w) \<and>\<^sub>i (f; g) \<supset>\<^sub>i ((init w) \<and>\<^sub>i f); g "
proof -
 have  1: "\<turnstile> (init w)\<supset>\<^sub>i bi  (init w) " by (rule StateImpBi)
 hence 2: "\<turnstile> (init w) \<and>\<^sub>i (f; g) \<supset>\<^sub>i bi (init w) \<and>\<^sub>i (f; g) " by auto
 have  3: "\<turnstile> bi  (init w) \<and>\<^sub>i (f; g) \<supset>\<^sub>i ((init w) \<and>\<^sub>i f); g "  by (rule BiAndChopImport)
 from 2 3 show ?thesis using MP by  auto
qed

subsection \<open>Further Properties Di and Bi\<close>

lemma ImpDi:
 "\<turnstile> f \<supset>\<^sub>i  di  f "
proof -
 have  1: "\<turnstile> f;  empty  \<equiv>\<^sub>i f " by (rule ChopEmpty)
 have  2: "\<turnstile> empty  \<supset>\<^sub>i true\<^sub>i " by auto
 hence 3: "\<turnstile> f;  empty  \<supset>\<^sub>i f; true\<^sub>i " by (rule RightChopImpChop)
 have 4 : "\<turnstile> f \<supset>\<^sub>i f; true\<^sub>i " by auto
 from 4 show ?thesis by (simp add: di_d_def)
qed

lemma DiState:
 "\<turnstile> di (init w) \<equiv>\<^sub>i (init w) "
proof -
 have  0: "\<turnstile> (init \<not>\<^sub>iw) \<supset>\<^sub>i bi   (init \<not>\<^sub>iw) " using StateImpBi by fastforce
 hence 1: "\<turnstile> \<not>\<^sub>i(init w) \<supset>\<^sub>i bi  \<not>\<^sub>i (init w)" using Initprop by auto
 hence 2: "\<turnstile> \<not>\<^sub>i  (init w)\<supset>\<^sub>i \<not>\<^sub>i ( di \<not>\<^sub>i \<not>\<^sub>i  (init w)) " by (simp add: bi_d_def)
 have  3: "\<turnstile> (\<not>\<^sub>i  (init w) \<supset>\<^sub>i\<not>\<^sub>i  (di \<not>\<^sub>i \<not>\<^sub>i  (init w))) \<supset>\<^sub>i ( di \<not>\<^sub>i \<not>\<^sub>i  (init w) \<supset>\<^sub>i (init w)) " by auto
 have  4: "\<turnstile>  di \<not>\<^sub>i \<not>\<^sub>i  (init w) \<supset>\<^sub>i (init w) " using "2" "3" MP by blast
 have  5: "\<turnstile> (init w) \<supset>\<^sub>i \<not>\<^sub>i \<not>\<^sub>i  (init w) " by auto
 hence 6: "\<turnstile>  di  (init w) \<supset>\<^sub>i  di \<not>\<^sub>i \<not>\<^sub>i  (init w) " by (rule DiImpDi) 
 have  7: "\<turnstile>  di  (init w) \<supset>\<^sub>i (init w)" using "6" "4" MP using prop02 by blast
 have  8: "\<turnstile> (init w) \<supset>\<^sub>i  di  (init w) " by (rule ImpDi)
 from 7 8 show ?thesis using itl_prop(31) by blast
qed

lemma StateChop:
 "\<turnstile> (init w); f \<supset>\<^sub>i (init w) "
using DiState by auto

lemma StateChopExportA:
 "\<turnstile> ((init w) \<and>\<^sub>i f); g \<supset>\<^sub>i (init w) "
using DiState by auto

lemma StateAndChop:
 "\<turnstile> ((init w) \<and>\<^sub>i f); g \<equiv>\<^sub>i(init w) \<and>\<^sub>i (f; g) "
using StateAndChopImport StateChopExportA AndChopB itl_prop(31) itl_prop(32) by blast
 
lemma StateAndChopImpChopRule:
 assumes "\<turnstile> (init w) \<and>\<^sub>i f \<supset>\<^sub>i f1 " 
 shows   "\<turnstile> (init w) \<and>\<^sub>i (f; g) \<supset>\<^sub>i (f1; g) "
proof -
 have  1: "\<turnstile> (init w) \<and>\<^sub>i f \<supset>\<^sub>i f1 " using assms by auto
 hence 2: "\<turnstile> ((init w) \<and>\<^sub>i f); g \<supset>\<^sub>i f1; g " by (rule LeftChopImpChop)
 have  3: "\<turnstile> ((init w) \<and>\<^sub>i f); g \<equiv>\<^sub>i (init w) \<and>\<^sub>i (f; g) " by (rule StateAndChop)
 from 2 3 show ?thesis by auto
qed

lemma StateImpChopEqvChop :
 assumes "\<turnstile>(init w) \<supset>\<^sub>i (f \<equiv>\<^sub>i f1)" 
 shows   "\<turnstile> (init w) \<supset>\<^sub>i ((f; g) \<equiv>\<^sub>i (f1; g)) "
proof -
 have  1: "\<turnstile> (init w) \<supset>\<^sub>i (f\<equiv>\<^sub>i f1)" using assms by auto
 hence 2: "\<turnstile> (init w) \<and>\<^sub>i f \<supset>\<^sub>i f1 " by auto
 hence 3: "\<turnstile> (init w) \<and>\<^sub>i (f; g) \<supset>\<^sub>i (f1; g) " by (rule StateAndChopImpChopRule)
 have  4: "\<turnstile> (init w) \<and>\<^sub>i f1 \<supset>\<^sub>i f " using "1" by auto
 hence 5: "\<turnstile> (init w) \<and>\<^sub>i (f1; g) \<supset>\<^sub>i (f; g)" by (rule StateAndChopImpChopRule)
 from 3 5 show ?thesis by auto
qed

lemma ChopEqvStateAndChop:
  assumes "\<turnstile> f \<equiv>\<^sub>i (init w) \<and>\<^sub>i f1 "
  shows   "\<turnstile> (f; g) \<equiv>\<^sub>i (init w) \<and>\<^sub>i (f1; g) "
proof -
 have  1: "\<turnstile> f \<equiv>\<^sub>i (init w) \<and>\<^sub>i f1 " using assms by auto
 hence 2: "\<turnstile> f; g \<equiv>\<^sub>i ((init w) \<and>\<^sub>i f1); g " by (rule LeftChopEqvChop)
 have  3: "\<turnstile> ((init w) \<and>\<^sub>i f1); g \<equiv>\<^sub>i (init w) \<and>\<^sub>i (f1; g) " by (rule StateAndChop)
 from 2 3 show ?thesis by auto
qed

lemma DiIntro:
 "\<turnstile> f \<supset>\<^sub>i  di  f "
proof -
  have  1: "\<turnstile> f;  empty  \<equiv>\<^sub>i f " by (rule ChopEmpty)
  have  2: "\<turnstile>  empty  \<supset>\<^sub>i true\<^sub>i " by auto
  hence 3: "\<turnstile> \<box>( empty  \<supset>\<^sub>i true\<^sub>i)" by (rule BoxGen)
  have  4: "\<turnstile> \<box>( empty  \<supset>\<^sub>i true\<^sub>i) \<supset>\<^sub>i  (f;  empty  \<supset>\<^sub>i f; true\<^sub>i) " by (rule BoxChopImpChop)
  have  5: "\<turnstile> f;  empty  \<supset>\<^sub>i f; true\<^sub>i " using "3" "4" MP by auto
  hence 6: "\<turnstile> f;  empty  \<supset>\<^sub>i  di  f"  by (simp add: di_d_def)
  from 1 6 show ?thesis by auto
qed

lemma BiElim:
 "\<turnstile>   bi  f \<supset>\<^sub>i f "
proof - 
  have  1: "\<turnstile> \<not>\<^sub>i  f \<supset>\<^sub>i  di \<not>\<^sub>i  f " by (rule DiIntro)
  have  2: "\<turnstile> (\<not>\<^sub>i  f \<supset>\<^sub>i  di \<not>\<^sub>i  f) \<supset>\<^sub>i (\<not>\<^sub>i ( di \<not>\<^sub>i  f) \<supset>\<^sub>i f) " by auto
  have  3: "\<turnstile> \<not>\<^sub>i  (di \<not>\<^sub>i  f) \<supset>\<^sub>i f " using 1 2 MP by blast
  from 3 show ?thesis by (metis bi_d_def)
qed

lemma BiContraPosImpDist:
  "\<turnstile> bi (\<not>\<^sub>i  g \<supset>\<^sub>i \<not>\<^sub>i  f) \<supset>\<^sub>i (bi  f) \<supset>\<^sub>i (bi  g) "
proof -
  have  1: "\<turnstile> bi (\<not>\<^sub>i  g \<supset>\<^sub>i \<not>\<^sub>i  f) \<supset>\<^sub>i ( di \<not>\<^sub>i  g) \<supset>\<^sub>i ( di \<not>\<^sub>i  f) " by (rule BiImpDiImpDi)
  hence 2: "\<turnstile> bi (\<not>\<^sub>i  g \<supset>\<^sub>i \<not>\<^sub>i  f) \<supset>\<^sub>i (\<not>\<^sub>i ( di \<not>\<^sub>i  f)) \<supset>\<^sub>i (\<not>\<^sub>i ( di \<not>\<^sub>i  g)) " by auto
  from 2 show ?thesis by (metis bi_d_def)
qed

lemma BiImpDist:
 "\<turnstile> bi (f\<supset>\<^sub>i g) \<supset>\<^sub>i (bi  f) \<supset>\<^sub>i (bi  g) "
proof -
 have  1: "\<turnstile> (f \<supset>\<^sub>i g) \<supset>\<^sub>i (\<not>\<^sub>i  g \<supset>\<^sub>i \<not>\<^sub>i  f) " by auto
 hence 2: "\<turnstile> \<not>\<^sub>i (\<not>\<^sub>i  g \<supset>\<^sub>i \<not>\<^sub>i  f) \<supset>\<^sub>i \<not>\<^sub>i (f \<supset>\<^sub>i g) " by auto
 hence 3: "\<turnstile> bi (\<not>\<^sub>i (\<not>\<^sub>i  g \<supset>\<^sub>i \<not>\<^sub>i  f) \<supset>\<^sub>i \<not>\<^sub>i (f \<supset>\<^sub>i g)) " by (rule BiGen)
 have  4: "\<turnstile> bi (\<not>\<^sub>i (\<not>\<^sub>i  g \<supset>\<^sub>i \<not>\<^sub>i  f) \<supset>\<^sub>i \<not>\<^sub>i (f \<supset>\<^sub>i g))
             \<supset>\<^sub>i  
             bi (f \<supset>\<^sub>i g) \<supset>\<^sub>i bi (\<not>\<^sub>i  g \<supset>\<^sub>i \<not>\<^sub>i  f) " by (rule BiContraPosImpDist)
 have  5: "\<turnstile> bi (f \<supset>\<^sub>i g) \<supset>\<^sub>i bi (\<not>\<^sub>i  g \<supset>\<^sub>i \<not>\<^sub>i  f) " using 3 4 MP by blast
 have  6: "\<turnstile> bi (\<not>\<^sub>i  g \<supset>\<^sub>i \<not>\<^sub>i  f) \<supset>\<^sub>i (bi  f) \<supset>\<^sub>i (bi  g) " by (rule BiContraPosImpDist)
 from 5 6 show ?thesis  using prop02 by blast 
qed

lemma IfChopEqvRule:
 assumes "\<turnstile> f \<equiv>\<^sub>i  if\<^sub>i  (init w)  then  f1  else  f2 "
 shows   "\<turnstile> f; g \<equiv>\<^sub>i  if\<^sub>i  (init w)  then  (f1; g)  else  (f2; g) "
proof -
 have  1: "\<turnstile> f \<equiv>\<^sub>i  if\<^sub>i  (init w)  then  f1  else  f2" using assms by auto
 hence 2: "\<turnstile> f \<equiv>\<^sub>i ((init w) \<and>\<^sub>i f1) \<or>\<^sub>i  ( (init \<not>\<^sub>i w) \<and>\<^sub>i f2) " by (simp add: ifthenelse_d_def)
 hence 3: "\<turnstile> f; g \<equiv>\<^sub>i ((init w) \<and>\<^sub>i f1); g \<or>\<^sub>i  (  (init \<not>\<^sub>i w) \<and>\<^sub>i f2); g " by (rule OrChopEqvRule)
 have  4: "\<turnstile> ((init w) \<and>\<^sub>i f1); g \<equiv>\<^sub>i (init w) \<and>\<^sub>i (f1; g) " by (rule StateAndChop)
 have  5: "\<turnstile> (  (init \<not>\<^sub>i w) \<and>\<^sub>i f2); g \<equiv>\<^sub>i   (init \<not>\<^sub>i w) \<and>\<^sub>i (f2; g) " by (rule StateAndChop)
 have  6: "\<turnstile> f; g \<equiv>\<^sub>i ((init w) \<and>\<^sub>i f1; g)  \<or>\<^sub>i  (  (init \<not>\<^sub>i w) \<and>\<^sub>i f2; g) " using "3" "4" "5" by auto
 from 6 show ?thesis by (simp add: ifthenelse_d_def)
qed

lemma ChopOrEqvRule:
 assumes "\<turnstile> g \<equiv>\<^sub>i g1 \<or>\<^sub>i  g2 "
 shows   "\<turnstile> f; g \<equiv>\<^sub>i (f; g1) \<or>\<^sub>i  (f; g2) "
proof -
 have  1: "\<turnstile> g \<equiv>\<^sub>i g1 \<or>\<^sub>i  g2 " using assms by auto
 hence 2: "\<turnstile> f; g \<equiv>\<^sub>i f; (g1 \<or>\<^sub>i  g2) " by (rule RightChopEqvChop)
 have  3: "\<turnstile> f; (g1 \<or>\<^sub>i  g2) \<equiv>\<^sub>i f; g1 \<or>\<^sub>i  f; g2 " by (rule ChopOrEqv)
 from 2 3 show ?thesis by auto
qed

lemma EmptyOrChopEqv: 
 "\<turnstile> ( empty  \<or>\<^sub>i  f); g \<equiv>\<^sub>i g \<or>\<^sub>i  (f; g) "
proof -
 have 1: "\<turnstile> ( empty  \<or>\<^sub>i  f); g \<equiv>\<^sub>i ( empty ; g) \<or>\<^sub>i  (f; g) " by (rule OrChopEqv)
 have 2: "\<turnstile>  empty ; g \<equiv>\<^sub>i g " by (rule EmptyChop)
 from 1 2 show ?thesis by auto
qed

lemma EmptyOrNextChopEqv:
  "\<turnstile> ( empty  \<or>\<^sub>i  \<circle> f); g \<equiv>\<^sub>i g \<or>\<^sub>i  \<circle>(f; g) "
proof -
 have 1: "\<turnstile> ( empty  \<or>\<^sub>i  \<circle> f); g \<equiv>\<^sub>i g \<or>\<^sub>i  ((\<circle> f); g) " by (rule EmptyOrChopEqv)
 have 2: "\<turnstile> (\<circle> f); g \<equiv>\<^sub>i \<circle>(f; g) " by (rule NextChop)
 from 1 2 show ?thesis by auto
qed

lemma EmptyOrChopImpRule:
 assumes "\<turnstile>   f \<supset>\<^sub>i  empty  \<or>\<^sub>i  f1 " 
 shows   "\<turnstile> f; g \<supset>\<^sub>i g \<or>\<^sub>i  (f1; g) " 
proof -
 have  1: "\<turnstile> f \<supset>\<^sub>i  empty  \<or>\<^sub>i  f1 " using assms by auto
 hence 2: "\<turnstile> f; g \<supset>\<^sub>i ( empty  \<or>\<^sub>i  f1); g " by (rule LeftChopImpChop)
 have  3: "\<turnstile> ( empty  \<or>\<^sub>i  f1); g \<equiv>\<^sub>i g \<or>\<^sub>i  (f1; g) " by (rule EmptyOrChopEqv)
 from 2 3 show ?thesis by auto
qed

lemma EmptyOrChopEqvRule:
 assumes "\<turnstile>   f \<equiv>\<^sub>i  empty  \<or>\<^sub>i  f1 "
 shows   "\<turnstile> f; g \<equiv>\<^sub>i g \<or>\<^sub>i  (f1; g) "
proof -
 have  1: "\<turnstile> f \<equiv>\<^sub>i  empty  \<or>\<^sub>i  f1  " using assms by auto
 hence 2: "\<turnstile> f; g \<equiv>\<^sub>i ( empty  \<or>\<^sub>i  f1); g " by (rule LeftChopEqvChop)
 have  3: "\<turnstile> ( empty  \<or>\<^sub>i  f1); g \<equiv>\<^sub>i g \<or>\<^sub>i  (f1; g)" by (rule EmptyOrChopEqv)
 from 2 3 show ?thesis by auto
qed

lemma EmptyOrNextChopImpRule:
 assumes "\<turnstile> f \<supset>\<^sub>i  empty  \<or>\<^sub>i  \<circle> f1 "
 shows   "\<turnstile> f; g \<supset>\<^sub>i g \<or>\<^sub>i  \<circle>(f1; g) "
proof -
 have  1: "\<turnstile> f \<supset>\<^sub>i  empty  \<or>\<^sub>i  \<circle> f1 " using assms by auto
 hence 2: "\<turnstile> f; g \<supset>\<^sub>i ( empty  \<or>\<^sub>i  \<circle> f1); g " by (rule LeftChopImpChop)
 have  3: "\<turnstile> ( empty  \<or>\<^sub>i  \<circle> f1); g \<equiv>\<^sub>i g \<or>\<^sub>i  \<circle>(f1; g) " by (rule EmptyOrNextChopEqv)
 from 2 3 show ?thesis by auto
qed

lemma EmptyOrNextChopEqvRule:
 assumes "\<turnstile> f \<equiv>\<^sub>i  empty  \<or>\<^sub>i  \<circle> f1 "
 shows   "\<turnstile> f; g \<equiv>\<^sub>i g \<or>\<^sub>i  \<circle>(f1; g) "
proof -
 have  1: "\<turnstile> f \<equiv>\<^sub>i  empty  \<or>\<^sub>i  \<circle> f1 " using assms by auto
 hence 2: "\<turnstile> f; g \<equiv>\<^sub>i ( empty  \<or>\<^sub>i  \<circle> f1); g " by (rule LeftChopEqvChop)
 have  3: "\<turnstile> ( empty  \<or>\<^sub>i  \<circle> f1); g \<equiv>\<^sub>i g \<or>\<^sub>i  \<circle>(f1; g) " by (rule EmptyOrNextChopEqv)
 from 2 3 show ?thesis by auto
qed

lemma ChopEmptyOrImpRule:
 assumes "\<turnstile>  g \<supset>\<^sub>i  empty  \<or>\<^sub>i  g1 "
 shows "\<turnstile> f; g \<supset>\<^sub>i f \<or>\<^sub>i  (f; g1) "
proof -
 have  1: "\<turnstile> g \<supset>\<^sub>i  empty  \<or>\<^sub>i  g1 " using assms by auto
 hence 2: "\<turnstile> f; g \<supset>\<^sub>i (f;  empty ) \<or>\<^sub>i  (f; g1) " by (rule ChopOrImpRule)
 have  3: "\<turnstile> f;  empty  \<equiv>\<^sub>i f " by (rule ChopEmpty)
 from 2 3 show ?thesis by auto
qed

lemma StateAndEmptyImpBoxState:
 "\<turnstile> (init w) \<and>\<^sub>i empty \<supset>\<^sub>i \<box> (init w)"
by simp

lemma BoxEqvAndBox:
 "\<turnstile> \<box> f \<equiv>\<^sub>i f \<and>\<^sub>i \<box> f"
by fastforce

lemma NotBoxImpNotOrNotNextBox:
 "\<turnstile> \<not>\<^sub>i( \<box> f) \<supset>\<^sub>i \<not>\<^sub>if \<or>\<^sub>i  \<not>\<^sub>i( \<circle> (\<box> f) )"
proof -
  have 1: "\<turnstile> f \<and>\<^sub>i wnext (\<box> f) \<equiv>\<^sub>i f \<and>\<^sub>i \<box> f"
       by (meson BoxEqvAndBox BoxEqvAndWnextBox prop21) 
  have 2: "\<turnstile> \<not>\<^sub>i (wnext (\<box> f)) \<supset>\<^sub>i \<not>\<^sub>i (\<circle> (\<box> f))"
       by (metis (full_types) NextImpNotNextNot prop27 wnext_d_def) 
  then show ?thesis using 1 by (metis (no_types) and_d_def itl_prop(33) prop19 prop35) 
qed


lemma BoxStateChopBoxEqvBox:
 "\<turnstile>   \<box> (init w); \<box> (init w) \<equiv>\<^sub>i \<box> (init w) "
proof -
 have   1: "\<turnstile> \<box> (init w) \<equiv>\<^sub>i (init w) \<and>\<^sub>i ( empty  \<or>\<^sub>i  \<circle>(\<box> (init w))) " 
        by (rule BoxEqvAndEmptyOrNextBox)
 hence  2: "\<turnstile> \<box> (init w); \<box> (init w) \<equiv>\<^sub>i
               (init w) \<and>\<^sub>i (( empty  \<or>\<^sub>i  \<circle>(\<box> (init w))); \<box> (init w)) " 
        by (rule ChopEqvStateAndChop)
 have   3: "\<turnstile> ( empty  \<or>\<^sub>i  \<circle>(\<box> (init w))); \<box> (init w) \<equiv>\<^sub>i
              \<box> (init w) \<or>\<^sub>i  \<circle>(\<box> (init w); \<box> (init w)) " 
        by (rule EmptyOrNextChopEqv)
 have   4: "\<turnstile> \<box> (init w); \<box> (init w) \<equiv>\<^sub>i 
              (init w) \<and>\<^sub>i (\<box> (init w) \<or>\<^sub>i  \<circle>(\<box> (init w); \<box> (init w))) " 
        using "2" "3" by auto
 have   5: "\<turnstile> \<not>\<^sub>i (\<box> (init w)) \<supset>\<^sub>i \<not>\<^sub>i  (init w) \<or>\<^sub>i  \<not>\<^sub>i( \<circle>(\<box> (init w))) " 
        by (rule NotBoxImpNotOrNotNextBox)
 have   6: "\<turnstile> (\<box> (init w); \<box> (init w)) \<and>\<^sub>i \<not>\<^sub>i( \<box> (init w)) \<supset>\<^sub>i
              \<circle>(\<box> (init w); \<box> (init w)) \<and>\<^sub>i \<not>\<^sub>i( \<circle>(\<box> (init w)))  " 
        using "4" "5" by auto
 hence  7: "\<turnstile> \<box> (init w); \<box> (init w) \<supset>\<^sub>i \<box> (init w) " 
       by (rule NextContra) 
 have  11: "\<turnstile> \<box> (init w) \<equiv>\<^sub>i (init w) \<and>\<^sub>i \<box> (init w) " 
       by (rule BoxEqvAndBox)
 have  12: "\<turnstile> empty ; \<box> (init w) \<equiv>\<^sub>i \<box> (init w)" 
       by (rule EmptyChop)
 have  13: "\<turnstile> ((init w) \<and>\<^sub>i  empty ); \<box> (init w) \<equiv>\<^sub>i (init w) \<and>\<^sub>i ( empty ; \<box> (init w)) " 
       by (rule StateAndChop)
 have  14: "\<turnstile> \<box> (init w) \<equiv>\<^sub>i ((init w) \<and>\<^sub>i  empty ); \<box> (init w) " 
       using "11" "12" "13" by auto
 have  15: "\<turnstile> (init w) \<and>\<^sub>i  empty  \<supset>\<^sub>i \<box> (init w) " 
       by (rule StateAndEmptyImpBoxState)
 hence 16: "\<turnstile> ((init w) \<and>\<^sub>i  empty ); \<box> (init w) \<supset>\<^sub>i \<box> (init w); \<box> (init w) " 
       by (rule LeftChopImpChop)
 have  17: "\<turnstile> \<box> (init w) \<supset>\<^sub>i \<box> (init w); \<box> (init w)" 
       using "14" "16" by auto
 from 7 17 show ?thesis using  itl_prop(31) by blast
qed

lemma NotBoxStateImpBoxYieldsNotBox:
 "\<turnstile>   \<not>\<^sub>i( \<box> (init w)) \<supset>\<^sub>i (\<box> (init w)) yields \<not>\<^sub>i( \<box> (init w))"
proof -
 have  1: "\<turnstile> \<box> (init w); \<box> (init w) \<equiv>\<^sub>i \<box> (init w) " by (rule BoxStateChopBoxEqvBox)
 have  2: "\<turnstile> \<box> (init w) \<equiv>\<^sub>i \<not>\<^sub>i \<not>\<^sub>i( \<box> (init w)) " by auto
 hence 3: "\<turnstile> \<box> (init w); \<box> (init w) \<equiv>\<^sub>i \<box> (init w); \<not>\<^sub>i \<not>\<^sub>i( \<box> (init w)) " by (rule RightChopEqvChop)
 have  4: "\<turnstile> \<not>\<^sub>i( \<box> (init w)) \<supset>\<^sub>i \<not>\<^sub>i (\<box> (init w); \<not>\<^sub>i \<not>\<^sub>i (\<box> (init w))) " using "1" "3" by auto
 from 4 show ?thesis by (simp add: yields_d_def)
qed

lemma StateEqvBi:
 "\<turnstile>   (init w) \<equiv>\<^sub>i bi  (init w) "
proof -
  have 1: "\<turnstile> (init w) \<supset>\<^sub>i bi  (init w) " by (rule StateImpBi)
  have 2: "\<turnstile> bi  (init w) \<supset>\<^sub>i (init w) " by (rule BiElim)
  from 1 2 show ?thesis using itl_prop(31) by blast
qed


lemma TrueChopEqvDiamond:
 "\<turnstile>   true\<^sub>i; f \<equiv>\<^sub>i \<diamond> f"
by simp

subsection \<open>Properties of Da and Ba\<close>

lemma DaEqvDtDi:
 "\<turnstile>    da  f \<equiv>\<^sub>i \<diamond> (di  f) "
proof -
 have  1: "\<turnstile> true\<^sub>i; (f; true\<^sub>i) \<equiv>\<^sub>i true\<^sub>i; (f; true\<^sub>i) " by auto
 hence 2: "\<turnstile> true\<^sub>i; (f; true\<^sub>i) \<equiv>\<^sub>i true\<^sub>i;  di  f " by (simp add: di_d_def)
 have  3: "\<turnstile> true\<^sub>i;  di  f \<equiv>\<^sub>i \<diamond>( di  f)" by (rule TrueChopEqvDiamond)
 have  4: "\<turnstile> true\<^sub>i; (f; true\<^sub>i) \<equiv>\<^sub>i \<diamond>( di  f) " using "2" "3" by auto
 from 4 show ?thesis by (simp add:da_d_def)
qed

lemma DaEqvDiDt:
 "\<turnstile>    da  f \<equiv>\<^sub>i  di (\<diamond> f) "
proof -
 have  1: "\<turnstile> true\<^sub>i; f \<equiv>\<^sub>i \<diamond> f " by (rule TrueChopEqvDiamond)
 hence 2: "\<turnstile> (true\<^sub>i; f); true\<^sub>i \<equiv>\<^sub>i (\<diamond> f); true\<^sub>i " by (rule LeftChopEqvChop)
 hence 3: "\<turnstile> (true\<^sub>i; f); true\<^sub>i \<equiv>\<^sub>i  di( \<diamond> f) " by (simp add: di_d_def)
 have  4: "\<turnstile> true\<^sub>i; (f; true\<^sub>i) \<equiv>\<^sub>i (true\<^sub>i; f); true\<^sub>i   " by (rule ChopAssoc)
 have  5: "\<turnstile> true\<^sub>i; (f; true\<^sub>i) \<equiv>\<^sub>i  di (\<diamond> f) " using "3" "4" by auto
 from 5 show ?thesis by (simp add: da_d_def)
qed

lemma DtDiEqvDiDt:
 "\<turnstile>   \<diamond> (di  f) \<equiv>\<^sub>i  di (\<diamond> f) "
by (metis ChopAssoc di_d_def sometimes_d_def)

lemma DiamondNotEqvNotBox:
 "\<turnstile> \<diamond> \<not>\<^sub>i  f \<equiv>\<^sub>i \<not>\<^sub>i (\<box> f)"
by simp

lemma BaEqvBiBt:
 "\<turnstile>    ba  f \<equiv>\<^sub>i bi( \<box> f) "
proof -
 have  1: "\<turnstile>  da \<not>\<^sub>i  f \<equiv>\<^sub>i  di( \<diamond> \<not>\<^sub>i  f) " by (rule DaEqvDiDt)
 have  2: "\<turnstile> \<diamond> \<not>\<^sub>i  f \<equiv>\<^sub>i \<not>\<^sub>i( \<box> f) " by (rule DiamondNotEqvNotBox)
 hence 3: "\<turnstile>  di (\<diamond>\<not>\<^sub>i  f) \<equiv>\<^sub>i  di \<not>\<^sub>i (\<box> f) " by (rule DiEqvDi)
 have  4: "\<turnstile>  da \<not>\<^sub>i  f \<equiv>\<^sub>i  di \<not>\<^sub>i( \<box> f) " using "1" "3" by auto
 hence 5: "\<turnstile> \<not>\<^sub>i  (da \<not>\<^sub>i  f) \<equiv>\<^sub>i \<not>\<^sub>i ( di \<not>\<^sub>i( \<box> f)) " by auto
 hence 6: "\<turnstile> \<not>\<^sub>i ( da \<not>\<^sub>i  f) \<equiv>\<^sub>i bi( \<box> f) " by (simp add: bi_d_def)
 from 6 show ?thesis by (simp add: ba_d_def)
qed
 
lemma DiNotEqvNotBi:
 "\<turnstile>    di \<not>\<^sub>i  f \<equiv>\<^sub>i \<not>\<^sub>i( bi  f) "
proof -
 have 1: "\<turnstile> bi  f \<equiv>\<^sub>i \<not>\<^sub>i ( di \<not>\<^sub>i  f) " by (simp add: bi_d_def)
 from 1 show ?thesis by auto
qed

lemma NotDiamondNotEqvBox:
 "\<turnstile> \<not>\<^sub>i (\<diamond>\<not>\<^sub>i f) \<equiv>\<^sub>i \<box> f "
by simp

lemma BaEqvBtBi:
 "\<turnstile>    ba  f \<equiv>\<^sub>i \<box> (bi  f) "
proof -
 have  1: "\<turnstile>  da \<not>\<^sub>i  f \<equiv>\<^sub>i \<diamond> (di \<not>\<^sub>i  f) " by (rule DaEqvDtDi)
 have  2: "\<turnstile>  di \<not>\<^sub>i  f \<equiv>\<^sub>i \<not>\<^sub>i (bi  f) " by (rule DiNotEqvNotBi)
 hence 3: "\<turnstile> \<diamond> (di \<not>\<^sub>i  f) \<equiv>\<^sub>i \<diamond>\<not>\<^sub>i (bi  f) " by (rule DiamondEqvDiamond)
 have  4: "\<turnstile> \<not>\<^sub>i (\<diamond>\<not>\<^sub>i (bi  f)) \<equiv>\<^sub>i \<box>(bi  f)  " by (rule NotDiamondNotEqvBox)
 have  5: "\<turnstile> \<not>\<^sub>i ( da \<not>\<^sub>i  f) \<equiv>\<^sub>i \<box>(bi  f) " using "1" "2" "3" by auto
 from 5 show ?thesis by (simp add: ba_d_def)
qed

lemma BtBiEqvBiBt:
 "\<turnstile>   \<box> (bi  f) \<equiv>\<^sub>i bi( \<box> f) "
proof -
 have 1: "\<turnstile>    ba  f \<equiv>\<^sub>i \<box> (bi  f)  " by (rule BaEqvBtBi)
 have 2: "\<turnstile>    ba  f \<equiv>\<^sub>i bi( \<box> f) "  by (rule BaEqvBiBt)
 from 1 2 show ?thesis by auto
qed

lemma BoxStateEqvBaBoxState:
 "\<turnstile>   \<box> (init w) \<equiv>\<^sub>i  ba (\<box> (init w)) "
proof -
 have  1: "\<turnstile> (init w) \<equiv>\<^sub>i bi  (init w) " by (rule StateEqvBi)
 hence 2: "\<turnstile> \<box> (init w) \<equiv>\<^sub>i \<box> (bi  (init w)) " by (rule BoxEqvBox)
 have  3: "\<turnstile> \<box> (bi  (init w)) \<equiv>\<^sub>i bi( \<box> (init w))  " by (rule BtBiEqvBiBt)
 have  4: "\<turnstile> \<box> (init w) \<equiv>\<^sub>i \<box>(\<box> (init w)) " by (rule BoxEqvBoxBox)
 hence 5: "\<turnstile> bi( \<box> (init w)) \<equiv>\<^sub>i bi (\<box>(\<box> (init w))) " by (rule BiEqvBi)
 have  6: " \<turnstile>  ba( \<box> (init w)) \<equiv>\<^sub>i bi( \<box>(\<box> (init w))) "  by (rule BaEqvBiBt)
 from 2 3 5 6 show ?thesis  by simp 
qed

lemma BaImpBi:
 "\<turnstile>    ba  f \<supset>\<^sub>i bi  f "
proof -
 have 1: "\<turnstile>  ba  f \<equiv>\<^sub>i \<box>(bi  f) " by (rule BaEqvBtBi)
 have 2: "\<turnstile> \<box>(bi  f) \<supset>\<^sub>i bi  f " by (rule BoxElim)
 from 1 2 show ?thesis using MP  using itl_prop(31) prop02 by blast 
qed

lemma BaImpBt:
 "\<turnstile>    ba  f \<supset>\<^sub>i \<box> f "
proof -
 have 1: "\<turnstile>  ba  f \<equiv>\<^sub>i bi( \<box> f) " by (rule BaEqvBiBt)
 have 2: "\<turnstile> bi( \<box> f) \<supset>\<^sub>i \<box> f " by (rule BiElim)
 from 1 2 show ?thesis using MP  using itl_prop(31) prop02 by blast 
qed

lemma DiamondImpDa:
 "\<turnstile>   \<diamond> f \<supset>\<^sub>i  da  f "
by (metis DiIntro DiamondImpDiamond da_d_def di_d_def sometimes_d_def)

lemma DiImpDa:
 "\<turnstile>    di  f \<supset>\<^sub>i  da  f"
by (metis NowImpDiamond da_d_def di_d_def sometimes_d_def)

lemma BoxAndChopImport:
 "\<turnstile>   \<box> h \<and>\<^sub>i f; g \<supset>\<^sub>i f; (h \<and>\<^sub>i g) " 
proof -
 have  1: "\<turnstile> h \<supset>\<^sub>i g \<supset>\<^sub>i (h\<and>\<^sub>i g) " by auto
 hence 2: "\<turnstile> \<box> h \<supset>\<^sub>i \<box>(g \<supset>\<^sub>i (h\<and>\<^sub>i g)) " by (rule ImpBoxRule)
 have  3: "\<turnstile> \<box>(g \<supset>\<^sub>i (h\<and>\<^sub>i g)) \<supset>\<^sub>i f; g \<supset>\<^sub>i f; (h\<and>\<^sub>i g) " by (rule BoxChopImpChop)
 from 2 3 show ?thesis by auto
qed

lemma BaAndChopImport:
 " \<turnstile>    ba  f \<and>\<^sub>i (g; g1) \<supset>\<^sub>i (f \<and>\<^sub>i g); (f \<and>\<^sub>i g1) "
proof -
 have  1: "\<turnstile> ba  f \<supset>\<^sub>i bi  f " by (rule BaImpBi)
 have  2: "\<turnstile> bi  f \<and>\<^sub>i (g; g1) \<supset>\<^sub>i (f \<and>\<^sub>i g); g1 " by (rule BiAndChopImport)
 have  3: "\<turnstile> ba  f \<supset>\<^sub>i \<box> f " by (rule BaImpBt)
 have  4: "\<turnstile> \<box> f \<and>\<^sub>i (f \<and>\<^sub>i g); g1 \<supset>\<^sub>i (f \<and>\<^sub>i g); (f \<and>\<^sub>i g1) " by (rule BoxAndChopImport)
 from 1 2 3 4 show ?thesis by simp
qed

lemma ChopAndCommute:
 "\<turnstile>   f; (g \<and>\<^sub>i g1) \<equiv>\<^sub>i f; (g1 \<and>\<^sub>i g) "
proof -
 have 1: "\<turnstile> (g \<and>\<^sub>i g1) \<equiv>\<^sub>i (g1 \<and>\<^sub>i g)" by auto
 from 1 show ?thesis by (rule RightChopEqvChop)
qed

lemma ChopAndA: 
 "\<turnstile>   f; (g \<and>\<^sub>i g1) \<supset>\<^sub>i f; g "
proof -
 have 1: "\<turnstile> (g \<and>\<^sub>i g1) \<supset>\<^sub>i  g " by auto
 from 1 show ?thesis by (rule RightChopImpChop)
qed

lemma ChopAndB:
 "\<turnstile>   f; (g \<and>\<^sub>i g1) \<supset>\<^sub>i f; g1 "
proof -
 have 1: "\<turnstile> (g \<and>\<^sub>i g1) \<supset>\<^sub>i  g1 " by auto
from 1 show ?thesis by (rule RightChopImpChop)
qed
 
lemma BoxStateAndChopEqvChop:
 "\<turnstile>  \<box> (init w) \<and>\<^sub>i (f; g) \<equiv>\<^sub>i (\<box> (init w) \<and>\<^sub>i f); (\<box> (init w) \<and>\<^sub>i g) "
proof -
 have  1: "\<turnstile> \<box> (init w) \<equiv>\<^sub>i  ba( \<box> (init w)) " 
       by (rule BoxStateEqvBaBoxState)
 have  2: "\<turnstile> ba( \<box> (init w)) \<and>\<^sub>i (f; g) \<supset>\<^sub>i (\<box> (init w) \<and>\<^sub>i f); (\<box> (init w) \<and>\<^sub>i g) " 
       by (rule BaAndChopImport)
 have  3: "\<turnstile> \<box> (init w) \<and>\<^sub>i (f; g) \<supset>\<^sub>i (\<box> (init w) \<and>\<^sub>i f); (\<box> (init w) \<and>\<^sub>i g) " 
       using "1" "2" prop18 by blast  
 have 11: "\<turnstile> (\<box> (init w) \<and>\<^sub>i f); (\<box> (init w) \<and>\<^sub>i g) \<supset>\<^sub>i (\<box> (init w)); (\<box> (init w) \<and>\<^sub>i g) " 
       by (rule AndChopA)
 have 12: "\<turnstile> (\<box> (init w)); (\<box> (init w) \<and>\<^sub>i g) \<supset>\<^sub>i (\<box> (init w)); (\<box> (init w)) " 
       by (rule ChopAndA)
 have 13: "\<turnstile> (\<box> (init w)); (\<box> (init w)) \<equiv>\<^sub>i \<box> (init w) " 
       by (rule BoxStateChopBoxEqvBox)
 have 14: "\<turnstile> (\<box> (init w) \<and>\<^sub>i f); (\<box> (init w) \<and>\<^sub>i g) \<supset>\<^sub>i f; (\<box> (init w) \<and>\<^sub>i g) " 
       by (rule AndChopB)
 have 15: "\<turnstile> f; (\<box> (init w) \<and>\<^sub>i g) \<supset>\<^sub>i f; g " 
       by (rule ChopAndB)
 have 16: "\<turnstile> (\<box> (init w) \<and>\<^sub>i f); (\<box> (init w) \<and>\<^sub>i g) \<supset>\<^sub>i \<box> (init w) \<and>\<^sub>i (f; g) " 
       using  "11" "12" "13" "14" "15"  using itl_prop(31) itl_prop(32) prop02 by metis 
 from 3 16 show ?thesis using itl_prop(31) by blast
qed

lemma DiEqvNotBiNot:
 " \<turnstile>    di  f \<equiv>\<^sub>i \<not>\<^sub>i( bi \<not>\<^sub>i  f) "
proof -
 have  1: "\<turnstile> bi \<not>\<^sub>i  f \<equiv>\<^sub>i \<not>\<^sub>i ( di \<not>\<^sub>i \<not>\<^sub>i  f) " by (simp add: bi_d_def)
 hence 2: "\<turnstile>  di \<not>\<^sub>i \<not>\<^sub>i  f \<equiv>\<^sub>i \<not>\<^sub>i( bi \<not>\<^sub>i  f) " by auto
 have  3: "\<turnstile> f \<equiv>\<^sub>i \<not>\<^sub>i \<not>\<^sub>i  f " by auto
 hence 4: "\<turnstile>  di  f \<equiv>\<^sub>i  di  \<not>\<^sub>i \<not>\<^sub>i  f " by (rule DiEqvDi)
 from 2 4 show ?thesis by auto
qed

lemma ChopAndBoxImport:
 "\<turnstile>   f; g \<and>\<^sub>i \<box> h \<supset>\<^sub>i f; (g \<and>\<^sub>i h) "
proof -
 have 1: "\<turnstile> \<box> h \<and>\<^sub>i f; g \<supset>\<^sub>i f; (h \<and>\<^sub>i g) " by (rule BoxAndChopImport)
 have 2: "\<turnstile> f; (h \<and>\<^sub>i g) \<equiv>\<^sub>i f; (g \<and>\<^sub>i h) " by (rule ChopAndCommute)
 from 1 2 show ?thesis by auto
qed

lemma AndChopAndCommute:
 "\<turnstile>   (f \<and>\<^sub>i g); (f1 \<and>\<^sub>i g1) \<equiv>\<^sub>i (g \<and>\<^sub>i f); (g1 \<and>\<^sub>i f1) "
proof -
 have 1: "\<turnstile> (f \<and>\<^sub>i g); (f1 \<and>\<^sub>i g1) \<equiv>\<^sub>i (g \<and>\<^sub>i f); (f1 \<and>\<^sub>i g1) " by (rule AndChopCommute)
 have 2: "\<turnstile> (g \<and>\<^sub>i f); (f1 \<and>\<^sub>i g1) \<equiv>\<^sub>i (g \<and>\<^sub>i f); (g1 \<and>\<^sub>i f1) "  by (rule ChopAndCommute)
 from 1 2 show ?thesis by auto
qed

lemma ChopImpChop:
 assumes "\<turnstile>   f\<supset>\<^sub>i f1 " "\<turnstile> g\<supset>\<^sub>i g1 "
 shows   "\<turnstile> f; g \<supset>\<^sub>i f1; g1 "
proof -
 have  1: "\<turnstile> f \<supset>\<^sub>i f1 " using assms by auto
 hence 2: "\<turnstile> f; g \<supset>\<^sub>i f1; g " by (rule LeftChopImpChop)
 have  3: "\<turnstile> g \<supset>\<^sub>i g1 " using assms by auto
 hence 4: "\<turnstile> f1; g \<supset>\<^sub>i f1; g1 " by (rule RightChopImpChop)
 from 2 4 show ?thesis by auto
qed

lemma ChopEqvChop:
 assumes "\<turnstile>   f\<equiv>\<^sub>i f1" "\<turnstile> g\<equiv>\<^sub>i g1 "
 shows   "\<turnstile> f; g \<equiv>\<^sub>i f1; g1 "
proof -
 have  1: "\<turnstile> f \<equiv>\<^sub>i f1 " using assms by auto
 hence 2: "\<turnstile> f; g \<equiv>\<^sub>i f1; g " by (rule LeftChopEqvChop)
 have  3: "\<turnstile> g \<equiv>\<^sub>i g1 " using assms by auto
 hence 4: "\<turnstile> f1; g \<equiv>\<^sub>i f1; g1 " by (rule RightChopEqvChop)
 from 2 4 show ?thesis by auto
qed

lemma BoxImpBoxImpBox:
 "\<turnstile> \<box> h \<supset>\<^sub>i \<box>(g \<supset>\<^sub>i \<box> h \<and>\<^sub>i g ) "
proof -
 have 1: "\<turnstile> \<box> h \<supset>\<^sub>i (g \<supset>\<^sub>i \<box> h \<and>\<^sub>i g ) " by simp
 hence 2: "\<turnstile> \<box>(\<box> h) \<supset>\<^sub>i \<box>(g \<supset>\<^sub>i \<box> h \<and>\<^sub>i g ) " by (rule ImpBoxRule)
 have 3: "\<turnstile> \<box> h \<equiv>\<^sub>i \<box>(\<box>h) " by (rule BoxEqvBoxBox)
 from 2 3 show ?thesis by auto
qed

lemma BoxChopImpChopBox:
 "\<turnstile>   \<box> h \<supset>\<^sub>i f; g \<supset>\<^sub>i f; (\<box> h \<and>\<^sub>i g) "
proof -
 have 1: "\<turnstile> \<box> h \<supset>\<^sub>i \<box>(g \<supset>\<^sub>i \<box> h \<and>\<^sub>i g ) " by (rule BoxImpBoxImpBox)
 have 2: "\<turnstile> \<box>(g \<supset>\<^sub>i \<box> h \<and>\<^sub>i g ) \<supset>\<^sub>i f; g \<supset>\<^sub>i f; (\<box> h \<and>\<^sub>i g) " by (rule BoxChopImpChop)
 from 1 2 show ?thesis by auto
qed

lemma NotChopEqvYieldsNot:
"\<turnstile>   \<not>\<^sub>i (f; g) \<equiv>\<^sub>i f yields  \<not>\<^sub>i  g "
proof -
 have  1: "\<turnstile> g \<equiv>\<^sub>i \<not>\<^sub>i \<not>\<^sub>i  g " by auto
 hence 2: "\<turnstile> f; g \<equiv>\<^sub>i f; \<not>\<^sub>i \<not>\<^sub>i  g " by (rule RightChopEqvChop)
 hence 3: "\<turnstile> \<not>\<^sub>i (f; g) \<equiv>\<^sub>i \<not>\<^sub>i (f; \<not>\<^sub>i \<not>\<^sub>i  g) " by auto
 from 3 show ?thesis by (simp add: yields_d_def)
qed

lemma NotDiFalse:
 "\<turnstile>   \<not>\<^sub>i ( di  false\<^sub>i) "
proof -
 have  1: "\<turnstile> (init true\<^sub>i) \<supset>\<^sub>i bi (init true\<^sub>i) " by (rule StateImpBi)
 hence 2: "\<turnstile> true\<^sub>i \<supset>\<^sub>i bi true\<^sub>i " by auto
 have  3: "\<turnstile> true\<^sub>i" by auto
 have  4: "\<turnstile> bi true\<^sub>i" using "2" "3" MP by auto
 hence 5: "\<turnstile> \<not>\<^sub>i ( di \<not>\<^sub>i true\<^sub>i) " by simp
 have  6: "\<turnstile> \<not>\<^sub>i true\<^sub>i \<equiv>\<^sub>i  false\<^sub>i " by auto
 hence 7: "\<turnstile>  di \<not>\<^sub>i true\<^sub>i \<equiv>\<^sub>i  di  false\<^sub>i " by (rule DiEqvDi)
 from 5 7 show ?thesis by auto
qed

lemma StateAndEmptyChop:
 "\<turnstile>   ((init w) \<and>\<^sub>i  empty ); f \<equiv>\<^sub>i (init w) \<and>\<^sub>i f "
proof - 
 have 1: "\<turnstile> ((init w) \<and>\<^sub>i  empty ); f \<equiv>\<^sub>i (init w) \<and>\<^sub>i  empty ; f " by (rule StateAndChop)
 have 2: "\<turnstile>  empty ; f \<equiv>\<^sub>i f " by (rule EmptyChop)
 from 1 2 show ?thesis by auto
qed

lemma StateAndNextChop:
 "\<turnstile>   ((init w) \<and>\<^sub>i \<circle> f); g \<equiv>\<^sub>i (init w) \<and>\<^sub>i \<circle>(f; g) "
proof -
 have 1: "\<turnstile> ((init w) \<and>\<^sub>i \<circle> f); g \<equiv>\<^sub>i (init w) \<and>\<^sub>i (\<circle> f); g " by (rule StateAndChop)
 have 2: "\<turnstile> (\<circle> f); g \<equiv>\<^sub>i \<circle>(f; g) " by (rule NextChop)
 from 1 2 show ?thesis by auto
qed

lemma NextAndEqvNextAndNext:
 "\<turnstile> \<circle> (f \<and>\<^sub>i g) \<equiv>\<^sub>i \<circle> f \<and>\<^sub>i \<circle> g"
by auto

lemma NextStateAndChop:
 "\<turnstile>   \<circle>(((init w) \<and>\<^sub>i f); g) \<equiv>\<^sub>i \<circle> (init w) \<and>\<^sub>i \<circle>(f; g) "
proof -
 have  1: "\<turnstile> ((init w) \<and>\<^sub>i f); g \<equiv>\<^sub>i (init w) \<and>\<^sub>i f; g  " by (rule StateAndChop)
 hence 2: "\<turnstile> \<circle>(((init w) \<and>\<^sub>i f); g) \<equiv>\<^sub>i \<circle>((init w) \<and>\<^sub>i f; g) " by (rule NextEqvNext)
 have  3: "\<turnstile> \<circle>((init w) \<and>\<^sub>i f; g) \<equiv>\<^sub>i \<circle> (init w) \<and>\<^sub>i \<circle>(f; g) " by (rule NextAndEqvNextAndNext)
 from 2 3 show ?thesis by auto
qed

lemma StateYieldsEqv:
 "\<turnstile>    ((init w) \<supset>\<^sub>i (f yields  g)) \<equiv>\<^sub>i ((init w) \<and>\<^sub>i f) yields  g " 
proof -
 have  1: "\<turnstile> ((init w) \<and>\<^sub>i f); \<not>\<^sub>i  g \<equiv>\<^sub>i (init w) \<and>\<^sub>i f; (\<not>\<^sub>i  g)  " by (rule StateAndChop)
 hence 2: "\<turnstile> ((init w) \<supset>\<^sub>i \<not>\<^sub>i (f; \<not>\<^sub>i  g)) \<equiv>\<^sub>i \<not>\<^sub>i (((init w) \<and>\<^sub>i f); \<not>\<^sub>i  g ) " by auto
 from 2 show ?thesis by (simp add: yields_d_def)
qed

lemma StateAndDi:
 "\<turnstile>   (init w) \<and>\<^sub>i  di  f \<equiv>\<^sub>i  di ((init w) \<and>\<^sub>i f) "
proof - 
 have 1: "\<turnstile> ((init w) \<and>\<^sub>i f); true\<^sub>i\<equiv>\<^sub>i  (init w) \<and>\<^sub>i f; true\<^sub>i  " by (rule StateAndChop)
 from 1 show ?thesis by (simp add: di_d_def)
qed

lemma DiNext: 
 "\<turnstile>    di( \<circle> f) \<equiv>\<^sub>i \<circle> (di  f) "
proof -
 have 1: "\<turnstile> (\<circle> f); true\<^sub>i \<equiv>\<^sub>i \<circle>(f; true\<^sub>i) " by (rule NextChop)
 from 1 show ?thesis by (simp add: di_d_def)
qed

lemma DiNextState:
 "\<turnstile>    di( \<circle> (init w)) \<equiv>\<^sub>i \<circle> (init w) "
proof -
 have  1: "\<turnstile>  di( \<circle> (init w)) \<equiv>\<^sub>i \<circle>( di  (init w)) " by (rule DiNext)
 have  2: "\<turnstile>  di  (init w) \<equiv>\<^sub>i (init w) " by (rule DiState)
 hence 3: "\<turnstile> \<circle>( di  (init w)) \<equiv>\<^sub>i \<circle> (init w) " by (rule NextEqvNext)
 from 1 3 show ?thesis by auto
qed

lemma StateImpBiGen:
 assumes "\<turnstile>   (init w)\<supset>\<^sub>i f "
 shows   "\<turnstile> (init w) \<supset>\<^sub>i bi  f "
proof -
 have  1: "\<turnstile> (init w) \<supset>\<^sub>i f " using assms by auto
 hence 2: "\<turnstile> \<not>\<^sub>i  f \<supset>\<^sub>i \<not>\<^sub>i  (init w) " by auto
 hence 3: "\<turnstile>  di \<not>\<^sub>i  f \<supset>\<^sub>i  di \<not>\<^sub>i  (init w) " by (rule DiImpDi)
 hence 4: "\<turnstile>  di \<not>\<^sub>i  f \<supset>\<^sub>i  di  (init \<not>\<^sub>iw) " by auto
 have  5: "\<turnstile>  di (init \<not>\<^sub>i w) \<equiv>\<^sub>i   (init \<not>\<^sub>i w) " by (rule DiState)
 have  6: "\<turnstile>  di \<not>\<^sub>i  f \<supset>\<^sub>i \<not>\<^sub>i  (init w) " using 4 5 by auto
 hence 7: "\<turnstile> (init w) \<supset>\<^sub>i \<not>\<^sub>i ( di \<not>\<^sub>i  f) " by auto
 from 7 show ?thesis by (simp add: bi_d_def)
qed

lemma ChopAndNotChopImp:
 "\<turnstile>   f; g \<and>\<^sub>i \<not>\<^sub>i (f; g1) \<supset>\<^sub>i f; (g \<and>\<^sub>i \<not>\<^sub>i  g1) "
proof -
 have  1: "\<turnstile> g \<supset>\<^sub>i (g\<and>\<^sub>i \<not>\<^sub>i  g1) \<or>\<^sub>i  g1 " by auto
 hence 2: "\<turnstile> f; g \<supset>\<^sub>i f; ((g\<and>\<^sub>i \<not>\<^sub>i  g1) \<or>\<^sub>i  g1) " by (rule RightChopImpChop)
 have  3: "\<turnstile> f; ((g\<and>\<^sub>i \<not>\<^sub>i  g1) \<or>\<^sub>i  g1) \<supset>\<^sub>i (f; (g\<and>\<^sub>i \<not>\<^sub>i  g1)) \<or>\<^sub>i  (f; g1) " by (rule ChopOrImp)
 have  4: "\<turnstile> f; g \<supset>\<^sub>i f; (g\<and>\<^sub>i \<not>\<^sub>i  g1) \<or>\<^sub>i  f; g1 " using "2" "3" MP by auto
 from 4 show ?thesis by auto
qed

lemma ChopAndYieldsImp:
 "\<turnstile>   f; g \<and>\<^sub>i f yields  g1 \<supset>\<^sub>i f; (g \<and>\<^sub>i g1) "
proof -
 have  1: "\<turnstile> g \<supset>\<^sub>i (g\<and>\<^sub>i g1) \<or>\<^sub>i  \<not>\<^sub>i  g1 " by auto
 hence 2: "\<turnstile> f; g \<supset>\<^sub>i f; ((g\<and>\<^sub>i   g1) \<or>\<^sub>i  \<not>\<^sub>i g1) "  by (rule RightChopImpChop)
 have  3: "\<turnstile> f; ((g\<and>\<^sub>i   g1) \<or>\<^sub>i  \<not>\<^sub>i g1) \<supset>\<^sub>i (f; (g\<and>\<^sub>i   g1)) \<or>\<^sub>i  (f; \<not>\<^sub>i g1) " by (rule ChopOrImp)
 have  4: "\<turnstile> f; g \<supset>\<^sub>i f; (g\<and>\<^sub>i   g1) \<or>\<^sub>i  f; \<not>\<^sub>i g1 " using "2" "3" MP by auto
 hence 5: "\<turnstile> f; g \<and>\<^sub>i \<not>\<^sub>i  (f; \<not>\<^sub>i  g1) \<supset>\<^sub>i f; (g \<and>\<^sub>i g1) " by auto
 from 5 show ?thesis by (simp add: yields_d_def)
qed

lemma ChopAndYieldsMP:
 "\<turnstile>   f; g \<and>\<^sub>i f yields (g\<supset>\<^sub>i g1) \<supset>\<^sub>i f; g1 "
proof -
 have  1: "\<turnstile> f; g \<and>\<^sub>i f yields (g\<supset>\<^sub>i g1) \<supset>\<^sub>i f; (g \<and>\<^sub>i (g \<supset>\<^sub>i g1)) " by (rule ChopAndYieldsImp)
 have  2: "\<turnstile> g \<and>\<^sub>i (g \<supset>\<^sub>i g1) \<supset>\<^sub>i g1 " by auto
 hence 3: "\<turnstile> f; (g \<and>\<^sub>i (g \<supset>\<^sub>i g1)) \<supset>\<^sub>i f; g1 " by (rule RightChopImpChop)
 from 1 3 show ?thesis by auto
qed

lemma OrYieldsImp:
  "\<turnstile>   (f \<or>\<^sub>i  f1) yields  g \<equiv>\<^sub>i (f yields  g) \<and>\<^sub>i (f1 yields  g) "
proof -
 have  1: "\<turnstile> ((f\<or>\<^sub>i  f1); \<not>\<^sub>i  g) \<equiv>\<^sub>i ((f; \<not>\<^sub>i  g) \<or>\<^sub>i  (f1; \<not>\<^sub>i  g)) " by (rule OrChopEqv)
 hence 2: "\<turnstile> \<not>\<^sub>i ((f\<or>\<^sub>i  f1); \<not>\<^sub>i  g) \<equiv>\<^sub>i \<not>\<^sub>i (f; \<not>\<^sub>i  g) \<and>\<^sub>i  \<not>\<^sub>i(f1; \<not>\<^sub>i  g) " by auto
 from 2 show ?thesis by (simp add: yields_d_def)
qed

lemma LeftYieldsImpYields:
 assumes "\<turnstile>   f\<supset>\<^sub>i f1 "
 shows   "\<turnstile> (f1 yields  g) \<supset>\<^sub>i (f yields  g) "
proof -
 have  1: "\<turnstile> f \<supset>\<^sub>i f1 " using assms by auto
 hence 2: "\<turnstile> f; \<not>\<^sub>i  g \<supset>\<^sub>i f1; \<not>\<^sub>i  g  " by (rule LeftChopImpChop)
 hence 3: "\<turnstile> \<not>\<^sub>i (f1; \<not>\<^sub>i  g) \<supset>\<^sub>i \<not>\<^sub>i (f; \<not>\<^sub>i  g) " by auto
 from 3 show ?thesis by (simp add: yields_d_def)
qed

lemma LeftYieldsEqvYields:
 assumes "\<turnstile>  f \<equiv>\<^sub>i f1 "
 shows "\<turnstile> (f yields  g) \<equiv>\<^sub>i (f1 yields  g) "
proof -
 have  1: "\<turnstile> f \<equiv>\<^sub>i f1 " using assms by auto
 hence 2: "\<turnstile> f; \<not>\<^sub>i  g \<equiv>\<^sub>i f1; \<not>\<^sub>i  g " by (rule LeftChopEqvChop)
 hence 3: "\<turnstile> \<not>\<^sub>i (f; \<not>\<^sub>i  g) \<equiv>\<^sub>i \<not>\<^sub>i (f1; \<not>\<^sub>i  g) " by auto
 from 3 show ?thesis by (simp add: yields_d_def)
qed

subsection \<open>Properties of Fin\<close>

lemma FinEqvTrueChopAndEmpty:
  " \<turnstile> fin f \<equiv>\<^sub>i true\<^sub>i;(f \<and>\<^sub>i empty)"
proof -
 have 1: "\<turnstile>fin f \<equiv>\<^sub>i \<box>(empty \<supset>\<^sub>i f)" by (simp add: fin_d_def)
 have 2: "\<turnstile> \<box>(empty \<supset>\<^sub>i f) \<equiv>\<^sub>i \<not>\<^sub>i(\<diamond>(\<not>\<^sub>i(empty \<supset>\<^sub>i f ) ) )" by (simp add: always_d_def)
 have 3: "\<turnstile> (\<not>\<^sub>i(empty \<supset>\<^sub>i f )) \<equiv>\<^sub>i (\<not>\<^sub>i f \<and>\<^sub>i empty)  " by auto
 hence 4: "\<turnstile> \<diamond>(\<not>\<^sub>i(empty \<supset>\<^sub>i f )) \<equiv>\<^sub>i \<diamond>(\<not>\<^sub>i f \<and>\<^sub>i empty) " using DiamondEqvDiamond by blast
 hence 5: "\<turnstile> \<not>\<^sub>i(\<diamond>(\<not>\<^sub>i(empty \<supset>\<^sub>i f ))) \<equiv>\<^sub>i \<not>\<^sub>i(\<diamond>(\<not>\<^sub>i f \<and>\<^sub>i empty))" by auto
 have 6: "\<turnstile> \<not>\<^sub>i(\<diamond>(\<not>\<^sub>i f \<and>\<^sub>i empty)) \<equiv>\<^sub>i true\<^sub>i;(f \<and>\<^sub>i empty)" using Finprop by auto
 from 1 2 5 6 show ?thesis by auto
qed


lemma DiamondFin:
 "\<turnstile> \<diamond>(fin w) \<equiv>\<^sub>i fin w"
by (metis DiamondDiamondEqvDiamond DiamondEqvDiamond FinEqvTrueChopAndEmpty 
          itl_prop(30) prop03 sometimes_d_def)

lemma ChopFinExportA:
 "\<turnstile> f;(g \<and>\<^sub>i fin w) \<supset>\<^sub>i fin w"
using DiamondFin by auto

lemma FinImpBox:
 "\<turnstile> fin w \<supset>\<^sub>i \<box>(fin w)" 
by (metis BoxImpBoxBox fin_d_def)

lemma FinAndChopImport:
 "\<turnstile> (fin w) \<and>\<^sub>i (f;g) \<supset>\<^sub>i f;((fin w) \<and>\<^sub>i g)"
proof -
 have 1: "\<turnstile> fin w \<supset>\<^sub>i \<box>(fin w)" by (rule FinImpBox)
 hence 2: "\<turnstile> fin w \<and>\<^sub>i f;g \<supset>\<^sub>i \<box>(fin w) \<and>\<^sub>i (f;g)" by auto
 have 3: "\<turnstile> \<box>(fin w) \<and>\<^sub>i (f;g) \<supset>\<^sub>i f;((fin w) \<and>\<^sub>i g)" using BoxAndChopImport by blast
 from 2 3 show ?thesis using MP by auto
qed

lemma FinAndChop:
 "\<turnstile> f;(g \<and>\<^sub>i fin w) \<equiv>\<^sub>i fin w \<and>\<^sub>i f;g  "
using FinAndChopImport ChopFinExportA ChopAndA ChopAndCommute itl_prop(31) itl_prop(32) prop15 
by blast 

lemma ChopAndEmptyEqvEmptyChopEmpty:
 "\<turnstile> ((f;g) \<and>\<^sub>i empty) \<equiv>\<^sub>i (f \<and>\<^sub>i empty);(g \<and>\<^sub>i empty) "
by auto

lemma FinAndEmpty:
 "\<turnstile> (fin w) \<and>\<^sub>i empty \<equiv>\<^sub>i w \<and>\<^sub>i empty"
proof -
 have 1: "\<turnstile> (fin w) \<and>\<^sub>i empty \<equiv>\<^sub>i true\<^sub>i;(w \<and>\<^sub>i empty) \<and>\<^sub>i empty" 
      using FinEqvTrueChopAndEmpty by auto
 have 2: "\<turnstile> true\<^sub>i;(w \<and>\<^sub>i empty) \<and>\<^sub>i empty \<equiv>\<^sub>i (true\<^sub>i \<and>\<^sub>i empty);(w \<and>\<^sub>i empty)" 
      using ChopAndEmptyEqvEmptyChopEmpty by auto
 have 3: "\<turnstile> (true\<^sub>i \<and>\<^sub>i empty);(w \<and>\<^sub>i empty) \<equiv>\<^sub>i empty;(w \<and>\<^sub>i empty)" 
      using LeftChopEqvChop itl_prop(17) by blast
 have 4: "\<turnstile> empty;(w \<and>\<^sub>i empty) \<equiv>\<^sub>i w \<and>\<^sub>i empty" 
      using EmptyChop by blast
 from 1 2 3 4 show ?thesis by auto
qed

lemma AndFinEqvChopAndEmpty:
 "\<turnstile>   f \<and>\<^sub>i  fin g \<equiv>\<^sub>i f; (g \<and>\<^sub>i  empty ) "
proof -
 have 1: "\<turnstile> f \<and>\<^sub>i  fin g \<equiv>\<^sub>i f;empty \<and>\<^sub>i fin g" using ChopEmpty itl_prop(30) prop06 by blast 
 have 2: "\<turnstile> fin g \<and>\<^sub>i f;empty \<equiv>\<^sub>i f;(empty \<and>\<^sub>i fin g)  " using FinAndChop using itl_prop(30) by blast
 have 3: "\<turnstile> empty \<and>\<^sub>i fin g \<equiv>\<^sub>i  fin g \<and>\<^sub>i empty" by auto
 have 4: "\<turnstile> fin g \<and>\<^sub>i empty \<equiv>\<^sub>i g \<and>\<^sub>i empty" using FinAndEmpty by auto
 have 5: "\<turnstile> empty \<and>\<^sub>i fin g \<equiv>\<^sub>i g \<and>\<^sub>i empty" using "3" "4" by auto
 hence 6: "\<turnstile> f;(empty \<and>\<^sub>i fin g) \<equiv>\<^sub>i f;(g \<and>\<^sub>i empty)" using RightChopEqvChop by blast
 from 1 2 5 show ?thesis by auto
qed

lemma AndFinEqvChopStateAndEmpty:
 "\<turnstile>   f \<and>\<^sub>i  fin  (init w) \<equiv>\<^sub>i f; ((init w) \<and>\<^sub>i  empty ) "
using AndFinEqvChopAndEmpty by blast

lemma FinStateEqvStateAndEmptyOrNextFinState:
 "\<turnstile>  fin  (init w) \<equiv>\<^sub>i ((init w) \<and>\<^sub>i  empty ) \<or>\<^sub>i  \<circle>( fin  (init w)) "
proof -
 have 1: "\<turnstile> fin (init w) \<equiv>\<^sub>i \<box>( empty \<supset>\<^sub>i init w)" 
      by (simp add: fin_d_def)
 have 2 : "\<turnstile> \<box>(empty \<supset>\<^sub>i init w) \<equiv>\<^sub>i 
               (empty \<supset>\<^sub>i init w) \<and>\<^sub>i wnext (\<box> (empty \<supset>\<^sub>i init w)) " 
      by (rule BoxEqvAndWnextBox)
 have 3: "\<turnstile> fin (init w) \<equiv>\<^sub>i  (empty \<supset>\<^sub>i init w) \<and>\<^sub>i wnext (fin (init w)) " 
      using "1" "2" by (simp add: fin_d_def) 
 have 4: "\<turnstile> wnext (fin (init w)) \<equiv>\<^sub>i empty \<or>\<^sub>i \<circle> (fin (init w)) " 
      by (rule WnextEqvEmptyOrNext)
 have 5: "\<turnstile> fin (init w) \<equiv>\<^sub>i  (empty \<supset>\<^sub>i init w) \<and>\<^sub>i (empty \<or>\<^sub>i \<circle> (fin (init w)))" 
      using "3" "4" by (simp add: fin_d_def)
 have 6: "\<turnstile> (empty \<supset>\<^sub>i init w) \<and>\<^sub>i (empty \<or>\<^sub>i \<circle> (fin (init w))) \<equiv>\<^sub>i
            ((empty \<supset>\<^sub>i init w) \<and>\<^sub>i empty) \<or>\<^sub>i ((empty \<supset>\<^sub>i init w) \<and>\<^sub>i \<circle> (fin (init w)))" 
      by auto
 have 7: "\<turnstile> (empty \<supset>\<^sub>i init w) \<and>\<^sub>i empty \<equiv>\<^sub>i (init w) \<and>\<^sub>i empty" 
      by auto
 have 8: "\<turnstile> (empty \<supset>\<^sub>i init w) \<and>\<^sub>i \<circle> (fin (init w)) \<equiv>\<^sub>i  \<circle> (fin (init w))" 
      by auto
 have 9: "\<turnstile> ((empty \<supset>\<^sub>i init w) \<and>\<^sub>i empty) \<or>\<^sub>i ((empty \<supset>\<^sub>i init w) \<and>\<^sub>i \<circle> (fin (init w))) \<equiv>\<^sub>i
            ((init w) \<and>\<^sub>i  empty ) \<or>\<^sub>i  \<circle>( fin  (init w))" 
      using "7" "8" by auto
 from 5 6 9 show ?thesis using prop03 by blast
qed

lemma FinChopEqvOr:
 " \<turnstile>   ( fin  (init w)); f \<equiv>\<^sub>i ((init w) \<and>\<^sub>i f) \<or>\<^sub>i  \<circle>(( fin  (init w)); f) "
proof -
 have  1: "\<turnstile>  fin  (init w) \<equiv>\<^sub>i ((init w) \<and>\<^sub>i  empty ) \<or>\<^sub>i  \<circle>( fin  (init w)) " 
       by (rule FinStateEqvStateAndEmptyOrNextFinState)
 hence 2: "\<turnstile> ( fin  (init w)); f \<equiv>\<^sub>i (((init w) \<and>\<^sub>i  empty )\<or>\<^sub>i  \<circle>( fin  (init w))); f" 
       by (rule LeftChopEqvChop)
 have  3: "\<turnstile> (((init w) \<and>\<^sub>i  empty )\<or>\<^sub>i  \<circle> (fin  (init w))); f
            \<equiv>\<^sub>i ((init w) \<and>\<^sub>i  empty ); f \<or>\<^sub>i  (\<circle> (fin  (init w))); f " 
       by (rule OrChopEqv)
 have  4: "\<turnstile> ((init w) \<and>\<^sub>i  empty ); f \<equiv>\<^sub>i (init w) \<and>\<^sub>i f " 
       by (rule StateAndEmptyChop)
 have  5: "\<turnstile> (\<circle> (fin  (init w))); f \<equiv>\<^sub>i \<circle>(( fin  (init w)); f) " 
       by (rule NextChop)
 from 2 3 4 5 show ?thesis by auto
qed

lemma FinChopEqvDiamond:
 "\<turnstile>   ( fin  (init w)); f \<equiv>\<^sub>i \<diamond> ((init w) \<and>\<^sub>i f) "
proof -
 have  1: "\<turnstile> ( fin  (init w)) \<equiv>\<^sub>i (true\<^sub>i;((init w) \<and>\<^sub>i empty))" 
       by (rule FinEqvTrueChopAndEmpty)
 hence 2: "\<turnstile> ( fin  (init w));f \<equiv>\<^sub>i (true\<^sub>i;((init w) \<and>\<^sub>i empty));f" 
       by (rule LeftChopEqvChop)
 have  3: "\<turnstile> true\<^sub>i;(( (init w) \<and>\<^sub>i empty);f)  \<equiv>\<^sub>i (true\<^sub>i;((init w) \<and>\<^sub>i empty));f " 
       by (rule ChopAssoc)
 have  4: "\<turnstile> true\<^sub>i;(( (init w) \<and>\<^sub>i empty);f)\<equiv>\<^sub>i \<diamond> ( ( (init w) \<and>\<^sub>i empty);f) " 
       by (simp add: sometimes_d_def)
 have  5: "\<turnstile> ( (init w) \<and>\<^sub>i empty);f \<equiv>\<^sub>i (init w) \<and>\<^sub>i f " 
       using StateAndEmptyChop by blast 
 hence 6: "\<turnstile> \<diamond> ( ( (init w) \<and>\<^sub>i empty);f) \<equiv>\<^sub>i \<diamond> ( (init w) \<and>\<^sub>i f) " 
       by (rule DiamondEqvDiamond)
 from 2 3 4 6 show ?thesis by simp
qed

lemma NotDiamondAndNot:
 "\<turnstile> \<not>\<^sub>i( \<diamond> ( f \<and>\<^sub>i \<not>\<^sub>i f))"
by simp

lemma FinYields:
 "\<turnstile>    ( fin  (init w))  yields  (init w) "
proof -
 have 1: "\<turnstile> (fin (init w)); \<not>\<^sub>i(init w) \<equiv>\<^sub>i \<diamond>((init w) \<and>\<^sub>i \<not>\<^sub>i(init w)) " by (rule FinChopEqvDiamond)
 have 2: "\<turnstile>  \<not>\<^sub>i( \<diamond>((init w) \<and>\<^sub>i \<not>\<^sub>i  (init w))) " by (rule NotDiamondAndNot)
 have 3: "\<turnstile> \<not>\<^sub>i (( fin  (init w)); \<not>\<^sub>i  (init w)) " using "1" "2" by auto
 from 3 show ?thesis by (simp add: yields_d_def)
qed

lemma ImpAndFinStateOrFinNotState:
 "\<turnstile> f \<supset>\<^sub>i (f \<and>\<^sub>i fin (init w)) \<or>\<^sub>i fin \<not>\<^sub>i (init w) "
by (simp)

lemma AndFinChopEqvStateAndChop:
 "\<turnstile>   (f \<and>\<^sub>i  fin  (init w)); g \<equiv>\<^sub>i f; ((init w) \<and>\<^sub>i g) "
proof -
 have   1: "\<turnstile> ( fin  (init w)) yields  (init w) " 
        by (rule FinYields)
 have   2: "\<turnstile> f \<and>\<^sub>i  fin  (init w) \<supset>\<^sub>i  fin  (init w) " 
        by auto
 hence  3: "\<turnstile> ( fin  (init w)) yields  (init w) \<supset>\<^sub>i (f \<and>\<^sub>i  fin  (init w)) yields  (init w) " 
        by (rule LeftYieldsImpYields)
 have   4: "\<turnstile> (f \<and>\<^sub>i  fin  (init w)) yields  (init w) " 
        using "1" "3" MP by auto
 have   5: "\<turnstile> (f \<and>\<^sub>i  fin  (init w)); g \<and>\<^sub>i (f \<and>\<^sub>i  fin  (init w)) yields  (init w)
              \<supset>\<^sub>i (f \<and>\<^sub>i  fin  (init w)); (g \<and>\<^sub>i (init w)) " 
        by (rule ChopAndYieldsImp)
 have   6: "\<turnstile> (f \<and>\<^sub>i  fin  (init w)); g \<supset>\<^sub>i (f \<and>\<^sub>i  fin  (init w)); (g \<and>\<^sub>i (init w)) " 
        using "4" "5" by auto
 have   7: "\<turnstile> (f \<and>\<^sub>i  fin  (init w)); (g \<and>\<^sub>i (init w)) \<supset>\<^sub>i f; (g \<and>\<^sub>i (init w)) " 
        by (rule AndChopA)
 have   8: "\<turnstile> g \<and>\<^sub>i (init w) \<supset>\<^sub>i (init w) \<and>\<^sub>i g " 
        by auto
 hence  9: "\<turnstile> f; (g \<and>\<^sub>i (init w)) \<supset>\<^sub>i f; ((init w) \<and>\<^sub>i g) " 
        by (rule RightChopImpChop)
 have  10: "\<turnstile> (f \<and>\<^sub>i  fin  (init w)); g \<supset>\<^sub>i f; ((init w) \<and>\<^sub>i g) " 
        using "6" "7" "9" by auto
 have  11: "\<turnstile> f \<supset>\<^sub>i (f \<and>\<^sub>i  fin  (init w)) \<or>\<^sub>i   fin \<not>\<^sub>i  (init w) " 
        by (rule ImpAndFinStateOrFinNotState)
 hence 12: "\<turnstile> f; ((init w) \<and>\<^sub>i g) \<supset>\<^sub>i
              ((f \<and>\<^sub>i  fin  (init w))\<or>\<^sub>i  fin \<not>\<^sub>i  (init w)); ((init w) \<and>\<^sub>i g) " 
        by (rule LeftChopImpChop)
 have  13: "\<turnstile> ((f \<and>\<^sub>i  fin  (init w))\<or>\<^sub>i  fin \<not>\<^sub>i  (init w)); ((init w) \<and>\<^sub>i g)
              \<equiv>\<^sub>i  
              (f \<and>\<^sub>i  fin  (init w)); ((init w) \<and>\<^sub>i g) \<or>\<^sub>i  ( fin \<not>\<^sub>i  (init w)); ((init w) \<and>\<^sub>i g) " 
        by (rule OrChopEqv)
 have  14: "\<turnstile> ( fin   (init (\<not>\<^sub>i w))); ((init w) \<and>\<^sub>i g) \<supset>\<^sub>i \<diamond>(  (init (\<not>\<^sub>i w)) \<and>\<^sub>i ((init w) \<and>\<^sub>i g)) " 
        using FinChopEqvDiamond itl_prop(31) by blast
 have 141: "\<turnstile> \<not>\<^sub>i( \<diamond>(  (init (\<not>\<^sub>i w)) \<and>\<^sub>i ((init w) \<and>\<^sub>i g))) \<supset>\<^sub>i 
              \<not>\<^sub>i ( ( fin   (init (\<not>\<^sub>i w))); ((init w) \<and>\<^sub>i g))" 
        using "14" by auto
 have  15: "\<turnstile> \<not>\<^sub>i( \<diamond>(  (init (\<not>\<^sub>i w)) \<and>\<^sub>i ((init w) \<and>\<^sub>i g))) " 
        using NotDiamondAndNot Initprop by simp
 have 151: "\<turnstile> \<not>\<^sub>i ( ( fin   (init (\<not>\<^sub>i w))); ((init w) \<and>\<^sub>i g))" 
        using "15" "141" by auto 
 have 152: "\<turnstile> (f \<and>\<^sub>i  fin  (init w)); ((init w) \<and>\<^sub>i g) \<or>\<^sub>i  ( fin \<not>\<^sub>i  (init w)); ((init w) \<and>\<^sub>i g) \<supset>\<^sub>i
              (f \<and>\<^sub>i  fin  (init w)); ((init w) \<and>\<^sub>i g)" 
        using "151" by auto
 have  16: "\<turnstile> f; ((init w) \<and>\<^sub>i g) \<supset>\<^sub>i (f \<and>\<^sub>i  fin  (init w)); ((init w) \<and>\<^sub>i g) " 
        using "12" "13" "152" by fastforce
 have  17: "\<turnstile> (f \<and>\<^sub>i  fin  (init w)); ((init w) \<and>\<^sub>i g) \<supset>\<^sub>i (f\<and>\<^sub>i  fin  (init w)); g " 
        by (rule ChopAndB)
 have  18: "\<turnstile> f; ((init w) \<and>\<^sub>i g) \<supset>\<^sub>i (f \<and>\<^sub>i  fin  (init w)); g " 
        using "16" "17" by auto
 from 10 18 show ?thesis by auto
qed

lemma DiAndFinEqvChopState:
 "\<turnstile>    di (f \<and>\<^sub>i  fin  (init w)) \<equiv>\<^sub>i f; (init w) "
proof -
 have  1: "\<turnstile> (f \<and>\<^sub>i fin(init w)); true\<^sub>i \<equiv>\<^sub>i f;((init w) \<and>\<^sub>i true\<^sub>i) " by (rule AndFinChopEqvStateAndChop)
 have  2: "\<turnstile> (init w) \<and>\<^sub>i true\<^sub>i \<equiv>\<^sub>i (init w) " by auto
 hence 3: "\<turnstile> f; ((init w) \<and>\<^sub>i true\<^sub>i) \<equiv>\<^sub>i f; (init w)" by (rule RightChopEqvChop)
 have  4: "\<turnstile> (f \<and>\<^sub>i  fin  (init w)); true\<^sub>i \<equiv>\<^sub>i f; (init w) " using "1" "3" by auto
 from 4 show ?thesis by (simp add: di_d_def)
qed

lemma FinNotStateEqvNotFinState:
 "\<turnstile> fin   (init \<not>\<^sub>i w) \<equiv>\<^sub>i \<not>\<^sub>i( fin   (init w))"
by (simp )

lemma BiImpFinEqvYieldsState:
 "\<turnstile>   bi (f \<supset>\<^sub>i  fin  (init w)) \<equiv>\<^sub>i f yields  (init w) "
proof -
 have  1: "\<turnstile>  di (f \<and>\<^sub>i fin (init \<not>\<^sub>i w)) \<equiv>\<^sub>i f;   (init \<not>\<^sub>i w) " by (rule DiAndFinEqvChopState)
 have  2: "\<turnstile> f \<and>\<^sub>i fin(init \<not>\<^sub>i w)  \<equiv>\<^sub>i f \<and>\<^sub>i \<not>\<^sub>i(fin(init w)) " using FinNotStateEqvNotFinState by auto
 have  3: "\<turnstile> f \<and>\<^sub>i \<not>\<^sub>i (fin(init w)) \<equiv>\<^sub>i \<not>\<^sub>i (f \<supset>\<^sub>i  fin  (init w)) " by auto
 have  4: "\<turnstile> f \<and>\<^sub>i fin(init  \<not>\<^sub>i w) \<equiv>\<^sub>i \<not>\<^sub>i (f \<supset>\<^sub>i  fin(init w))" using "2" "3" by(simp add: fin_d_def)
 hence 5: "\<turnstile>  di (f \<and>\<^sub>i  fin   (init \<not>\<^sub>i w)) \<equiv>\<^sub>i  di \<not>\<^sub>i (f \<supset>\<^sub>i fin(init w)) " by (rule DiEqvDi)
 have  6: "\<turnstile>  di \<not>\<^sub>i (f \<supset>\<^sub>i  fin  (init w)) \<equiv>\<^sub>i \<not>\<^sub>i( bi (f \<supset>\<^sub>i fin(init w)))" by (rule DiNotEqvNotBi)
 have  7: "\<turnstile> \<not>\<^sub>i (bi (f \<supset>\<^sub>i  fin  (init w))) \<equiv>\<^sub>i f;(init \<not>\<^sub>i w) " using "1"  "5" "6" Initprop by auto
 hence 8: "\<turnstile> bi (f \<supset>\<^sub>i  fin  (init w)) \<equiv>\<^sub>i \<not>\<^sub>i (f; \<not>\<^sub>i  (init w)) " by auto
 from 8 show ?thesis by (simp add: yields_d_def)
qed

lemma StateImpYields:
 assumes " \<turnstile>   (init w) \<and>\<^sub>i f \<supset>\<^sub>i  fin  (init w1) "
 shows   "\<turnstile> (init w) \<supset>\<^sub>i (f yields  (init w1)) "
proof -
 have  1: "\<turnstile> (init w) \<and>\<^sub>i f \<supset>\<^sub>i  fin  (init w1) " using assms by auto
 hence 2: "\<turnstile> (init w) \<supset>\<^sub>i (f \<supset>\<^sub>i  fin  (init w1)) " by auto
 hence 3: "\<turnstile> (init w) \<supset>\<^sub>i bi (f \<supset>\<^sub>i  fin  (init w1)) " by (rule StateImpBiGen)
 have  4: "\<turnstile> bi (f \<supset>\<^sub>i  fin  (init w1)) \<equiv>\<^sub>i f yields  (init w1) " by (rule BiImpFinEqvYieldsState)
 from 3 4 show ?thesis by auto
qed

lemma StateAndYieldsImpYields:
 assumes "\<turnstile>   (init w) \<and>\<^sub>i f \<supset>\<^sub>i f1 "
 shows   "\<turnstile> (init w) \<and>\<^sub>i (f1 yields  g) \<supset>\<^sub>i (f yields  g) "
proof -
 have  1: "\<turnstile> (init w) \<and>\<^sub>i f \<supset>\<^sub>i f1 " using assms by auto
 hence 2: "\<turnstile> (init w) \<and>\<^sub>i (f; \<not>\<^sub>i  g) \<supset>\<^sub>i f1; \<not>\<^sub>i  g " by (rule StateAndChopImpChopRule)
 hence 3: "\<turnstile> (init w) \<and>\<^sub>i \<not>\<^sub>i (f1; \<not>\<^sub>i  g) \<supset>\<^sub>i \<not>\<^sub>i (f; \<not>\<^sub>i  g) " by auto
 from 3 show ?thesis by (simp add: yields_d_def)
qed

lemma AndYieldsA:
 "\<turnstile>   f yields  g \<supset>\<^sub>i (f \<and>\<^sub>i f1) yields  g "
proof -
 have 1: "\<turnstile> f \<and>\<^sub>i f1 \<supset>\<^sub>i f " by auto
 from 1 show ?thesis by (rule LeftYieldsImpYields)
qed

lemma AndYieldsB:
 " \<turnstile>   f1 yields  g \<supset>\<^sub>i (f \<and>\<^sub>i f1) yields  g "
proof -
 have 1: "\<turnstile> f \<and>\<^sub>i f1 \<supset>\<^sub>i f1 " by auto
 from 1 show ?thesis by (rule LeftYieldsImpYields)
qed

lemma RightYieldsImpYields:
 assumes "\<turnstile>   g\<supset>\<^sub>i g1 "
 shows   "\<turnstile> (f yields  g) \<supset>\<^sub>i (f yields  g1) "
proof -
 have  1: "\<turnstile> g \<supset>\<^sub>i g1 " using assms by auto
 hence 2: " \<turnstile> \<not>\<^sub>i  g1 \<supset>\<^sub>i \<not>\<^sub>i  g" by auto
 hence 3: "\<turnstile> f; \<not>\<^sub>i  g1 \<supset>\<^sub>i f; \<not>\<^sub>i  g " by (rule RightChopImpChop)
 hence 4: "\<turnstile> \<not>\<^sub>i (f; \<not>\<^sub>i  g) \<supset>\<^sub>i \<not>\<^sub>i (f; \<not>\<^sub>i  g1) " by auto
 from 4 show ?thesis by (simp add: yields_d_def)
qed

lemma RightYieldsEqvYields:
 assumes "\<turnstile>   g\<equiv>\<^sub>i g1 "
 shows   "\<turnstile> (f yields  g) \<equiv>\<^sub>i (f yields  g1) "
proof -
 have  1: "\<turnstile> g \<equiv>\<^sub>i g1 " using assms by auto
 hence 2: "\<turnstile> \<not>\<^sub>i  g \<equiv>\<^sub>i \<not>\<^sub>i  g1 " by auto
 hence 3: "\<turnstile> f; \<not>\<^sub>i  g \<equiv>\<^sub>i f; \<not>\<^sub>i  g1 " by (rule RightChopEqvChop)
 hence 4: "\<turnstile> \<not>\<^sub>i (f; \<not>\<^sub>i  g) \<equiv>\<^sub>i \<not>\<^sub>i (f; \<not>\<^sub>i  g1) " by auto
 from 4 show ?thesis by (simp add: yields_d_def)
qed

lemma BoxImpYields:
 "\<turnstile>   \<box> g \<supset>\<^sub>i f yields  g "
proof -
 have  1: "\<turnstile> f; \<not>\<^sub>i  g \<supset>\<^sub>i \<diamond>\<not>\<^sub>i  g " by (rule ChopImpDiamond)
 hence 2: "\<turnstile> \<not>\<^sub>i (\<diamond>\<not>\<^sub>i  g) \<supset>\<^sub>i \<not>\<^sub>i (f; \<not>\<^sub>i  g) " by auto
 from 2 show ?thesis by (simp add: yields_d_def)
qed

lemma BoxEqvTrueYields:
 "\<turnstile>   \<box> f \<equiv>\<^sub>i true\<^sub>i yields  f "
proof -
 have  1: "\<turnstile> true\<^sub>i; \<not>\<^sub>i  f \<equiv>\<^sub>i \<diamond> \<not>\<^sub>i  f " by (rule TrueChopEqvDiamond)
 hence 2: "\<turnstile> \<not>\<^sub>i (true\<^sub>i; \<not>\<^sub>i  f) \<equiv>\<^sub>i \<not>\<^sub>i( \<diamond> \<not>\<^sub>i  f) " by auto
 have  3: "\<turnstile> \<box> f \<equiv>\<^sub>i \<not>\<^sub>i ( \<diamond> \<not>\<^sub>i  f) " by (simp add: always_d_def)
 have  4: "\<turnstile> \<box> f \<equiv>\<^sub>i \<not>\<^sub>i (true\<^sub>i; \<not>\<^sub>i  f) " using "2" "3" by auto
 from 4 show ?thesis by (simp add: yields_d_def)
qed

lemma YieldsGen:
 assumes "\<turnstile>   g "
 shows   "\<turnstile> f yields  g "
proof -
  have  1: "\<turnstile> g " using assms by auto
  hence 2: "\<turnstile> \<box> g " by (rule BoxGen)
  have  3: "\<turnstile> \<box> g \<supset>\<^sub>i f yields  g " by (rule BoxImpYields)
  from 2 3 show ?thesis using MP by auto
qed

lemma YieldsAndYieldsEqvYieldsAnd:
  "\<turnstile>   (f yields  g) \<and>\<^sub>i (f  yields  g1) \<equiv>\<^sub>i f yields (g \<and>\<^sub>i g1) "
proof -
 have  1: "\<turnstile> f; (\<not>\<^sub>i  g \<or>\<^sub>i  \<not>\<^sub>i  g1) \<equiv>\<^sub>i (f; \<not>\<^sub>i  g) \<or>\<^sub>i  (f; \<not>\<^sub>i  g1) " by (rule ChopOrEqv)
 hence 2: "\<turnstile> (f; \<not>\<^sub>i  g) \<or>\<^sub>i  (f; \<not>\<^sub>i  g1) \<equiv>\<^sub>i f; (\<not>\<^sub>i  g \<or>\<^sub>i  \<not>\<^sub>i  g1) " by auto
 have  3: "\<turnstile> \<not>\<^sub>i  g \<or>\<^sub>i  \<not>\<^sub>i  g1  \<equiv>\<^sub>i  \<not>\<^sub>i (g \<and>\<^sub>i g1) " by auto
 hence 4: "\<turnstile> f; (\<not>\<^sub>i  g \<or>\<^sub>i  \<not>\<^sub>i  g1)  \<equiv>\<^sub>i  f; \<not>\<^sub>i (g \<and>\<^sub>i g1) " by (rule RightChopEqvChop)
 have  5: "\<turnstile> (f; \<not>\<^sub>i  g) \<or>\<^sub>i  (f; \<not>\<^sub>i  g1) \<equiv>\<^sub>i f; \<not>\<^sub>i (g \<and>\<^sub>i g1) " using "2" "4" by auto
 hence 6: "\<turnstile> \<not>\<^sub>i (f; \<not>\<^sub>i  g) \<and>\<^sub>i \<not>\<^sub>i (f; \<not>\<^sub>i  g1) \<equiv>\<^sub>i \<not>\<^sub>i (f; \<not>\<^sub>i (g \<and>\<^sub>i g1)) " by auto
 from 6 show ?thesis by (simp add: yields_d_def)
qed

lemma YieldsAndYieldsImpAndYieldsAnd:
 "\<turnstile>   (f yields  g) \<and>\<^sub>i (f1 yields  g1) \<supset>\<^sub>i (f\<and>\<^sub>i f1) yields (g\<and>\<^sub>i g1) "
proof -
 have 1: "\<turnstile> f yields  g \<supset>\<^sub>i (f \<and>\<^sub>i f1) yields  g" 
      by (rule AndYieldsA)
 have 2: "\<turnstile> f1 yields  g1 \<supset>\<^sub>i (f \<and>\<^sub>i f1) yields  g1 " 
      by (rule AndYieldsB)
 have 3: "\<turnstile> (f \<and>\<^sub>i f1) yields  g \<and>\<^sub>i (f \<and>\<^sub>i f1) yields  g1 \<equiv>\<^sub>i (f \<and>\<^sub>i f1) yields  (g \<and>\<^sub>i g1) " 
      by (rule YieldsAndYieldsEqvYieldsAnd)
 from 1 2 3 show ?thesis by auto
qed

lemma YieldsYieldsEqvChopYields:
 "\<turnstile>   f yields (g yields  h) \<equiv>\<^sub>i (f; g) yields h  " 
proof -
 have  1: "\<turnstile> f; (g; \<not>\<^sub>i  h) \<equiv>\<^sub>i (f; g); \<not>\<^sub>i  h   " by (rule ChopAssoc)
 hence 2: "\<turnstile> f; (g; \<not>\<^sub>i  h) \<equiv>\<^sub>i (f; g); \<not>\<^sub>i  h " by auto
 have  3: "\<turnstile> g; \<not>\<^sub>i  h \<equiv>\<^sub>i \<not>\<^sub>i \<not>\<^sub>i (g; \<not>\<^sub>i  h) " by auto
 hence 4: "\<turnstile> f; (g; \<not>\<^sub>i  h) \<equiv>\<^sub>i f; \<not>\<^sub>i \<not>\<^sub>i (g; \<not>\<^sub>i  h) " by (rule RightChopEqvChop)
 have  5: "\<turnstile> f; \<not>\<^sub>i \<not>\<^sub>i (g; \<not>\<^sub>i  h) \<equiv>\<^sub>i (f; g); \<not>\<^sub>i  h " using "2" "4" by auto
 hence 6: "\<turnstile> f; \<not>\<^sub>i (g yields  h) \<equiv>\<^sub>i (f; g); \<not>\<^sub>i  h " by (simp add: yields_d_def)
 hence 7: "\<turnstile> \<not>\<^sub>i (f; \<not>\<^sub>i (g yields  h)) \<equiv>\<^sub>i \<not>\<^sub>i ((f; g); \<not>\<^sub>i  h) " by auto
 from 7 show ?thesis by (simp add: yields_d_def)
qed

lemma EmptyYields:
 "\<turnstile>    empty   yields  f \<equiv>\<^sub>i f "
proof -
 have  1: "\<turnstile>  empty ; \<not>\<^sub>i  f \<equiv>\<^sub>i \<not>\<^sub>i  f " by (rule EmptyChop)
 hence 2: "\<turnstile> \<not>\<^sub>i ( empty ; \<not>\<^sub>i  f) \<equiv>\<^sub>i f " by auto
 from 2 show ?thesis by (simp add: yields_d_def)
qed

lemma NextYields:
 "\<turnstile>   (\<circle> f) yields  g \<equiv>\<^sub>i wnext (f yields  g) "
proof -
 have  1: "\<turnstile> (\<circle> f); \<not>\<^sub>i  g \<equiv>\<^sub>i \<circle>(f; \<not>\<^sub>i  g) " by (rule NextChop)
 hence 2: "\<turnstile> \<not>\<^sub>i ((\<circle> f); \<not>\<^sub>i  g) \<equiv>\<^sub>i \<not>\<^sub>i (\<circle>(f; \<not>\<^sub>i  g)) " by auto
 hence 3: "\<turnstile> (\<circle> f) yields  g \<equiv>\<^sub>i \<not>\<^sub>i (\<circle>(f; \<not>\<^sub>i  g)) " by (simp add: yields_d_def)
 have  4: "\<turnstile> \<not>\<^sub>i( \<circle>(f; \<not>\<^sub>i  g)) \<equiv>\<^sub>i wnext \<not>\<^sub>i (f; \<not>\<^sub>i  g) " by auto
 have  5: "\<turnstile> (\<circle> f) yields  g \<equiv>\<^sub>i wnext \<not>\<^sub>i (f; \<not>\<^sub>i  g) " using "3" "4" by auto
 from 5 show ?thesis by (simp add: yields_d_def)
qed

lemma SkipChopEqvNext:
 " \<turnstile>    skip ; f \<equiv>\<^sub>i \<circle> f "
by (simp add: next_d_def)

lemma SkipYieldsEqvWeakNext:
 "\<turnstile>    skip  yields  f \<equiv>\<^sub>i wnext  f "
proof -
 have  1: "\<turnstile>  skip ; \<not>\<^sub>i  f \<equiv>\<^sub>i \<circle>\<not>\<^sub>i  f " by (rule SkipChopEqvNext)
 hence 2: "\<turnstile> \<not>\<^sub>i ( skip ; \<not>\<^sub>i  f) \<equiv>\<^sub>i \<not>\<^sub>i( \<circle>\<not>\<^sub>i  f) " by auto
 have  3: "\<turnstile> \<not>\<^sub>i (\<circle>\<not>\<^sub>i  f) \<equiv>\<^sub>i wnext  f " by auto
 have  4: "\<turnstile> \<not>\<^sub>i ( skip ; \<not>\<^sub>i  f) \<equiv>\<^sub>i wnext  f " using "2" "3" by auto
 from 4 show ?thesis by (simp add: yields_d_def)
qed

lemma NextImpSkipYields:
 "\<turnstile>   \<circle> f \<supset>\<^sub>i  skip  yields  f "
proof -
 have 1: "\<turnstile> \<circle> f \<supset>\<^sub>i wnext  f " by auto
 have 2: "\<turnstile>  skip  yields  f \<equiv>\<^sub>i wnext  f " by (rule SkipYieldsEqvWeakNext)
 from 1 2 show ?thesis by auto
qed

lemma MoreEqvSkipChopTrue:
 "\<turnstile>    more  \<equiv>\<^sub>i  skip ; true\<^sub>i"
proof -
 have  1: "\<turnstile>  skip ; true\<^sub>i \<equiv>\<^sub>i \<circle>true\<^sub>i " by (rule SkipChopEqvNext)
 hence 2: "\<turnstile> \<circle>true\<^sub>i \<equiv>\<^sub>i  skip ; true\<^sub>i " by auto
 from 2 show ?thesis by (simp add: more_d_def)
qed

lemma MoreChopImpMore:
 "\<turnstile>    more ; f \<supset>\<^sub>i  more "
proof -
 have 1: "\<turnstile> (\<circle>true\<^sub>i); f \<equiv>\<^sub>i \<circle>(true\<^sub>i; f)" by (rule NextChop)
 have 2: "\<turnstile> \<circle>(true\<^sub>i; f) \<supset>\<^sub>i  more " by auto
 have 3: "\<turnstile> (\<circle>true\<^sub>i; f) \<supset>\<^sub>i  more " using "1" "2" by auto
 from 3 show ?thesis by (metis more_d_def)
qed

lemma ChopMoreImpMore:
 "\<turnstile>   f;  more  \<supset>\<^sub>i  more  "
proof -
 have 1: "\<turnstile> f;  more  \<supset>\<^sub>i \<diamond> more " by (rule ChopImpDiamond)
 have 2: "\<turnstile> \<diamond> more  \<supset>\<^sub>i  more " by auto
 from 1 2 show ?thesis by auto
qed

lemma MoreChopEqvNextDiamond:
 "\<turnstile>    more ; f \<equiv>\<^sub>i \<circle>(\<diamond> f) "
proof -
 have 1: "\<turnstile>  more ; f \<equiv>\<^sub>i (\<circle> true\<^sub>i); f " by (simp add: more_d_def)
 have 2: "\<turnstile> (\<circle>true\<^sub>i); f \<equiv>\<^sub>i \<circle>(true\<^sub>i; f) " by (rule NextChop)
 have 3: "\<turnstile>  more ; f \<equiv>\<^sub>i \<circle>(true\<^sub>i; f) " using "1" "2" by auto
 from 3 show ?thesis by (simp add: sometimes_d_def)
qed

lemma WeakNextBoxImpMoreYields:
 "\<turnstile>    more  yields  f \<equiv>\<^sub>i wnext( \<box> f) "
proof - 
 have 1: "\<turnstile> more ; \<not>\<^sub>i f \<equiv>\<^sub>i \<circle>(\<diamond> \<not>\<^sub>if) " by (rule MoreChopEqvNextDiamond)
 have 2: "\<turnstile> \<circle>(\<diamond> \<not>\<^sub>if) \<equiv>\<^sub>i \<circle>(\<not>\<^sub>i(\<box>f))" by auto
 have 3: "\<turnstile> \<circle>(\<not>\<^sub>i(\<box>f)) \<equiv>\<^sub>i \<not>\<^sub>i ( wnext( \<box> f) )" by auto
 have 4: "\<turnstile> more ; \<not>\<^sub>i f  \<equiv>\<^sub>i \<not>\<^sub>i(more  yields  f)" by (metis itl_prop(30) itl_prop(4) yields_d_def) 
 from 1 2 3 4 show ?thesis by (simp add: yields_d_def)
qed

lemma NotEqvYieldsMore:
 "\<turnstile>   \<not>\<^sub>i  f \<equiv>\<^sub>i f yields  more  "
proof -
 have  1: "\<turnstile> f;  empty  \<equiv>\<^sub>i f " by (rule ChopEmpty)
 hence 2: "\<turnstile> \<not>\<^sub>i (f;  empty ) \<equiv>\<^sub>i \<not>\<^sub>i  f " by auto
 have  3: "\<turnstile>  empty  \<equiv>\<^sub>i \<not>\<^sub>i  more " by auto
 hence 4: "\<turnstile> f;  empty  \<equiv>\<^sub>i f; \<not>\<^sub>i  more " by (rule RightChopEqvChop)
 hence 5: "\<turnstile> \<not>\<^sub>i (f;  empty ) \<equiv>\<^sub>i \<not>\<^sub>i (f; \<not>\<^sub>i  more ) " by auto
 have  6: "\<turnstile> \<not>\<^sub>i  f \<equiv>\<^sub>i \<not>\<^sub>i (f; \<not>\<^sub>i  more ) " using "2" "5" by auto
 from 6 show ?thesis by (metis yields_d_def)
qed

lemma LeftChopImpMoreRule:
 assumes "\<turnstile>   f \<supset>\<^sub>i  more "
 shows "\<turnstile> f; g \<supset>\<^sub>i  more "
proof -
 have  1: "\<turnstile> f \<supset>\<^sub>i  more " using assms by auto
 hence 2: "\<turnstile> f; g \<supset>\<^sub>i  more ; g " by (rule LeftChopImpChop)
 have 3: " \<turnstile>  more ; g \<supset>\<^sub>i  more " by (rule MoreChopImpMore)
 from 2 3 show ?thesis using prop02 by blast
qed

lemma RightChopImpMoreRule:
 assumes "\<turnstile>   g \<supset>\<^sub>i  more "
 shows   "\<turnstile> f; g \<supset>\<^sub>i  more "
proof -
 have  1: "\<turnstile> g \<supset>\<^sub>i  more " using assms by auto
 hence 2: "\<turnstile> f; g \<supset>\<^sub>i f;  more " by (rule RightChopImpChop)
 have  3: "\<turnstile> f;  more  \<supset>\<^sub>i  more " by (rule ChopMoreImpMore)
 from 2 3 show ?thesis using prop02 by blast
qed

lemma NotDiEqvBiNot:
 "\<turnstile>   \<not>\<^sub>i ( di  f) \<equiv>\<^sub>i bi (\<not>\<^sub>i  f) "
proof -
 have  1: "\<turnstile> f \<equiv>\<^sub>i \<not>\<^sub>i \<not>\<^sub>i  f " by auto
 hence 2: "\<turnstile>  di  f \<equiv>\<^sub>i  di \<not>\<^sub>i \<not>\<^sub>i  f" by (rule DiEqvDi)
 hence 3: "\<turnstile> \<not>\<^sub>i  ( di  f) \<equiv>\<^sub>i \<not>\<^sub>i ( di \<not>\<^sub>i \<not>\<^sub>i  f)" by auto
 from 3 show ?thesis by (simp add: bi_d_def)
qed

lemma ChopImpDi:
 "\<turnstile>   f; g \<supset>\<^sub>i  di  f "
proof -
 have  1: "\<turnstile> g \<supset>\<^sub>i true\<^sub>i " by auto
 hence 2: "\<turnstile> f; g \<supset>\<^sub>i f; true\<^sub>i " by (rule RightChopImpChop)
 from 2 show ?thesis by (simp add: bi_d_def)
qed

lemma TrueEqvTrueChopTrue:
 "\<turnstile>   true\<^sub>i \<equiv>\<^sub>i true\<^sub>i; true\<^sub>i "
proof -
 have  1: "\<turnstile> true\<^sub>i; true\<^sub>i \<supset>\<^sub>i true\<^sub>i" by auto
 have  2: "\<turnstile> true\<^sub>i \<supset>\<^sub>i  di true\<^sub>i " by (rule DiIntro)
 hence 3: "\<turnstile> true\<^sub>i \<supset>\<^sub>i true\<^sub>i; true\<^sub>i " by (simp add: di_d_def)
 from 1 3 show ?thesis by auto
qed

lemma DiEqvDiDi:
 "\<turnstile>    di  f \<equiv>\<^sub>i  di (  di  f) "
proof -
 have  1: "\<turnstile> true\<^sub>i \<equiv>\<^sub>i true\<^sub>i; true\<^sub>i " by (rule TrueEqvTrueChopTrue)
 hence 2: "\<turnstile> f; true\<^sub>i \<equiv>\<^sub>i f; (true\<^sub>i; true\<^sub>i) " by (rule RightChopEqvChop)
 have  3: "\<turnstile> f; (true\<^sub>i; true\<^sub>i)\<equiv>\<^sub>i (f; true\<^sub>i); true\<^sub>i   " by (rule ChopAssoc)
 have  4: "\<turnstile> f; true\<^sub>i \<equiv>\<^sub>i (f; true\<^sub>i); true\<^sub>i " using "2" "3"  using prop03 by blast 
 from 4 show ?thesis by (metis di_d_def)
qed

lemma BiEqvBiBi:
 "\<turnstile>   bi  f \<equiv>\<^sub>i bi( bi  f) "
proof -
 have  1: "\<turnstile>  di \<not>\<^sub>i  f \<equiv>\<^sub>i  di(  di \<not>\<^sub>i  f) " by (rule DiEqvDiDi)
 have  2: "\<turnstile>  di \<not>\<^sub>i  f \<equiv>\<^sub>i \<not>\<^sub>i ( bi  f) " by (rule DiNotEqvNotBi)
 hence 3: "\<turnstile>  di ( di \<not>\<^sub>i  f) \<equiv>\<^sub>i  di \<not>\<^sub>i ( bi  f) " by (rule DiEqvDi)
 have  4: "\<turnstile>  di \<not>\<^sub>i  f \<equiv>\<^sub>i  di \<not>\<^sub>i(  bi  f) " using "1" "3"  using prop03 by blast  
 hence 5: "\<turnstile> \<not>\<^sub>i ( di \<not>\<^sub>i  f) \<equiv>\<^sub>i \<not>\<^sub>i ( di \<not>\<^sub>i ( bi  f)) " using itl_prop(33) by blast
 from 5 show ?thesis by (metis bi_d_def)
qed

lemma DiOrEqv:
 "\<turnstile>    di (f \<or>\<^sub>i  g) \<equiv>\<^sub>i  di  f \<or>\<^sub>i   di  g " 
proof -
 have 1: "\<turnstile> (f\<or>\<^sub>i  g); true\<^sub>i \<equiv>\<^sub>i f; true\<^sub>i \<or>\<^sub>i  g; true\<^sub>i " by (rule OrChopEqv)
 from 1 show ?thesis by (simp add: di_d_def)
qed

lemma DiAndA:
 "\<turnstile>    di (f \<and>\<^sub>i g) \<supset>\<^sub>i  di  f "
proof -
 have 1: "\<turnstile> (f \<and>\<^sub>i g); true\<^sub>i \<supset>\<^sub>i f; true\<^sub>i " by (rule AndChopA)
 from 1 show ?thesis by (simp add: di_d_def)
qed

lemma DiAndB:
 "\<turnstile>    di (f \<and>\<^sub>i g) \<supset>\<^sub>i  di  g "
proof -
 have 1: "\<turnstile> (f \<and>\<^sub>i g); true\<^sub>i \<supset>\<^sub>i g; true\<^sub>i " by (rule AndChopB)
 from 1 show ?thesis by (simp add: di_d_def)
qed

lemma DiAndImpAnd:
 "\<turnstile>    di (f \<and>\<^sub>i g) \<supset>\<^sub>i  di  f \<and>\<^sub>i  di  g "  
proof -
 have 1: "\<turnstile>  di (f \<and>\<^sub>i g) \<supset>\<^sub>i  di  f " by (rule DiAndA)
 have 2: "\<turnstile>  di (f \<and>\<^sub>i g) \<supset>\<^sub>i  di  g " by (rule DiAndB)
 from 1 2 show ?thesis by auto
qed

lemma DiSkipEqvMore:
 "\<turnstile>    di  skip  \<equiv>\<^sub>i  more "
proof -
 have 1: "\<turnstile>  skip ; true\<^sub>i \<equiv>\<^sub>i \<circle>true\<^sub>i " by (rule SkipChopEqvNext)
 have 2: "\<turnstile> \<circle>true\<^sub>i \<equiv>\<^sub>i  more " by auto
 have 3: "\<turnstile>  skip ; true\<^sub>i \<equiv>\<^sub>i  more " using "1" "2" by auto
 from 3 show ?thesis by (simp add: di_d_def)
qed

lemma DiMoreEqvMore:
 "\<turnstile>    di  more  \<equiv>\<^sub>i  more "
proof -
 have  1: "\<turnstile>  di (\<circle> true\<^sub>i ) \<equiv>\<^sub>i \<circle>( di true\<^sub>i) " by (rule DiNext)
 have  2: "\<turnstile> \<circle>( di  true\<^sub>i)  \<supset>\<^sub>i  more " by auto
 have  3: "\<turnstile>  di( \<circle> true\<^sub>i)  \<supset>\<^sub>i  more " using "1" "2" by auto
 hence 4: "\<turnstile>  di  more  \<supset>\<^sub>i  more " by (simp add: more_d_def)
 have  5: "\<turnstile>  more  \<supset>\<^sub>i  di  more " by (rule ImpDi)
 from 4 5 show ?thesis by auto
qed

lemma DiIfEqvRule:
 assumes "\<turnstile>   f \<equiv>\<^sub>i  if\<^sub>i  (init w)  then  g  else  h "
 shows   "\<turnstile>  di  f \<equiv>\<^sub>i  if\<^sub>i  (init w)  then  ( di  g)  else   (di  h) "
proof -
 have  1: "\<turnstile> f \<equiv>\<^sub>i  if\<^sub>i  (init w)  then  g  else  h " using assms by auto
 hence 2: "\<turnstile> f; true\<^sub>i \<equiv>\<^sub>i  if\<^sub>i (init w) then (g; true\<^sub>i) else (h; true\<^sub>i) " by (rule IfChopEqvRule)
 from 2 show ?thesis by (simp add: di_d_def)
qed

lemma DiEmpty:
 "\<turnstile>    di  empty "
proof -
 have 1: "\<turnstile> true\<^sub>i  " by auto
 have 2: "\<turnstile>  empty ; true\<^sub>i \<equiv>\<^sub>i true\<^sub>i " by (rule EmptyChop) 
 have 3: "\<turnstile>  empty ; true\<^sub>i " using "1" "2" by auto
 from 3 show ?thesis by (simp add: di_d_def)
qed

lemma DaNotEqvNotBa:
 "\<turnstile>    da \<not>\<^sub>i  f \<equiv>\<^sub>i \<not>\<^sub>i ( ba  f) "
proof -
 have 1: "\<turnstile>  ba  f \<equiv>\<^sub>i \<not>\<^sub>i ( da \<not>\<^sub>i  f) " by (simp add: ba_d_def)
 from 1 show ?thesis by simp
qed


lemma DaEqvDa:
 assumes "\<turnstile>   f \<equiv>\<^sub>i g "
 shows "\<turnstile> da  f \<equiv>\<^sub>i  da  g "
using assms by auto

lemma DaEqvNotBaNot:
 "\<turnstile>    da  f \<equiv>\<^sub>i \<not>\<^sub>i ( ba \<not>\<^sub>i  f) "
proof -
 have  1: "\<turnstile>  ba \<not>\<^sub>i  f \<equiv>\<^sub>i \<not>\<^sub>i ( da \<not>\<^sub>i \<not>\<^sub>i  f)" by (simp add: ba_d_def)
 hence 2: "\<turnstile>  da \<not>\<^sub>i \<not>\<^sub>i  f \<equiv>\<^sub>i \<not>\<^sub>i(  ba \<not>\<^sub>i  f) " by simp
 have  3: "\<turnstile> f \<equiv>\<^sub>i \<not>\<^sub>i \<not>\<^sub>i  f " by simp
 hence 4: "\<turnstile>  da  f \<equiv>\<^sub>i  da  \<not>\<^sub>i \<not>\<^sub>i  f " by (rule DaEqvDa)
 from 2 4 show ?thesis by simp
qed

lemma BaElim:
 "\<turnstile>    ba  f \<supset>\<^sub>i f "
proof -
 have  1: "\<turnstile>  ba  f \<equiv>\<^sub>i \<box>(bi  f) " by (rule BaEqvBtBi)
 have  2: "\<turnstile> bi  f \<supset>\<^sub>i f " by (rule BiElim)
 hence 3: "\<turnstile> \<box>(bi  f \<supset>\<^sub>i f) " by (rule BoxGen)
 have  4: "\<turnstile> \<box>(bi  f \<supset>\<^sub>i f) \<supset>\<^sub>i \<box>(bi  f) \<supset>\<^sub>i \<box> f " by (rule BoxImpDist)
 have  5: "\<turnstile> \<box>(bi  f) \<supset>\<^sub>i \<box> f " using "3" "4" MP by simp
 have  6: "\<turnstile> \<box> f \<supset>\<^sub>i f " by (rule BoxElim)
 from 1 5 6 show ?thesis  using BaImpBt prop02 by blast 
qed

lemma DaIntro:
 "\<turnstile>   f \<supset>\<^sub>i  da  f "
proof -
 have  1: "\<turnstile>  ba  \<not>\<^sub>i  f \<supset>\<^sub>i \<not>\<^sub>i  f "  by (rule BaElim)
 hence 2: "\<turnstile> \<not>\<^sub>i \<not>\<^sub>i  f \<supset>\<^sub>i \<not>\<^sub>i (  ba  \<not>\<^sub>i  f) " using prop27 by blast 
 have  3: "\<turnstile> f \<equiv>\<^sub>i \<not>\<^sub>i \<not>\<^sub>i  f " by simp
 have  4: "\<turnstile>  da  f \<equiv>\<^sub>i \<not>\<^sub>i  ( ba  \<not>\<^sub>i  f) " by (rule DaEqvNotBaNot)
 from 2 3 4 show ?thesis by simp
qed

lemma BaGen:
 assumes "\<turnstile>   f "
 shows   "\<turnstile>  ba  f "
proof -
 have  1: "\<turnstile>   f " using assms by auto
 hence 2: "\<turnstile> \<box> f " by (rule BoxGen)
 hence 3: "\<turnstile> bi( \<box> f) " by (rule BiGen)
 have  4: "\<turnstile>  ba  f \<equiv>\<^sub>i bi (\<box> f) " by (rule BaEqvBiBt)
 from 3 4 show ?thesis by simp
qed

lemma BaImpDist:
 "\<turnstile>  ba (f \<supset>\<^sub>i g) \<supset>\<^sub>i  ba  f \<supset>\<^sub>i  ba  g "
proof -
 have  1: "\<turnstile> bi (f \<supset>\<^sub>i g) \<supset>\<^sub>i (bi  f \<supset>\<^sub>i bi  g) " by (rule BiImpDist)
 hence 2: "\<turnstile> \<box>(bi (f \<supset>\<^sub>i g) \<supset>\<^sub>i (bi  f \<supset>\<^sub>i bi  g)) " by (rule BoxGen)
 have  3: "\<turnstile> \<box>(bi (f \<supset>\<^sub>i g) \<supset>\<^sub>i (bi  f \<supset>\<^sub>i bi  g))
            \<supset>\<^sub>i  
            (\<box> (bi (f \<supset>\<^sub>i g)) \<supset>\<^sub>i (\<box>(bi  f) \<supset>\<^sub>i \<box>(bi  g))) " by simp
 have  4: "\<turnstile> \<box>(bi (f \<supset>\<^sub>i g)) \<supset>\<^sub>i (\<box>(bi  f) \<supset>\<^sub>i \<box>(bi  g)) " using "2" "3" MP by simp
 have  5: "\<turnstile>  ba (f\<supset>\<^sub>i g) \<equiv>\<^sub>i \<box>(bi (f \<supset>\<^sub>i g)) " by (rule BaEqvBtBi)
 have  6: "\<turnstile>  ba  f \<equiv>\<^sub>i \<box>(bi  f) " by (rule BaEqvBtBi)
 have  7: "\<turnstile>  ba  g \<equiv>\<^sub>i \<box>(bi  g) " by (rule BaEqvBtBi)
 from 4 5 6 7 show ?thesis by simp
qed

lemma BaAndEqv:
 "\<turnstile>    ba (f \<and>\<^sub>i g) \<equiv>\<^sub>i  ba  f \<and>\<^sub>i  ba  g "
proof -
 have  1: "\<turnstile>    ba (f \<and>\<^sub>i g) \<equiv>\<^sub>i   \<box>(bi (f \<and>\<^sub>i g)) " by (rule BaEqvBtBi)
 have  2: "\<turnstile>  bi (f \<and>\<^sub>i g) \<equiv>\<^sub>i bi f \<and>\<^sub>i bi g " by auto
 hence 3: "\<turnstile> \<box>(bi (f \<and>\<^sub>i g)) \<equiv>\<^sub>i \<box>(bi f \<and>\<^sub>i bi g) " by auto
 have  4: "\<turnstile> \<box>(bi f \<and>\<^sub>i bi g)\<equiv>\<^sub>i \<box>(bi f)  \<and>\<^sub>i \<box>(bi g) " by auto
 have  5: "\<turnstile>ba  f \<equiv>\<^sub>i \<box>(bi  f) " by (rule BaEqvBtBi)
 have  6: "\<turnstile>  ba  g \<equiv>\<^sub>i \<box>(bi  g) " by  (rule BaEqvBtBi)
 from 1 3 4 5 6 show ?thesis by  auto
qed

lemma BaImpBaEqvBa:
 "\<turnstile>   ba (f \<equiv>\<^sub>i g) \<supset>\<^sub>i  (ba  f \<equiv>\<^sub>i  ba  g) "
proof -
 have  1: "\<turnstile>  ba (f \<supset>\<^sub>i g) \<supset>\<^sub>i  ba  f \<supset>\<^sub>i  ba  g " by (rule BaImpDist)
 have  2: "\<turnstile>  ba (g \<supset>\<^sub>i f) \<supset>\<^sub>i  ba  g \<supset>\<^sub>i  ba  f " by (rule BaImpDist)
 have  3: "\<turnstile> ba (f \<equiv>\<^sub>i g) \<equiv>\<^sub>i ba ((f \<supset>\<^sub>i g) \<and>\<^sub>i (g \<supset>\<^sub>i f)) " by auto
 have  4: "\<turnstile> ba ((f \<supset>\<^sub>i g) \<and>\<^sub>i (g \<supset>\<^sub>i f)) \<equiv>\<^sub>i ba((f \<supset>\<^sub>i g)) \<and>\<^sub>i ba((g \<supset>\<^sub>i f)) " by (rule BaAndEqv) 
 have  5: "\<turnstile> (ba  f \<supset>\<^sub>i  ba  g) \<and>\<^sub>i (ba  g \<supset>\<^sub>i  ba  f) \<equiv>\<^sub>i (ba  f \<equiv>\<^sub>i  ba  g) " by auto
 from 1 2 3 4 5 show ?thesis  using itl_prop(31) itl_prop(32) prop02 by smt 
qed


(* dodgy
lemma DaImpDaImpDa:
 "\<turnstile>    da (f \<supset>\<^sub>i g) \<supset>\<^sub>i  da  f \<supset>\<^sub>i  da  g "
*)

(* dodgy
lemma DaImpDaEqvDa:
 "\<turnstile>    da (f \<equiv>\<^sub>i g) \<supset>\<^sub>i  da  f \<equiv>\<^sub>i  da  g"
*)

lemma BaImpBa:
 assumes "\<turnstile>   f \<supset>\<^sub>i g"
 shows   "\<turnstile>ba  f \<supset>\<^sub>i  ba  g"
using BaGen BaImpDist MP assms by blast

lemma BaEqvBa: 
 assumes "\<turnstile>   f \<equiv>\<^sub>i g"
 shows   "\<turnstile> ba  f \<equiv>\<^sub>i  ba  g"
using BaGen BaImpBaEqvBa MP assms by blast


lemma DaImpDa:
 assumes "\<turnstile>   f \<supset>\<^sub>i g"
 shows   "\<turnstile> da  f \<supset>\<^sub>i  da  g"
using assms by fastforce

lemma DiamondEqvDiamondDiamond:
 "\<turnstile> \<diamond> f \<equiv>\<^sub>i \<diamond> (\<diamond> f)"
proof -
 have 1: "\<turnstile> \<diamond> (\<diamond> f) \<equiv>\<^sub>i true\<^sub>i;(true\<^sub>i;f)" 
      by simp
 have 2: "\<turnstile> true\<^sub>i;(true\<^sub>i;f)  \<equiv>\<^sub>i (true\<^sub>i;true\<^sub>i);f" 
      by (rule ChopAssoc)
 have 3: "\<turnstile> (true\<^sub>i;true\<^sub>i);f \<equiv>\<^sub>i true\<^sub>i;f" 
      using LeftChopEqvChop TrueEqvTrueChopTrue itl_prop(30) by blast
 have 4: "\<turnstile> true\<^sub>i;f  \<equiv>\<^sub>i \<diamond>f" 
      by (simp add: sometimes_d_def)
 from 1 2 3 4 show ?thesis by auto
qed

lemma DaEqvDaDa:
  "\<turnstile>   da  f \<equiv>\<^sub>i  da (  da  f)"
proof -
 have  1: "\<turnstile>  da  f \<equiv>\<^sub>i \<diamond>( di  f)" 
       by (rule DaEqvDtDi)
 have  2: "\<turnstile>  di  f \<equiv>\<^sub>i  (di ( di  f))"  
       by (rule DiEqvDiDi)
 hence 3: "\<turnstile> \<diamond> ( di  f) \<equiv>\<^sub>i \<diamond> (di  (di  f))" 
       by (rule DiamondEqvDiamond)
 have  4: "\<turnstile> \<diamond>  (di  f) \<equiv>\<^sub>i \<diamond>(\<diamond> (di  (di  f)))" 
       using DiamondEqvDiamondDiamond DiEqvDiDi using "3" prop03 by blast 
 have  5: "\<turnstile> \<diamond>  (di  (di  f)) \<equiv>\<^sub>i  di (\<diamond>  (di  f))"  
       by (rule DtDiEqvDiDt)
 hence 6: "\<turnstile> \<diamond>(\<diamond>  (di  (di  f))) \<equiv>\<^sub>i \<diamond> (di (\<diamond>  (di  f)))" 
       by (rule DiamondEqvDiamond)
 have  7: "\<turnstile>  da  f \<equiv>\<^sub>i \<diamond> (di( \<diamond> ( di  f)))" 
       using "1" "3" "4" "6" using prop03 by blast
 have  8: "\<turnstile>  da (\<diamond> ( di  f)) \<equiv>\<^sub>i \<diamond>( di (\<diamond>  (di  f)))" 
       by (rule DaEqvDtDi)  
 have  9: "\<turnstile>  da ( da  f) \<equiv>\<^sub>i  da (\<diamond>  (di  f))" 
       using 1 by (rule DaEqvDa)
 from 7 8 9 show ?thesis by auto
qed

  
lemma BaEqvBaBa:
 "\<turnstile>    ba  f \<equiv>\<^sub>i  ba  (ba  f)"
proof -
 have 1: "\<turnstile>  da (\<not>\<^sub>i  f) \<equiv>\<^sub>i  da  (da (\<not>\<^sub>i  f))" by (rule DaEqvDaDa)  
 have 2: "\<turnstile>  da  (da (\<not>\<^sub>i  f)) \<equiv>\<^sub>i \<not>\<^sub>i  (ba (\<not>\<^sub>i  (da (\<not>\<^sub>i  f))))"  by (rule DaEqvNotBaNot)  
 have 3: "\<turnstile> \<not>\<^sub>i  (da  (da (\<not>\<^sub>i  f))) \<equiv>\<^sub>i  ba (\<not>\<^sub>i  (da (\<not>\<^sub>i  f)))" by auto  
 have 4: "\<turnstile> \<not>\<^sub>i  (da (\<not>\<^sub>i  f)) \<equiv>\<^sub>i  ba (\<not>\<^sub>i  (da (\<not>\<^sub>i  f)))" using "1" "2" "3" prop01 prop03 by blast 
 from 4 show ?thesis by (metis ba_d_def)
qed


(* was \<turnstile>    ba (f\<supset>\<^sub>i f1) \<supset>\<^sub>i f; g \<supset>\<^sub>i f; g1 (clearly a typo) *)
lemma BaLeftChopImpChop:
 "\<turnstile>    ba (f\<supset>\<^sub>i f1) \<supset>\<^sub>i f; g \<supset>\<^sub>i f1; g"
proof -
 have 1: "\<turnstile>  ba (f \<supset>\<^sub>i f1) \<supset>\<^sub>i bi (f \<supset>\<^sub>i f1)" by (rule BaImpBi)  
 have 2: "\<turnstile> bi (f \<supset>\<^sub>i f1) \<supset>\<^sub>i f; g \<supset>\<^sub>i f1; g" by (rule BiChopImpChop)  
 from 1 2 show ?thesis by auto
qed

lemma BaRightChopImpChop: 
 "\<turnstile>    ba (g\<supset>\<^sub>i g1) \<supset>\<^sub>i f; g \<supset>\<^sub>i f; g1"
proof -
 have 1: "\<turnstile>  ba (g \<supset>\<^sub>i g1) \<supset>\<^sub>i \<box>(g \<supset>\<^sub>i g1)" by (rule BaImpBt)  
 have 2: "\<turnstile> \<box>(g \<supset>\<^sub>i g1) \<supset>\<^sub>i f; g \<supset>\<^sub>i f; g1" by (rule BoxChopImpChop)  
 from 1 2 show ?thesis by auto
qed

lemma ChopAndBaImport:
  "\<turnstile>   (f; f1) \<and>\<^sub>i  ba  g \<supset>\<^sub>i (f \<and>\<^sub>i g); (f1 \<and>\<^sub>i g)"
proof -
 have 1: "\<turnstile>  ba  g \<and>\<^sub>i (f; f1) \<supset>\<^sub>i (g \<and>\<^sub>i f); (g \<and>\<^sub>i f1)" by (rule BaAndChopImport)  
 have 2: "\<turnstile> (g \<and>\<^sub>i f); (g \<and>\<^sub>i f1) \<equiv>\<^sub>i (f \<and>\<^sub>i g); (f1 \<and>\<^sub>i g)" by (rule AndChopAndCommute) 
 from 1 2 show ?thesis by auto
qed

lemma BaImpBaImpBaAnd:
 "\<turnstile> ba h \<supset>\<^sub>i ba(g \<supset>\<^sub>i ba h \<and>\<^sub>i g ) "
proof -
 have  1: "\<turnstile> ba h \<supset>\<^sub>i (g \<supset>\<^sub>i ba h \<and>\<^sub>i g ) " by simp
 hence 2: "\<turnstile> ba(ba h) \<supset>\<^sub>i ba(g \<supset>\<^sub>i ba h \<and>\<^sub>i g ) " by (rule BaImpBa)
 have  3: "\<turnstile> ba h \<equiv>\<^sub>i ba(ba h) " by (rule BaEqvBaBa)
 from 2 3 show ?thesis using itl_prop(31) prop02 by blast 
qed


lemma BaChopImpChopBa:
 "\<turnstile>    ba  f \<supset>\<^sub>i g; g1 \<supset>\<^sub>i g; (( ba  f) \<and>\<^sub>i g1)"
proof - 
 have 1: "\<turnstile>  ba  f \<supset>\<^sub>i  ba (g1 \<supset>\<^sub>i (ba f) \<and>\<^sub>i g1 )" by (rule BaImpBaImpBaAnd)
 have 2: "\<turnstile>  ba (g1 \<supset>\<^sub>i  ba  f \<and>\<^sub>i g1 ) \<supset>\<^sub>i g; g1 \<supset>\<^sub>i g; ( ba  f \<and>\<^sub>i g1)" by (rule BaRightChopImpChop)  
 from 1 2 show ?thesis by auto
qed

lemma DiNotBaImpNotBa:
  "\<turnstile>   di \<not>\<^sub>i ( ba  f) \<supset>\<^sub>i \<not>\<^sub>i ( ba  f)" 
proof -
 have  1: "\<turnstile>  ba  f \<equiv>\<^sub>i  ba(  ba  f)" by (rule BaEqvBaBa)  
 have  2: "\<turnstile>  ba ( ba  f) \<supset>\<^sub>i bi ( ba  f)" by (rule BaImpBi)  
 have  3: "\<turnstile>  ba  f \<supset>\<^sub>i bi ( ba  f)" using "1" "2" using itl_prop(31) prop02 by blast  
 hence 4: "\<turnstile>  ba  f \<supset>\<^sub>i \<not>\<^sub>i ( di \<not>\<^sub>i ( ba  f))" by (simp add: bi_d_def)    
 from 4 show ?thesis by fastforce
qed

lemma NotBaChopImpNotBa:
 "\<turnstile>   (\<not>\<^sub>i ( ba  f)); g \<supset>\<^sub>i \<not>\<^sub>i ( ba  f)"
proof -
 have 1: "\<turnstile> (\<not>\<^sub>i ( ba  f)); g \<supset>\<^sub>i  di \<not>\<^sub>i ( ba  f)" by (rule ChopImpDi)  
 have 2: "\<turnstile>  di \<not>\<^sub>i ( ba  f) \<supset>\<^sub>i \<not>\<^sub>i ( ba  f)"  by (rule DiNotBaImpNotBa) 
 from 1 2 show ?thesis using prop02 by blast  
qed

lemma DiamondFinImpFin:
 " \<turnstile> \<diamond> (fin f) \<supset>\<^sub>i fin f"
proof -
 have  1: "\<turnstile> fin f \<equiv>\<^sub>i true\<^sub>i;(f \<and>\<^sub>i empty)" 
       by (rule FinEqvTrueChopAndEmpty)
 hence 2: "\<turnstile> \<diamond> (fin f) \<equiv>\<^sub>i true\<^sub>i;(true\<^sub>i;(f \<and>\<^sub>i empty)) " 
       by fastforce
 have  3: "\<turnstile> true\<^sub>i;(true\<^sub>i;(f \<and>\<^sub>i empty)) \<equiv>\<^sub>i (true\<^sub>i;true\<^sub>i);(f \<and>\<^sub>i empty)" 
       by (rule ChopAssoc)
 have  4: "\<turnstile> (true\<^sub>i;true\<^sub>i);(f \<and>\<^sub>i empty) \<equiv>\<^sub>i true\<^sub>i;(f \<and>\<^sub>i empty)" 
       using TrueEqvTrueChopTrue using LeftChopEqvChop itl_prop(30) by blast
 from 1 2 3 4 show ?thesis by auto
qed

lemma ChopFinImpFin:
 " \<turnstile>   f;  fin  (init w) \<supset>\<^sub>i  fin  (init w)"
proof -
 have 1: "\<turnstile> f;  fin  (init w) \<supset>\<^sub>i \<diamond> ( fin  (init w))"  by (rule  ChopImpDiamond) 
 have 2: "\<turnstile> \<diamond>  (fin  (init w)) \<supset>\<^sub>i  fin  (init w)" by (rule DiamondFinImpFin)
 from  1 2  show ?thesis using prop02 by blast   
qed


lemma FinImpYieldsFin:
 "\<turnstile>    fin  (init w) \<supset>\<^sub>i f yields ( fin  (init w))"
proof -
 have  1: "\<turnstile> f;  fin   (init \<not>\<^sub>i w) \<supset>\<^sub>i  fin   (init \<not>\<^sub>i w)" 
       by (rule ChopFinImpFin) 
 have  2: "\<turnstile>  fin   (init \<not>\<^sub>i w) \<equiv>\<^sub>i \<not>\<^sub>i ( fin  (init w))" 
       using FinNotStateEqvNotFinState by blast
 hence 3: "\<turnstile> f;  fin   (init \<not>\<^sub>i w) \<equiv>\<^sub>i f; \<not>\<^sub>i ( fin  (init w))"  
       by (rule RightChopEqvChop)  
 have  4: "\<turnstile> f; \<not>\<^sub>i ( fin  (init w)) \<supset>\<^sub>i \<not>\<^sub>i ( fin  (init w))" 
       using "1" "2" "3" itl_prop(31) prop02 by blast  
 hence 5: "\<turnstile>  fin  (init w) \<supset>\<^sub>i \<not>\<^sub>i (f; \<not>\<^sub>i ( fin  (init w)))" 
       using itl_prop(35) by auto
 from 5 show ?thesis by (simp add: yields_d_def)
qed


lemma ChopAndFin:
 " \<turnstile>  (f; g) \<and>\<^sub>i  fin  (init w) \<equiv>\<^sub>i f; (g \<and>\<^sub>i  fin  (init w))"
proof -
 have  1: "\<turnstile>  fin  (init w) \<supset>\<^sub>i f yields ( fin  (init w))" 
       by (rule FinImpYieldsFin)  
 hence 2: "\<turnstile> (f; g) \<and>\<^sub>i  fin  (init w) \<supset>\<^sub>i (f; g) \<and>\<^sub>i f yields  ( fin  (init w))" 
       by auto
 have  3: "\<turnstile> (f; g) \<and>\<^sub>i f yields  ( fin  (init w)) \<supset>\<^sub>i f; (g \<and>\<^sub>i  fin  (init w))" 
       by (rule ChopAndYieldsImp) 
 have  4: "\<turnstile> (f; g) \<and>\<^sub>i  fin  (init w) \<supset>\<^sub>i f; (g \<and>\<^sub>i  fin  (init w))" 
       using "2" "3"  by auto
 have 11: "\<turnstile> f; (g \<and>\<^sub>i  fin  (init w)) \<supset>\<^sub>i f; g"  
      by (rule ChopAndA)  
 have 12: "\<turnstile> f; (g \<and>\<^sub>i  fin  (init w)) \<supset>\<^sub>i f;  fin  (init w)" 
      by (rule ChopAndB)  
 have 13: "\<turnstile> f;  fin  (init w) \<supset>\<^sub>i \<diamond> ( fin  (init w))" 
      by (rule ChopImpDiamond)  
 have 14: "\<turnstile> \<diamond>( fin  (init w)) \<supset>\<^sub>i  fin  (init w)"  
      by (rule DiamondFinImpFin)
 have 15: "\<turnstile> f; (g \<and>\<^sub>i  fin  (init w)) \<supset>\<^sub>i (f; g) \<and>\<^sub>i  fin  (init w)" 
      using "11" "12" "13" "14" itl_prop(32) prop02 by smt  
 from 4 15 show ?thesis using itl_prop(31) by blast
qed

lemma ChopAndNotFin:
  " \<turnstile>   f; g \<and>\<^sub>i \<not>\<^sub>i ( fin  (init w)) \<equiv>\<^sub>i f; (g \<and>\<^sub>i \<not>\<^sub>i ( fin  (init w)))"
proof -
 have  1: "\<turnstile> f; g \<and>\<^sub>i  fin   (init \<not>\<^sub>i w) \<equiv>\<^sub>i f; (g \<and>\<^sub>i  fin   (init \<not>\<^sub>i w))" 
       by (rule ChopAndFin)  
 have  2: "\<turnstile>  fin   (init \<not>\<^sub>i w) \<equiv>\<^sub>i \<not>\<^sub>i  ( fin  (init w) )"   
       using FinNotStateEqvNotFinState by blast 
 hence 3: "\<turnstile> g \<and>\<^sub>i  fin   (init \<not>\<^sub>i w) \<equiv>\<^sub>i g \<and>\<^sub>i \<not>\<^sub>i(  fin  (init w))" 
       by auto 
 hence 4: "\<turnstile> f; (g \<and>\<^sub>i  fin    (init \<not>\<^sub>i w)) \<equiv>\<^sub>i f; (g \<and>\<^sub>i \<not>\<^sub>i ( fin  (init w)))" 
       by (rule RightChopEqvChop)  
 from 1 2 4 show ?thesis by auto
qed

lemma FinChopChain:
 "\<turnstile>   ((init w)\<supset>\<^sub>i  fin  (init w1)); ((init w1) \<supset>\<^sub>i  fin  (init w2))
       \<supset>\<^sub>i ((init w) \<supset>\<^sub>i  fin  (init w2))"
proof -
 have  1: "\<turnstile>  (init w) \<and>\<^sub>i  ((init w)\<supset>\<^sub>i  fin  (init w1)); ((init w1) \<supset>\<^sub>i  fin  (init w2))
           \<supset>\<^sub>i   
           ( (init w) \<and>\<^sub>i ((init w)\<supset>\<^sub>i fin  (init w1))); ((init w1)\<supset>\<^sub>i  fin  (init w2))"
       by (rule StateAndChopImport)  
 have  2: "\<turnstile> (init w) \<and>\<^sub>i ((init w) \<supset>\<^sub>i  fin  (init w1)) \<supset>\<^sub>i  fin  (init w1)" 
       by auto
 have  3: "\<turnstile> ((init w) \<and>\<^sub>i ((init w)\<supset>\<^sub>i  fin  (init w1))); ((init w1) \<supset>\<^sub>i  fin  (init w2))
           \<supset>\<^sub>i  
           ( fin  (init w1)); ((init w1) \<supset>\<^sub>i  fin  (init w2))" 
       using  "2" by (rule LeftChopImpChop)  
 have  4: "\<turnstile>  ( fin  (init w1)); ((init w1) \<supset>\<^sub>i  fin  (init w2)) \<equiv>\<^sub>i
            \<diamond>((init w1) \<and>\<^sub>i ((init w1) \<supset>\<^sub>i  fin  (init w2)))" 
      by (rule FinChopEqvDiamond)  
 have 41: "\<turnstile> ((init w1) \<and>\<^sub>i ((init w1) \<supset>\<^sub>i  fin (init w2))) \<supset>\<^sub>i fin (init w2)" 
      by auto
 have 42: "\<turnstile> \<diamond>((init w1) \<and>\<^sub>i ((init w1) \<supset>\<^sub>i  fin  (init w2))) \<supset>\<^sub>i \<diamond> ( fin (init w2))" 
      using "41" DiamondImpDiamond by blast  
 have  5: "\<turnstile> \<diamond>( fin( init w2)) \<supset>\<^sub>i  fin  (init w2)" 
      using DiamondFinImpFin by blast
 have  6: "\<turnstile> (init w) \<and>\<^sub>i  ((init w)\<supset>\<^sub>i  fin  (init w1)); ((init w1) \<supset>\<^sub>i  fin  (init w2))
               \<supset>\<^sub>i  fin  (init w2)" 
      using   "1" "3" "4" "5" "42" itl_prop(30) prop02 prop15 by smt  
 from 6 show ?thesis using prop32 by blast  
qed

lemma ChopRule:
 assumes "\<turnstile>   (init w) \<and>\<^sub>i f \<supset>\<^sub>i  fin  (init w1)"  
         "\<turnstile>   (init w1)\<and>\<^sub>i f1 \<supset>\<^sub>i  fin  (init w2)"  
 shows   "\<turnstile>   (init w) \<and>\<^sub>i (f; f1) \<supset>\<^sub>i  fin  (init w2)"
proof -
 have  1: "\<turnstile> (init w) \<and>\<^sub>i (f; f1) \<supset>\<^sub>i ((init w) \<and>\<^sub>i f); f1" by (rule StateAndChopImport)  
 have  2: "\<turnstile> (init w) \<and>\<^sub>i f \<supset>\<^sub>i  fin  (init w1)" using assms by auto
 hence 3: "\<turnstile> ((init w) \<and>\<^sub>i f); f1 \<supset>\<^sub>i ( fin  (init w1)); f1" by (rule LeftChopImpChop)  
 have  4: "\<turnstile> ( fin  (init w1)); f1 \<equiv>\<^sub>i  \<diamond>((init w1) \<and>\<^sub>i f1)" by (rule FinChopEqvDiamond)  
 have  5: "\<turnstile> (init w1) \<and>\<^sub>i f1 \<supset>\<^sub>i  fin  (init w2)" using assms by auto  
 hence 6: "\<turnstile> \<diamond>((init w1) \<and>\<^sub>i f1) \<supset>\<^sub>i \<diamond> (fin  (init w2))" by (rule DiamondImpDiamond)  
 have  7: "\<turnstile> \<diamond>( fin  (init w2)) \<supset>\<^sub>i  fin  (init w2)"  using DiamondFinImpFin by blast
 from 1 3 4 6 7 show ?thesis using itl_prop(30) prop02 prop15 by smt 
qed


lemma ChopRep:
 assumes "\<turnstile>  (init w) \<and>\<^sub>i f \<supset>\<^sub>i f1 \<and>\<^sub>i  fin  (init w1)"  
         "\<turnstile>   (init w1) \<and>\<^sub>i g \<supset>\<^sub>i g1"  
 shows   "\<turnstile>   (init w) \<and>\<^sub>i (f; g) \<supset>\<^sub>i (f1; g1)"
proof -
 have  1: "\<turnstile> (init w) \<and>\<^sub>i f \<supset>\<^sub>i f1 \<and>\<^sub>i  fin  (init w1)" using assms by auto  
 hence 2: "\<turnstile> (init w) \<and>\<^sub>i (f; g) \<supset>\<^sub>i (f1 \<and>\<^sub>i  fin  (init w1)); g" by (rule StateAndChopImpChopRule)  
 have  3: "\<turnstile> (f1 \<and>\<^sub>i  fin  (init w1)); g \<equiv>\<^sub>i f1; ((init w1) \<and>\<^sub>i g)" by (rule AndFinChopEqvStateAndChop)  
 have  4: "\<turnstile> (init w1)\<and>\<^sub>i g \<supset>\<^sub>i g1" using assms by auto   
 hence 5: "\<turnstile> f1; ((init w1) \<and>\<^sub>i g) \<supset>\<^sub>i f1; g1" by  (rule RightChopImpChop)  
 from 2 3 5 show ?thesis by simp
qed

lemma ChopRepAndFin:
 assumes "\<turnstile>   (init w) \<and>\<^sub>i f \<supset>\<^sub>i f1 \<and>\<^sub>i  fin  (init w1)"  
         "\<turnstile>   (init w1) \<and>\<^sub>i g \<supset>\<^sub>i g1 \<and>\<^sub>i  fin  (init w2)"  
 shows   "\<turnstile>   (init w) \<and>\<^sub>i (f; g) \<supset>\<^sub>i (f1; g1) \<and>\<^sub>i  fin  (init w2)"
proof -
 have  1: "\<turnstile> (init w) \<and>\<^sub>i f \<supset>\<^sub>i f1 \<and>\<^sub>i  fin  (init w1)" using assms by auto  
 have  2: "\<turnstile> (init w1) \<and>\<^sub>i g \<supset>\<^sub>i g1 \<and>\<^sub>i  fin  (init w2)" using assms by auto   
 have  3: "\<turnstile> (init w) \<and>\<^sub>i (f; g) \<supset>\<^sub>i f1; (g1 \<and>\<^sub>i  fin  (init w2))" using "1" "2" by (rule ChopRep)  
 have  4: "\<turnstile> f1; (g1 \<and>\<^sub>i  fin  (init w2)) \<supset>\<^sub>i f1; g1" by (rule ChopAndA)  
 have  5: "\<turnstile> f1; (g1 \<and>\<^sub>i  fin  (init w2)) \<supset>\<^sub>i f1;  fin  (init w2)" by (rule ChopAndB)  
 have  6: "\<turnstile> f1;  fin  (init w2) \<supset>\<^sub>i  fin  (init w2)" by (rule ChopFinImpFin)  
 from 1 2 3 4 5 6 show ?thesis using ChopRep ChopRule itl_prop(32) by blast
qed

lemma TrueChopMoreEqvMore:
 "\<turnstile> true\<^sub>i ; more \<equiv>\<^sub>i more"
by auto


(* changed proof *)
lemma MoreChopLoop:
 assumes "\<turnstile>   f \<supset>\<^sub>i  more ; f"
 shows   "\<turnstile> \<not>\<^sub>i  f"
proof -
 have   1: "\<turnstile> f \<supset>\<^sub>i  more ; f" 
        using assms by auto 
 hence 11: "\<turnstile> \<diamond> f \<supset>\<^sub>i \<diamond> (more;f)" 
       by (rule DiamondImpDiamond)
 have  12: "\<turnstile> \<diamond> (more;f) \<equiv>\<^sub>i true\<^sub>i;(more;f)" 
       by simp
 have  13: "\<turnstile> true\<^sub>i;(more;f) \<equiv>\<^sub>i (true\<^sub>i;more);f" 
       by (rule ChopAssoc)
 have  14: "\<turnstile>  \<diamond> (more;f) \<equiv>\<^sub>i more;f" 
       using TrueChopMoreEqvMore "12" "13" LeftChopChopImpChopRule NowImpDiamond 
             itl_prop(31) prop02 by metis 
 have   2: "\<turnstile>  more ; f \<equiv>\<^sub>i \<circle>(\<diamond> f)" 
       by  (rule MoreChopEqvNextDiamond)  
 have   3: "\<turnstile>  \<diamond> f \<supset>\<^sub>i \<circle>(\<diamond> f)"  
       using "11" "14" "2" by auto 
 hence  4: "\<turnstile> \<not>\<^sub>i (\<diamond> f)" 
       by (rule NextLoop)
 have   5: "\<turnstile> \<not>\<^sub>i (\<diamond> f) \<supset>\<^sub>i \<not>\<^sub>i f" 
       by auto
 from 4 5 show ?thesis  using MP by blast
qed

lemma MoreChopContra:
 assumes "\<turnstile>   f \<and>\<^sub>i \<not>\<^sub>i  g \<supset>\<^sub>i ( more ; (f \<and>\<^sub>i \<not>\<^sub>i  g))"
 shows   "\<turnstile> f \<supset>\<^sub>i g"
proof -
 have  1: "\<turnstile> f \<and>\<^sub>i \<not>\<^sub>i  g \<supset>\<^sub>i ( more ; (f \<and>\<^sub>i \<not>\<^sub>i  g))" using assms by auto
 hence 2: "\<turnstile> \<not>\<^sub>i (f \<and>\<^sub>i \<not>\<^sub>i  g)" by (rule MoreChopLoop)  
 from 2 show ?thesis by auto
qed

lemma ChopLoop:
 assumes "\<turnstile>   f \<supset>\<^sub>i g;f" 
         "\<turnstile> g \<supset>\<^sub>i  more"
 shows   "\<turnstile> \<not>\<^sub>i  f"
proof -
have  1: "\<turnstile> f \<supset>\<^sub>i g; f" using assms by auto  
have  2: "\<turnstile> g \<supset>\<^sub>i  more" using assms by auto  
hence 3: "\<turnstile> g; f \<supset>\<^sub>i  more ; f" by (rule LeftChopImpChop)  
have  4: "\<turnstile> f \<supset>\<^sub>i  more ; f" using "1" "3" by auto  
from 4 show ?thesis using MoreChopLoop by auto
qed

lemma ChopContra:
 assumes  "\<turnstile>   f \<and>\<^sub>i \<not>\<^sub>i  g \<supset>\<^sub>i h; f \<and>\<^sub>i \<not>\<^sub>i (h; g)"
          "\<turnstile> h \<supset>\<^sub>i  more" 
 shows    "\<turnstile> f \<supset>\<^sub>i g"
proof -
have  1: "\<turnstile> f \<and>\<^sub>i \<not>\<^sub>i  g \<supset>\<^sub>i h; f \<and>\<^sub>i \<not>\<^sub>i (h; g)" using assms by auto  
have  2: "\<turnstile> h \<supset>\<^sub>i  more" using assms by auto   
have  3: "\<turnstile> h; f \<and>\<^sub>i \<not>\<^sub>i (h; g) \<supset>\<^sub>i h; (f \<and>\<^sub>i \<not>\<^sub>i  g)" by (rule ChopAndNotChopImp)  
have  4: "\<turnstile> h; (f \<and>\<^sub>i \<not>\<^sub>i  g) \<supset>\<^sub>i  more ; (f \<and>\<^sub>i \<not>\<^sub>i  g)" using "2" by (rule LeftChopImpChop)  
have  5: "\<turnstile> f \<and>\<^sub>i \<not>\<^sub>i  g \<supset>\<^sub>i  more ; (f \<and>\<^sub>i \<not>\<^sub>i  g)" using "1" "3" "4" by auto  
from 5 show ?thesis using MoreChopContra by auto
qed

subsection \<open>Properties of Chopstar and Chopplus\<close>

lemma EmptyImpCS:
 "\<turnstile> empty  \<supset>\<^sub>i f\<^sup>\<star>"
proof -
 have 1: "\<turnstile> f\<^sup>\<star> \<equiv>\<^sub>i empty \<or>\<^sub>i (f \<and>\<^sub>i more);f\<^sup>\<star>" by (rule ChopstarEqv) 
 have 2: "\<turnstile> empty \<supset>\<^sub>i empty \<or>\<^sub>i (f \<and>\<^sub>i more);f\<^sup>\<star>" by auto
 from 1 2 show ?thesis using itl_prop(31) prop02 by blast
qed

lemma CSEqvOrChopCS:
 "\<turnstile>   f\<^sup>\<star> \<equiv>\<^sub>i  empty  \<or>\<^sub>i  (f; f\<^sup>\<star>) " 
proof -
 have 1: "\<turnstile> f\<^sup>\<star> \<equiv>\<^sub>i empty \<or>\<^sub>i (f \<and>\<^sub>i more);f\<^sup>\<star>" by (rule ChopstarEqv)
 have 2: "\<turnstile> (f \<and>\<^sub>i more);f\<^sup>\<star> \<supset>\<^sub>i f;f\<^sup>\<star>" by (rule AndChopA)
 have 3: "\<turnstile> f\<^sup>\<star> \<supset>\<^sub>i  empty  \<or>\<^sub>i  f; f\<^sup>\<star> "using "1" "2" using prop14 by blast
 have 4: "\<turnstile> empty  \<supset>\<^sub>i f\<^sup>\<star>" by (rule EmptyImpCS)
 have 5: "\<turnstile> f \<supset>\<^sub>i empty \<or>\<^sub>i (f \<and>\<^sub>i more)" by auto 
 have 6: "\<turnstile> f; f\<^sup>\<star> \<supset>\<^sub>i f\<^sup>\<star> \<or>\<^sub>i  (f \<and>\<^sub>i  more ); f\<^sup>\<star>" using "5" by (rule EmptyOrChopImpRule)
 have 7: "\<turnstile> f\<^sup>\<star> \<supset>\<^sub>i empty \<or>\<^sub>i (f \<and>\<^sub>i more);f\<^sup>\<star>" using "1" using itl_prop(31) by blast
 have 8: "\<turnstile> f; f\<^sup>\<star> \<supset>\<^sub>i empty \<or>\<^sub>i  (f \<and>\<^sub>i  more ); f\<^sup>\<star>"  using "6" "7"  prop16 by blast
 hence 9: "\<turnstile> f; f\<^sup>\<star> \<supset>\<^sub>i f\<^sup>\<star>" using "1" prop15 by blast
 have 10: "\<turnstile> empty  \<or>\<^sub>i  f; f\<^sup>\<star> \<supset>\<^sub>i f\<^sup>\<star> " using  "9" "4" prop30 by blast
 from 3 10 show ?thesis using itl_prop(31) by blast
qed

lemma CSAndMoreEqvAndMoreChop:
 " \<turnstile>  f\<^sup>\<star> \<and>\<^sub>i  more  \<equiv>\<^sub>i (f \<and>\<^sub>i  more ); f\<^sup>\<star> "
proof -
 have  1: "\<turnstile> ( empty  \<or>\<^sub>i  (f \<and>\<^sub>i  more ); f\<^sup>\<star>) \<and>\<^sub>i  more \<supset>\<^sub>i (f \<and>\<^sub>i  more ); f\<^sup>\<star> " by auto
 have  2: "\<turnstile> f\<^sup>\<star> \<equiv>\<^sub>i empty \<or>\<^sub>i (f \<and>\<^sub>i more);f\<^sup>\<star>" by (rule ChopstarEqv)
 have  3: "\<turnstile> f\<^sup>\<star> \<and>\<^sub>i  more  \<supset>\<^sub>i (f \<and>\<^sub>i  more ); f\<^sup>\<star> " using "1" "2" using prop18 by blast
 have  4: "\<turnstile> (f \<and>\<^sub>i  more ); f\<^sup>\<star> \<supset>\<^sub>i f\<^sup>\<star> " using "2" prop19 by blast
 have  5: "\<turnstile> (f \<and>\<^sub>i  more ) \<supset>\<^sub>i more" by auto
 hence 6: "\<turnstile> (f \<and>\<^sub>i  more ); f\<^sup>\<star> \<supset>\<^sub>i  more " by (rule LeftChopImpMoreRule)
 have  7: "\<turnstile> (f \<and>\<^sub>i  more ); f\<^sup>\<star> \<supset>\<^sub>i f\<^sup>\<star> \<and>\<^sub>i  more " using "4" "6" using itl_prop(32) by blast
 from 3 7 show ?thesis using itl_prop(31) by blast
qed
(* same as CSAndMoreImpChopPlus *)
lemma CSAndMoreImpChopCS:
 "\<turnstile>   f\<^sup>\<star> \<and>\<^sub>i  more  \<supset>\<^sub>i f; f\<^sup>\<star> "
proof -
 have 1: "\<turnstile> f\<^sup>\<star> \<and>\<^sub>i  more  \<equiv>\<^sub>i (f \<and>\<^sub>i  more ); f\<^sup>\<star> " by (rule CSAndMoreEqvAndMoreChop)
 have 2: "\<turnstile> (f \<and>\<^sub>i  more ); f\<^sup>\<star> \<supset>\<^sub>i f; f\<^sup>\<star> " by (rule AndChopA)
 from 1 2 show ?thesis by auto
qed

lemma NotAndMoreEqvEmptyOr:
 "\<turnstile> \<not>\<^sub>i (f \<and>\<^sub>i more) \<equiv>\<^sub>i (empty \<or>\<^sub>i \<not>\<^sub>if)"
by auto

lemma MoreAndEmptyOrEqvMoreAnd:
 "\<turnstile> more \<and>\<^sub>i (empty \<or>\<^sub>i \<not>\<^sub>if) \<equiv>\<^sub>i more \<and>\<^sub>i \<not>\<^sub>i f "
by auto

lemma CSMoreNotImpChopCSAndMore:
"\<turnstile>   f\<^sup>\<star> \<and>\<^sub>i  more  \<and>\<^sub>i \<not>\<^sub>i  f \<supset>\<^sub>i (f \<and>\<^sub>i  more ); (f\<^sup>\<star>\<and>\<^sub>i  more ) "
proof -
 have  1: "\<turnstile> f\<^sup>\<star> \<and>\<^sub>i  more  \<equiv>\<^sub>i (f \<and>\<^sub>i  more ); f\<^sup>\<star>" 
       by (rule CSAndMoreEqvAndMoreChop)
 have  2: "\<turnstile>  empty  \<or>\<^sub>i   more" 
       by auto  
 hence 3: "\<turnstile> f\<^sup>\<star> \<supset>\<^sub>i  empty  \<or>\<^sub>i  (f\<^sup>\<star> \<and>\<^sub>i  more )" 
       by auto   
 hence 4: "\<turnstile> (f \<and>\<^sub>i  more ); f\<^sup>\<star> \<supset>\<^sub>i (f \<and>\<^sub>i  more ) \<or>\<^sub>i  ((f \<and>\<^sub>i  more ); (f\<^sup>\<star> \<and>\<^sub>i  more ))" 
       by (rule ChopEmptyOrImpRule)  
 hence 5: "\<turnstile> (f \<and>\<^sub>i  more ); f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i(f \<and>\<^sub>i more) \<supset>\<^sub>i   ((f \<and>\<^sub>i  more ); (f\<^sup>\<star> \<and>\<^sub>i  more )) " 
       using prop29 by blast
 have  6: "\<turnstile> (f \<and>\<^sub>i  more ); f\<^sup>\<star> \<equiv>\<^sub>i  (f \<and>\<^sub>i  more ); f\<^sup>\<star> \<and>\<^sub>i more  " using "1" 
       by auto
 have  7: "\<turnstile> (f \<and>\<^sub>i  more ); f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i(f \<and>\<^sub>i more) \<equiv>\<^sub>i (f \<and>\<^sub>i  more ); f\<^sup>\<star> \<and>\<^sub>i more \<and>\<^sub>i \<not>\<^sub>i(f \<and>\<^sub>i more) " 
       using "6" by auto
 have  8: "\<turnstile> (f \<and>\<^sub>i  more ); f\<^sup>\<star> \<and>\<^sub>i  more  \<and>\<^sub>i \<not>\<^sub>i  f\<supset>\<^sub>i (f \<and>\<^sub>i  more ); (f\<^sup>\<star> \<and>\<^sub>i  more )" 
       using "5" "7" by auto 
 have  9: "\<turnstile> f\<^sup>\<star> \<and>\<^sub>i  more  \<and>\<^sub>i \<not>\<^sub>i  f \<equiv>\<^sub>i (f\<^sup>\<star> \<and>\<^sub>i  more) \<and>\<^sub>i (more  \<and>\<^sub>i \<not>\<^sub>i  f) " 
       by auto
 have  10: "\<turnstile> (f\<^sup>\<star> \<and>\<^sub>i  more) \<and>\<^sub>i (more  \<and>\<^sub>i \<not>\<^sub>i  f) \<equiv>\<^sub>i (f \<and>\<^sub>i  more ); f\<^sup>\<star> \<and>\<^sub>i  (more  \<and>\<^sub>i \<not>\<^sub>i  f)" 
       using "1" prop06 by auto
 from 1 8 9 10 show ?thesis by auto
qed

lemma CSAndMoreImpCSChop:
 "\<turnstile>   f\<^sup>\<star> \<and>\<^sub>i  more  \<supset>\<^sub>i f\<^sup>\<star>; f "
proof -
 have  1: "\<turnstile> f\<^sup>\<star> \<and>\<^sub>i  more  \<equiv>\<^sub>i (f \<and>\<^sub>i  more ); f\<^sup>\<star> " 
       by (rule CSAndMoreEqvAndMoreChop)
 have  2: "\<turnstile>  empty  \<or>\<^sub>i   more " 
       by auto
 hence 3: "\<turnstile> f\<^sup>\<star> \<supset>\<^sub>i  empty  \<or>\<^sub>i  (f\<^sup>\<star> \<and>\<^sub>i  more ) " 
       by auto
 hence 4: "\<turnstile> (f \<and>\<^sub>i  more ); f\<^sup>\<star> \<supset>\<^sub>i
             (f \<and>\<^sub>i  more ) \<or>\<^sub>i  ((f \<and>\<^sub>i  more ); (f\<^sup>\<star>\<and>\<^sub>i more )) " 
       by (rule ChopEmptyOrImpRule)
 have  5: "\<turnstile> f\<^sup>\<star> \<and>\<^sub>i  more  \<and>\<^sub>i \<not>\<^sub>i  f \<supset>\<^sub>i (f \<and>\<^sub>i  more ); (f\<^sup>\<star>\<and>\<^sub>i  more ) " 
       by (rule CSMoreNotImpChopCSAndMore)
 have  6: "\<turnstile> f\<^sup>\<star> \<equiv>\<^sub>i  empty  \<or>\<^sub>i  (f \<and>\<^sub>i  more ); f\<^sup>\<star> " 
       by (rule ChopstarEqv)
 hence 7: "\<turnstile> f\<^sup>\<star>; f \<equiv>\<^sub>i f \<or>\<^sub>i  ((f \<and>\<^sub>i  more ); f\<^sup>\<star>); f " 
       by (rule EmptyOrChopEqvRule)
 have  8: "\<turnstile> (f \<and>\<^sub>i  more ); (f\<^sup>\<star>; f)\<equiv>\<^sub>i ((f \<and>\<^sub>i  more ); f\<^sup>\<star>); f   " 
       by (rule ChopAssoc)
 have  9: "\<turnstile> (f\<^sup>\<star> \<and>\<^sub>i  more ) \<and>\<^sub>i \<not>\<^sub>i (f\<^sup>\<star>; f) \<supset>\<^sub>i 
             (f \<and>\<^sub>i  more ); (f\<^sup>\<star>\<and>\<^sub>i  more ) \<and>\<^sub>i \<not>\<^sub>i ((f \<and>\<^sub>i  more ); (f\<^sup>\<star>; f)) " 
       using "5" "7" "8" by auto
 have 10: "\<turnstile> f \<and>\<^sub>i  more  \<supset>\<^sub>i  more" 
       by auto
 from 9 10 show ?thesis by (rule ChopContra)
qed

lemma NotEmptyEqvMore:
 "\<turnstile> \<not>\<^sub>i empty \<equiv>\<^sub>i more"
by simp

lemma NotCSImpMore:
 "\<turnstile>    \<not>\<^sub>i  (f\<^sup>\<star>) \<supset>\<^sub>i  more "
proof -
 have  1: "\<turnstile> empty \<supset>\<^sub>i (f\<^sup>\<star>)  " using EmptyImpCS by blast
 hence 2: "\<turnstile>  \<not>\<^sub>i empty \<or>\<^sub>i (f\<^sup>\<star>)" using itl_prop(35) by metis
 from 2 show ?thesis  using "1" NotEmptyEqvMore itl_prop(31) prop02 prop27 by blast 
qed


lemma  CSChopCSImpCS:
 "\<turnstile>   f\<^sup>\<star>; f\<^sup>\<star> \<supset>\<^sub>i f\<^sup>\<star> "
proof -
 have  1: "\<turnstile> f\<^sup>\<star> \<equiv>\<^sub>i  empty  \<or>\<^sub>i  (f \<and>\<^sub>i  more ); f\<^sup>\<star> " 
       by (rule ChopstarEqv)
 hence 2: "\<turnstile> f\<^sup>\<star>; f\<^sup>\<star> \<equiv>\<^sub>i f\<^sup>\<star> \<or>\<^sub>i  ((f \<and>\<^sub>i  more ); f\<^sup>\<star>); f\<^sup>\<star> " 
       by (rule EmptyOrChopEqvRule)
 have 21: "\<turnstile> f\<^sup>\<star>; f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i (f\<^sup>\<star>) \<supset>\<^sub>i ((f \<and>\<^sub>i  more ); f\<^sup>\<star>); f\<^sup>\<star>  " 
       using "2" by (simp add: or_d_def)
 have 22: "\<turnstile> \<not>\<^sub>i (f\<^sup>\<star>) \<equiv>\<^sub>i \<not>\<^sub>iempty \<and>\<^sub>i \<not>\<^sub>i ((f \<and>\<^sub>i  more ); f\<^sup>\<star>)" 
       using "1" prop20 by blast
 have 23: "\<turnstile>  \<not>\<^sub>i (f\<^sup>\<star>) \<supset>\<^sub>i \<not>\<^sub>i ((f \<and>\<^sub>i  more ); f\<^sup>\<star>)" 
       using "2" "22" using itl_prop(31) itl_prop(32) by blast
 have 24: "\<turnstile>  f\<^sup>\<star>; f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i (f\<^sup>\<star>) \<supset>\<^sub>i \<not>\<^sub>i (f\<^sup>\<star>)" 
       by auto
 have 25: "\<turnstile>  f\<^sup>\<star>; f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i (f\<^sup>\<star>) \<supset>\<^sub>i \<not>\<^sub>i ((f \<and>\<^sub>i  more ); f\<^sup>\<star>) " 
       using "23" "24" MP by auto
 have  3: "\<turnstile> f\<^sup>\<star>; f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i (f\<^sup>\<star>) \<supset>\<^sub>i ((f \<and>\<^sub>i  more ); f\<^sup>\<star>); f\<^sup>\<star>\<and>\<^sub>i \<not>\<^sub>i ((f \<and>\<^sub>i  more ); f\<^sup>\<star>) " 
       using "21" "25" by auto
 have  4: "\<turnstile> (f \<and>\<^sub>i  more );( f\<^sup>\<star>; f\<^sup>\<star>)\<equiv>\<^sub>i ((f \<and>\<^sub>i  more ); f\<^sup>\<star>); f\<^sup>\<star>   " 
       by (rule ChopAssoc)
 have  5: "\<turnstile> f\<^sup>\<star>; f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i (f\<^sup>\<star>) \<supset>\<^sub>i (f \<and>\<^sub>i  more ); (f\<^sup>\<star>; f\<^sup>\<star>) \<and>\<^sub>i \<not>\<^sub>i ((f \<and>\<^sub>i  more ); f\<^sup>\<star>)  " 
       using "3" "4" by auto
 have  6: "\<turnstile> f \<and>\<^sub>i  more  \<supset>\<^sub>i  more " 
       by auto
 from 5 6 show ?thesis using ChopContra by blast
qed

lemma ImpChopPlus:
 "\<turnstile>   f \<supset>\<^sub>i f;f\<^sup>\<star> "
proof -
 have  1: "\<turnstile> f\<^sup>\<star> \<equiv>\<^sub>i empty \<or>\<^sub>i  f; f\<^sup>\<star>  " by (rule CSEqvOrChopCS)
 hence 2: "\<turnstile> f;f\<^sup>\<star> \<equiv>\<^sub>i f;empty \<or>\<^sub>i  f; (f;f\<^sup>\<star>) " using ChopOrEqvRule by blast
 have  3: "\<turnstile> f;empty \<equiv>\<^sub>i f" using ChopEmpty by blast
 from 2 3 show ?thesis by simp 
qed

lemma ImpCS:
 "\<turnstile>   f \<supset>\<^sub>i f\<^sup>\<star> "
proof -
 have  1: "\<turnstile> f \<supset>\<^sub>i f;f\<^sup>\<star> " by (rule ImpChopPlus)
 hence 2: "\<turnstile> f \<supset>\<^sub>i  empty  \<or>\<^sub>i  f;f\<^sup>\<star> " by auto
 from 2 show ?thesis using CSEqvOrChopCS using prop15 by blast 
qed 

lemma CSChopImpCS:
 "\<turnstile>   f\<^sup>\<star>; f \<supset>\<^sub>i f\<^sup>\<star> "
proof -
 have  1: "\<turnstile> f \<supset>\<^sub>i f\<^sup>\<star> " by (rule ImpCS)
 hence 2: "\<turnstile> f\<^sup>\<star>; f \<supset>\<^sub>i f\<^sup>\<star>; f\<^sup>\<star> " by (rule RightChopImpChop)
 have  3: "\<turnstile> f\<^sup>\<star>; f\<^sup>\<star> \<supset>\<^sub>i f\<^sup>\<star> " by (rule CSChopCSImpCS)
 from 2 3 show ?thesis using prop02 by blast
qed

(* same as ChopCSImpCS *)
lemma ChopPlusImpCS:
 " \<turnstile>   f;f\<^sup>\<star>  \<supset>\<^sub>i f\<^sup>\<star> "
proof -
 have 1: "\<turnstile>   f;f\<^sup>\<star> \<supset>\<^sub>i empty  \<or>\<^sub>i f;f\<^sup>\<star>" by auto
 from 1 show ?thesis using CSEqvOrChopCS using prop15 by blast 
qed

lemma CSChopEqvOrChopPlusChop:
 "\<turnstile>   f\<^sup>\<star>; g \<equiv>\<^sub>i g \<or>\<^sub>i  (f;f\<^sup>\<star>) ; g "
proof -
 have 1: "\<turnstile> f\<^sup>\<star> \<equiv>\<^sub>i  empty  \<or>\<^sub>i  f;f\<^sup>\<star> " by (rule CSEqvOrChopCS)
 from 1 show ?thesis using EmptyOrChopEqvRule by blast
qed

lemma CSElim:
 assumes "\<turnstile>    empty  \<supset>\<^sub>i g " 
         "\<turnstile> (f \<and>\<^sub>i  more ); g \<supset>\<^sub>i g "
 shows   "\<turnstile> f\<^sup>\<star> \<supset>\<^sub>i g "
proof -
 have  1: "\<turnstile> f\<^sup>\<star> \<equiv>\<^sub>i  empty  \<or>\<^sub>i  (f \<and>\<^sub>i  more ); f\<^sup>\<star> " 
       by (rule ChopstarEqv)
 have  2: "\<turnstile>  empty  \<supset>\<^sub>i g " 
       using assms by blast
 have  3: "\<turnstile> (f \<and>\<^sub>i  more ); g \<supset>\<^sub>i g " 
       using assms by blast
 have 31: "\<turnstile>  \<not>\<^sub>i g  \<supset>\<^sub>i more " 
       using "2" by auto
 have 32: "\<turnstile> \<not>\<^sub>i g  \<supset>\<^sub>i \<not>\<^sub>i ((f \<and>\<^sub>i  more ); g)" 
       using "3" prop27  by blast
 have 33: "\<turnstile> f\<^sup>\<star> \<and>\<^sub>i more \<supset>\<^sub>i (f \<and>\<^sub>i  more ); f\<^sup>\<star> " 
       using "1" using CSAndMoreEqvAndMoreChop itl_prop(31) by blast
 have 34: "\<turnstile> f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i  g \<supset>\<^sub>i f\<^sup>\<star> \<and>\<^sub>i more " 
       using "31" by auto
 have 35: "\<turnstile> f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i  g \<supset>\<^sub>i (f \<and>\<^sub>i  more ); f\<^sup>\<star> " 
       using "33" "34" by auto
 have 36: "\<turnstile> f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i  g \<supset>\<^sub>i \<not>\<^sub>i ((f \<and>\<^sub>i  more ); g) " 
       using "32" by auto
 have  4: "\<turnstile> f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i  g \<supset>\<^sub>i (f \<and>\<^sub>i  more ); f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i ((f \<and>\<^sub>i more ); g) " 
       using "35" "36"  by auto
 have  5: "\<turnstile> f \<and>\<^sub>i  more  \<supset>\<^sub>i  more " 
       by auto
 from 4 5 show ?thesis using ChopContra by blast
qed


lemma CSCSImpCS:
 "\<turnstile>   (f\<^sup>\<star>)\<^sup>\<star> \<supset>\<^sub>i f\<^sup>\<star> "
proof -
 have 1: "\<turnstile>  empty  \<supset>\<^sub>i f\<^sup>\<star> " by (rule EmptyImpCS)
 have 2: "\<turnstile> (f\<^sup>\<star> \<and>\<^sub>i  more ); f\<^sup>\<star> \<supset>\<^sub>i f\<^sup>\<star>; f\<^sup>\<star> "  by (rule AndChopA)
 have 3: "\<turnstile> f\<^sup>\<star>; f\<^sup>\<star> \<supset>\<^sub>i f\<^sup>\<star> " by (rule CSChopCSImpCS)
 have 4: "\<turnstile> (f\<^sup>\<star> \<and>\<^sub>i  more ); f\<^sup>\<star> \<supset>\<^sub>i f\<^sup>\<star> " using "2" "3" prop02 by blast 
 from 1 4 show ?thesis using CSElim by blast
qed

lemma RightEmptyOrChopEqv:
 "\<turnstile>   g;( empty  \<or>\<^sub>i  f) \<equiv>\<^sub>i g \<or>\<^sub>i  (g; f) "
proof -
 have 1: "\<turnstile> g;( empty  \<or>\<^sub>i  f) \<equiv>\<^sub>i g;empty \<or>\<^sub>i g;f  " by (rule ChopOrEqv)
 have 2: "\<turnstile> g;empty  \<equiv>\<^sub>i g " by (rule ChopEmpty)
 from 1 2 show ?thesis by auto
qed

lemma RightEmptyOrChopEqvRule:
 assumes "\<turnstile> f \<equiv>\<^sub>i empty \<or>\<^sub>i f1"
 shows   "\<turnstile> g;f \<equiv>\<^sub>i g \<or>\<^sub>i (g;f1)"
proof -
 have  1: "\<turnstile> f \<equiv>\<^sub>i empty \<or>\<^sub>i f1 " using assms by auto
 hence 2: "\<turnstile> g;f \<equiv>\<^sub>i g;(empty \<or>\<^sub>i f1) " by (rule RightChopEqvChop)
 have  3: "\<turnstile> g;(empty \<or>\<^sub>i f1) \<equiv>\<^sub>i g \<or>\<^sub>i (g;f1) " by (rule RightEmptyOrChopEqv)
 from 2 3 show ?thesis by auto
qed

lemma ChopPlusEqvOrChopChopPlus:
 "\<turnstile>   (f;f\<^sup>\<star>)  \<equiv>\<^sub>i f \<or>\<^sub>i  f; (f;f\<^sup>\<star>)  "
proof -
 have 1: "\<turnstile> f\<^sup>\<star> \<equiv>\<^sub>i  empty  \<or>\<^sub>i  f;f\<^sup>\<star> " by (rule CSEqvOrChopCS)
 from 1 show ?thesis by (rule RightEmptyOrChopEqvRule)
qed

lemma CSAndEmptyEqvEmpty:
 "\<turnstile> (f\<^sup>\<star>) \<and>\<^sub>i empty \<equiv>\<^sub>i empty"
using EmptyImpCS by auto

lemma NotAndMoreChopAndEmpty:
 "\<turnstile> \<not>\<^sub>i(((f \<and>\<^sub>i more);g) \<and>\<^sub>i empty) "
by auto


lemma NotChopAndMoreAndEmpty:
 "\<turnstile> \<not>\<^sub>i((f;(g\<and>\<^sub>i more)) \<and>\<^sub>i empty) "
by auto

lemma ChopCsAndEmptyEqvAndEmpty:
  "\<turnstile> ((f;f\<^sup>\<star>) \<and>\<^sub>i empty) \<equiv>\<^sub>i (f \<and>\<^sub>i empty)"
proof -
 have 1: "\<turnstile> ((f;f\<^sup>\<star>) \<and>\<^sub>i empty) \<equiv>\<^sub>i (f\<and>\<^sub>i empty);(f\<^sup>\<star> \<and>\<^sub>i empty) " 
      using ChopAndEmptyEqvEmptyChopEmpty by blast
 have 2: "\<turnstile> (f\<and>\<^sub>i empty);(f\<^sup>\<star> \<and>\<^sub>i empty) \<equiv>\<^sub>i (f\<and>\<^sub>i empty);empty " 
      using CSAndEmptyEqvEmpty using RightChopEqvChop by blast
 have 3: "\<turnstile> (f\<and>\<^sub>i empty);empty \<equiv>\<^sub>i f\<and>\<^sub>i empty" 
      by (rule ChopEmpty)
 from 1 2 3 show ?thesis by auto
qed

lemma AndMoreChopAndMoreEqvAndMoreChop:
 "\<turnstile> (f \<and>\<^sub>i more);g \<and>\<^sub>i more \<equiv>\<^sub>i (f \<and>\<^sub>i more);g"
apply simp_all
using interval_prefix_length_good by auto

lemma ChopPlusEqv:
 "\<turnstile> (f;f\<^sup>\<star>)  \<equiv>\<^sub>i f \<or>\<^sub>i  (f\<and>\<^sub>i  more ); (f;f\<^sup>\<star>)"
proof -
 have  1: "\<turnstile> f\<^sup>\<star> \<equiv>\<^sub>i  empty  \<or>\<^sub>i  (f \<and>\<^sub>i  more ); f\<^sup>\<star> " 
       by (rule ChopstarEqv)
 have  2: "\<turnstile> f\<^sup>\<star> \<equiv>\<^sub>i  empty  \<or>\<^sub>i f;f\<^sup>\<star>" 
       by (rule CSEqvOrChopCS)
 hence 3: "\<turnstile>   empty  \<or>\<^sub>i  f;f\<^sup>\<star>  \<equiv>\<^sub>i empty  \<or>\<^sub>i  (f \<and>\<^sub>i  more );f\<^sup>\<star>  " 
       using "1" "2" prop21 by blast
 have  4: "\<turnstile> (f \<and>\<^sub>i  more );(f\<^sup>\<star>) \<equiv>\<^sub>i (f \<and>\<^sub>i  more );(empty  \<or>\<^sub>i  f;f\<^sup>\<star>) " 
       using "2" using RightChopEqvChop by blast 
 hence 5: "\<turnstile>   empty  \<or>\<^sub>i  f;f\<^sup>\<star>  \<equiv>\<^sub>i empty  \<or>\<^sub>i  (f \<and>\<^sub>i  more );(empty  \<or>\<^sub>i  f;f\<^sup>\<star>) " 
       using "3" "4" by auto
 have  6: "\<turnstile> (f \<and>\<^sub>i  more ); ( empty  \<or>\<^sub>i  f;f\<^sup>\<star> ) \<equiv>\<^sub>i  
             (f \<and>\<^sub>i  more );  empty  \<or>\<^sub>i  (f \<and>\<^sub>i  more ); (f;f\<^sup>\<star>)  " 
       using ChopOrEqv by blast
 have  7: "\<turnstile> (f \<and>\<^sub>i  more );  empty  \<equiv>\<^sub>i f \<and>\<^sub>i  more " 
       using ChopEmpty by blast
 have  8: "\<turnstile>  empty  \<or>\<^sub>i  f;f\<^sup>\<star>  \<equiv>\<^sub>i
              empty  \<or>\<^sub>i  (f \<and>\<^sub>i  more ) \<or>\<^sub>i  (f \<and>\<^sub>i  more ); (f;f\<^sup>\<star>)" 
       using "5" "6" "7" by auto
 have  9: "\<turnstile>  (empty  \<or>\<^sub>i  f;f\<^sup>\<star>) \<and>\<^sub>i more \<equiv>\<^sub>i  f;f\<^sup>\<star> \<and>\<^sub>i more " 
       by auto
 have 10: "\<turnstile> (empty  \<or>\<^sub>i  (f \<and>\<^sub>i  more ) \<or>\<^sub>i  (f \<and>\<^sub>i  more ); (f;f\<^sup>\<star>)) \<and>\<^sub>i more \<equiv>\<^sub>i
            ((f \<and>\<^sub>i  more ) \<or>\<^sub>i  (f \<and>\<^sub>i  more ); (f;f\<^sup>\<star>)) \<and>\<^sub>i more " 
       by auto  
 have 11: "\<turnstile> ((f \<and>\<^sub>i  more ) \<or>\<^sub>i  (f \<and>\<^sub>i  more ); (f;f\<^sup>\<star>)) \<and>\<^sub>i more \<equiv>\<^sub>i 
             (f \<and>\<^sub>i  more ) \<or>\<^sub>i  (f \<and>\<^sub>i  more ); (f;f\<^sup>\<star>)" 
       using AndMoreChopAndMoreEqvAndMoreChop 
       by (metis "10" RightEmptyOrChopEqv itl_prop(31) itl_prop(32) prop15)  
 have 12: "\<turnstile> f;f\<^sup>\<star> \<and>\<^sub>i more \<equiv>\<^sub>i (f \<and>\<^sub>i  more ) \<or>\<^sub>i  (f \<and>\<^sub>i  more ); (f;f\<^sup>\<star>) " 
       using "8" "9" "10" "11" by auto
 have 13: "\<turnstile> f;f\<^sup>\<star> \<and>\<^sub>i empty \<equiv>\<^sub>i f \<and>\<^sub>i empty " 
       by (rule ChopCsAndEmptyEqvAndEmpty)
 have 14: "\<turnstile> (f \<and>\<^sub>i  more ) \<or>\<^sub>i  (f \<and>\<^sub>i  more ); (f;f\<^sup>\<star>) \<or>\<^sub>i (f \<and>\<^sub>i empty) \<equiv>\<^sub>i
             f \<or>\<^sub>i  (f \<and>\<^sub>i  more );(f;f\<^sup>\<star>) " 
       by auto
 have 15: "\<turnstile> f;f\<^sup>\<star> \<equiv>\<^sub>i ( f;f\<^sup>\<star> \<and>\<^sub>i empty) \<or>\<^sub>i ( f;f\<^sup>\<star> \<and>\<^sub>i more)" 
       by auto
 from 11 12 13 14 15 show ?thesis by auto 
qed


lemma ChopPlusImpChopPlus:
 assumes "\<turnstile>   f \<supset>\<^sub>i g "
 shows "\<turnstile>  f;f\<^sup>\<star> \<supset>\<^sub>i g;g\<^sup>\<star> " 
proof -
 have 1: "\<turnstile> f \<supset>\<^sub>i g " 
      using assms by auto
 have 2: "\<turnstile> f;f\<^sup>\<star>  \<equiv>\<^sub>i f \<or>\<^sub>i  (f\<and>\<^sub>i  more ); (f;f\<^sup>\<star>) " 
      by (rule ChopPlusEqv)
 have 3: "\<turnstile> g;g\<^sup>\<star>  \<equiv>\<^sub>i g \<or>\<^sub>i  (g \<and>\<^sub>i  more );(g;g\<^sup>\<star>) " 
      by (rule ChopPlusEqv)
 have 4: "\<turnstile>  f;f\<^sup>\<star>  \<and>\<^sub>i \<not>\<^sub>i (g;g\<^sup>\<star> ) \<supset>\<^sub>i ((f\<and>\<^sub>i  more ); (f;f\<^sup>\<star>) ) \<and>\<^sub>i \<not>\<^sub>i ((g \<and>\<^sub>i  more ); (g;g\<^sup>\<star>) ) " 
      using "1" "2" "3" by auto
 have 5: "\<turnstile> f \<and>\<^sub>i  more  \<supset>\<^sub>i g \<and>\<^sub>i  more " using "1" 
      by auto
 have 6: "\<turnstile> (f\<and>\<^sub>i  more ); (f;f\<^sup>\<star>)  \<supset>\<^sub>i (g \<and>\<^sub>i  more ); (f;f\<^sup>\<star>) " 
      using "5" by (rule LeftChopImpChop)
 have 7: "\<turnstile> f;f\<^sup>\<star>  \<and>\<^sub>i \<not>\<^sub>i (g;g\<^sup>\<star> ) \<supset>\<^sub>i
            ((g\<and>\<^sub>i  more ); (f;f\<^sup>\<star>) ) \<and>\<^sub>i \<not>\<^sub>i ((g \<and>\<^sub>i  more ); (g;g\<^sup>\<star>) )  " 
      using "4" "6" by auto
 have 8: "\<turnstile> g \<and>\<^sub>i  more  \<supset>\<^sub>i  more " 
      by auto
 from 7 8 show ?thesis using ChopContra by blast
qed

lemma ChopChopPlusImpChopPlus:
 "\<turnstile>   f; (f;f\<^sup>\<star>)  \<supset>\<^sub>i f;f\<^sup>\<star> "
proof -
 have  1: "\<turnstile>  empty  \<or>\<^sub>i   more " by auto
 hence 2: "\<turnstile> f \<supset>\<^sub>i  empty  \<or>\<^sub>i  (f \<and>\<^sub>i  more ) " by auto
 hence 3: "\<turnstile> f; (f;f\<^sup>\<star>)   \<supset>\<^sub>i (f;f\<^sup>\<star>)  \<or>\<^sub>i  (f \<and>\<^sub>i  more );(f;f\<^sup>\<star>) " by (rule EmptyOrChopImpRule)
 have  4: "\<turnstile>  f;f\<^sup>\<star> \<equiv>\<^sub>i f \<or>\<^sub>i  (f\<and>\<^sub>i  more ); (f;f\<^sup>\<star>) " by (rule ChopPlusEqv)
 hence 5: "\<turnstile> (f\<and>\<^sub>i  more ); (f;f\<^sup>\<star>)  \<supset>\<^sub>i f;f\<^sup>\<star> " by auto
 from 3 5 show ?thesis using ChopPlusImpCS RightChopImpChop by blast
qed

lemma CSImpCS:
 assumes "\<turnstile>   f \<supset>\<^sub>i g "
 shows   "\<turnstile> f\<^sup>\<star> \<supset>\<^sub>i g\<^sup>\<star> "
proof -
 have  1: "\<turnstile> f \<supset>\<^sub>i g  " using assms by auto
 hence 2: "\<turnstile> f;f\<^sup>\<star> \<supset>\<^sub>i g;g\<^sup>\<star> " by (rule ChopPlusImpChopPlus)
 hence 3: "\<turnstile>  empty  \<or>\<^sub>i  f;f\<^sup>\<star>  \<supset>\<^sub>i  empty  \<or>\<^sub>i   g;g\<^sup>\<star> " by auto
 from 2 3 show ?thesis using CSEqvOrChopCS prop14 prop15 by blast 
qed

lemma ChopPlusIntro:
 assumes "\<turnstile>   f \<and>\<^sub>i \<not>\<^sub>i  g \<supset>\<^sub>i (g \<and>\<^sub>i  more ); f "
 shows   "\<turnstile> f \<supset>\<^sub>i g;g\<^sup>\<star> "
proof -
 have 1: "\<turnstile> f \<and>\<^sub>i \<not>\<^sub>i  g \<supset>\<^sub>i (g \<and>\<^sub>i  more ); f " using assms by auto
 have 2: "\<turnstile> g;g\<^sup>\<star>  \<equiv>\<^sub>i g \<or>\<^sub>i  (g \<and>\<^sub>i  more ); (g;g\<^sup>\<star>) " by (rule ChopPlusEqv)
 have 3: "\<turnstile> f \<and>\<^sub>i \<not>\<^sub>i (g;g\<^sup>\<star> ) \<supset>\<^sub>i
          (g \<and>\<^sub>i  more ); f \<and>\<^sub>i \<not>\<^sub>i ((g \<and>\<^sub>i  more ); (g;g\<^sup>\<star>) ) " using "1" "2" by auto
 have 4: "\<turnstile> g \<and>\<^sub>i  more  \<supset>\<^sub>i  more " by auto
 from 3 4 show ?thesis using ChopContra by blast
qed

lemma ChopPlusElim:
 assumes "\<turnstile>   f \<supset>\<^sub>i g "
         "\<turnstile> (f\<and>\<^sub>i  more ); g \<supset>\<^sub>i g"
 shows   "\<turnstile> f;f\<^sup>\<star>  \<supset>\<^sub>i g "
proof -
 have   1: "\<turnstile> f;f\<^sup>\<star>  \<equiv>\<^sub>i f \<or>\<^sub>i  (f\<and>\<^sub>i  more ); (f;f\<^sup>\<star>) " by (rule ChopPlusEqv)
 have   2: "\<turnstile> f \<supset>\<^sub>i g " using assms by blast
 hence 21: "\<turnstile> \<not>\<^sub>i g \<supset>\<^sub>i \<not>\<^sub>i f " by auto
 have   3: "\<turnstile> (f\<and>\<^sub>i  more ); g \<supset>\<^sub>i g " using assms by blast
 hence 31: "\<turnstile> \<not>\<^sub>i  g \<supset>\<^sub>i \<not>\<^sub>i ((f\<and>\<^sub>i more ); g) "using prop27 by blast 
 hence 32: "\<turnstile> f;f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i  g \<supset>\<^sub>i \<not>\<^sub>i ((f\<and>\<^sub>i more ); g) " by auto
 have  33: "\<turnstile>f;f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i  g \<supset>\<^sub>i (f \<and>\<^sub>i  more ); (f;f\<^sup>\<star>)" using "1" "21" by auto 
 have   4: "\<turnstile>  f;f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i  g \<supset>\<^sub>i
               (f \<and>\<^sub>i  more ); (f;f\<^sup>\<star>)  \<and>\<^sub>i \<not>\<^sub>i ((f\<and>\<^sub>i more ); g) " using "31" "33" by auto
 have   5: "\<turnstile> f \<and>\<^sub>i  more  \<supset>\<^sub>i  more "  by auto
 from  4  5 show ?thesis using ChopContra by blast
qed

lemma ChopPlusElimWithoutMore:
 assumes "\<turnstile>   f \<supset>\<^sub>i g"
         "\<turnstile> f; g \<supset>\<^sub>i g " 
 shows   "\<turnstile> f;f\<^sup>\<star>  \<supset>\<^sub>i g "
proof -
 have 1: "\<turnstile> f \<supset>\<^sub>i g " using assms by blast
 have 2: "\<turnstile> (f; g) \<supset>\<^sub>i g " using assms by blast
 have 3: "\<turnstile> (f \<and>\<^sub>i  more ); g \<supset>\<^sub>i f; g " by (rule AndChopA)
 have 4: "\<turnstile> (f \<and>\<^sub>i  more ); g \<supset>\<^sub>i g " using "2" "3" prop02  by blast
 from 1 4 show ?thesis using ChopPlusElim by blast
qed

lemma ChopPlusEqvChopPlus:
 assumes "\<turnstile>   f \<equiv>\<^sub>i g "
 shows   "\<turnstile> f;f\<^sup>\<star>  \<equiv>\<^sub>i g;g\<^sup>\<star> "
proof -
 have  1: "\<turnstile> f \<equiv>\<^sub>i g " using assms by auto
 hence 2: "\<turnstile> f \<supset>\<^sub>i g " by auto
 hence 3: "\<turnstile> f;f\<^sup>\<star>  \<supset>\<^sub>i g;g\<^sup>\<star> " by (rule ChopPlusImpChopPlus)
 have  4: "\<turnstile> g \<supset>\<^sub>i f " using "1" by auto
 hence 5: "\<turnstile>  g;g\<^sup>\<star> \<supset>\<^sub>i f;f\<^sup>\<star> " by (rule ChopPlusImpChopPlus)
 from 3 5 show ?thesis using itl_prop(31) by blast
qed

lemma CSEqvCS: 
 assumes "\<turnstile>   f \<equiv>\<^sub>i g "
 shows   "\<turnstile> f\<^sup>\<star> \<equiv>\<^sub>i g\<^sup>\<star> "
proof -
 have  1: "\<turnstile> f \<equiv>\<^sub>i g " using assms by auto
 hence 2: "\<turnstile> f;f\<^sup>\<star>  \<equiv>\<^sub>i g;g\<^sup>\<star>" by (rule ChopPlusEqvChopPlus)
 hence 3: "\<turnstile>  empty  \<or>\<^sub>i f;f\<^sup>\<star>   \<equiv>\<^sub>i  empty  \<or>\<^sub>i  g;g\<^sup>\<star> " by auto
 from 3 show ?thesis using CSEqvOrChopCS using assms by auto
qed

lemma AndCSA:
 "\<turnstile>   (f \<and>\<^sub>i g)\<^sup>\<star> \<supset>\<^sub>i f\<^sup>\<star> "
proof -
 have 1: "\<turnstile> f \<and>\<^sub>i g \<supset>\<^sub>i f " by auto
 from 1 show ?thesis using CSImpCS by blast
qed

lemma AndCSB:
 "\<turnstile>   (f \<and>\<^sub>i g)\<^sup>\<star> \<supset>\<^sub>i g\<^sup>\<star> "
proof -
 have 1: "\<turnstile> f \<and>\<^sub>i g \<supset>\<^sub>i g " by auto
 from 1 show ?thesis using CSImpCS by blast
qed




lemma CSIntro:
 assumes "\<turnstile>   f \<and>\<^sub>i  more  \<supset>\<^sub>i (g \<and>\<^sub>i  more ); f "
 shows   "\<turnstile> f \<supset>\<^sub>i g\<^sup>\<star> " 
proof -
 have   1: "\<turnstile> f \<and>\<^sub>i  more  \<supset>\<^sub>i (g \<and>\<^sub>i  more ); f" 
        using assms by auto
 have   2: "\<turnstile>  more  \<equiv>\<^sub>i \<not>\<^sub>i  empty " 
        by auto
 have   3: "\<turnstile> f \<and>\<^sub>i \<not>\<^sub>i  empty  \<supset>\<^sub>i (g \<and>\<^sub>i  more ); f " 
        using "1" "2" by auto
 have   4: "\<turnstile> g\<^sup>\<star> \<equiv>\<^sub>i  empty  \<or>\<^sub>i  (g \<and>\<^sub>i  more ); g\<^sup>\<star> " 
        by (rule ChopstarEqv)
 hence 41: "\<turnstile> \<not>\<^sub>i(empty  \<or>\<^sub>i  (g \<and>\<^sub>i  more ); g\<^sup>\<star>) \<equiv>\<^sub>i \<not>\<^sub>iempty \<and>\<^sub>i \<not>\<^sub>i((g \<and>\<^sub>i  more ); g\<^sup>\<star>) " 
        using prop20 prop21 by blast
 have  411: "\<turnstile> \<not>\<^sub>iempty \<and>\<^sub>i \<not>\<^sub>i((g \<and>\<^sub>i  more ); g\<^sup>\<star>) \<equiv>\<^sub>i more \<and>\<^sub>i \<not>\<^sub>i((g \<and>\<^sub>i  more ); g\<^sup>\<star>)" 
        using NotEmptyEqvMore using prop06 by blast
 have  42: "\<turnstile> \<not>\<^sub>i( g\<^sup>\<star>)  \<equiv>\<^sub>i more \<and>\<^sub>i \<not>\<^sub>i((g \<and>\<^sub>i  more ); g\<^sup>\<star>)" 
        using "4" "41" "411" prop01 prop03 by blast
 have  43: "\<turnstile>f \<and>\<^sub>i \<not>\<^sub>i( g\<^sup>\<star>) \<supset>\<^sub>i f \<and>\<^sub>i more \<and>\<^sub>i \<not>\<^sub>i ((g \<and>\<^sub>i  more ); g\<^sup>\<star>) " 
        using "42" using itl_prop(31) prop12 by blast
 have  44: "\<turnstile> f \<and>\<^sub>i more \<and>\<^sub>i \<not>\<^sub>i ((g \<and>\<^sub>i  more ); g\<^sup>\<star>) \<supset>\<^sub>i (g \<and>\<^sub>i  more ); f \<and>\<^sub>i \<not>\<^sub>i ((g \<and>\<^sub>i  more ); g\<^sup>\<star>) " 
        using "3" "43" by auto
 have   5: "\<turnstile> f \<and>\<^sub>i \<not>\<^sub>i ( g\<^sup>\<star>) \<supset>\<^sub>i
              (g \<and>\<^sub>i  more ); f \<and>\<^sub>i \<not>\<^sub>i ((g \<and>\<^sub>i  more ); g\<^sup>\<star>) " 
        using "43" "44" prop02 by auto
 have   6: "\<turnstile> g \<and>\<^sub>i  more  \<supset>\<^sub>i  more  " 
        by auto
 from 5 6 show ?thesis using   ChopContra by blast
qed

lemma CSElimWithoutMore: 
 assumes "\<turnstile>    empty  \<supset>\<^sub>i g "
         "\<turnstile> f; g \<supset>\<^sub>i g "
 shows   "\<turnstile> f\<^sup>\<star> \<supset>\<^sub>i g "
proof -
 have 1: "\<turnstile>  empty  \<supset>\<^sub>i g" using assms by blast
 have 2: "\<turnstile> f; g \<supset>\<^sub>i g " using assms by blast
 have 3: "\<turnstile> (f \<and>\<^sub>i  more ); g \<supset>\<^sub>i f; g " by (rule AndChopA)
 have 4: "\<turnstile> (f \<and>\<^sub>i  more ); g \<supset>\<^sub>i g " using "2" "3" prop02 by blast
 from 1 4 show ?thesis using CSElim by blast
qed

lemma ChopAssocB:
 "\<turnstile> (f;g);h \<equiv>\<^sub>i f;(g;h)"
using ChopAssoc itl_prop(30) by blast

lemma CSChopEqvChopOrRule:
 assumes "\<turnstile>   f\<equiv>\<^sub>i (g\<^sup>\<star>; h) "
 shows   "\<turnstile> f \<equiv>\<^sub>i (g; f) \<or>\<^sub>i  h "
proof -
 have   1: "\<turnstile> f \<equiv>\<^sub>i (g\<^sup>\<star>; h) " using assms by auto
 have   2: "\<turnstile> g\<^sup>\<star> \<equiv>\<^sub>i  empty  \<or>\<^sub>i  (g; g\<^sup>\<star>) " by (rule CSEqvOrChopCS)
 hence  3: "\<turnstile> g\<^sup>\<star>; h \<equiv>\<^sub>i h \<or>\<^sub>i  ((g; g\<^sup>\<star>); h) " by (rule EmptyOrChopEqvRule)
 have   4: "\<turnstile> (g; g\<^sup>\<star>); h \<equiv>\<^sub>i  g; (g\<^sup>\<star>; h)   " by (rule ChopAssocB)
 hence 41: "\<turnstile> g\<^sup>\<star>; h \<equiv>\<^sub>i h \<or>\<^sub>i g; (g\<^sup>\<star>; h)" using "3" by auto
 have   5: "\<turnstile> g; f \<equiv>\<^sub>i g; (g\<^sup>\<star>; h) " using "1" by (rule RightChopEqvChop)
 hence  6: "\<turnstile> (g\<^sup>\<star>; h) \<equiv>\<^sub>i h \<or>\<^sub>i  g; f" using "41" by auto
 hence 61: "\<turnstile> (g\<^sup>\<star>; h) \<equiv>\<^sub>i (g; f) \<or>\<^sub>i h " by auto
 from 1 61 show ?thesis  using prop03 by blast
qed

lemma CSChopIntroRule:
 assumes "\<turnstile>   f \<and>\<^sub>i \<not>\<^sub>i  h \<supset>\<^sub>i g; f "
         "\<turnstile> g \<supset>\<^sub>i  more "
 shows   "\<turnstile> f \<supset>\<^sub>i g\<^sup>\<star>; h"
proof -
 have  1: "\<turnstile> f \<and>\<^sub>i \<not>\<^sub>i  h \<supset>\<^sub>i g; f " using assms by blast
 have  2: "\<turnstile> g \<supset>\<^sub>i  more " using assms by blast
 hence 3: "\<turnstile> g \<supset>\<^sub>i g \<and>\<^sub>i  more " by auto
 hence 4: "\<turnstile> g; f \<supset>\<^sub>i (g \<and>\<^sub>i  more ); f " by (rule LeftChopImpChop)
 have  5: "\<turnstile> f \<supset>\<^sub>i (g \<and>\<^sub>i  more ); f \<or>\<^sub>i  h " using "1" "4" by auto
 have  6: "\<turnstile> g\<^sup>\<star> \<equiv>\<^sub>i  empty  \<or>\<^sub>i  (g \<and>\<^sub>i  more ); g\<^sup>\<star> " by (rule ChopstarEqv)
 hence 7: "\<turnstile> (g\<^sup>\<star>); h \<equiv>\<^sub>i h \<or>\<^sub>i  ((g \<and>\<^sub>i  more ); g\<^sup>\<star>); h " by (rule EmptyOrChopEqvRule)
 have  8: "\<turnstile> ((g \<and>\<^sub>i  more ); g\<^sup>\<star>); h \<equiv>\<^sub>i (g \<and>\<^sub>i  more ); (g\<^sup>\<star>; h) " by (rule ChopAssocB)
 have  9: "\<turnstile> (g\<^sup>\<star>); h \<equiv>\<^sub>i h \<or>\<^sub>i  (g \<and>\<^sub>i  more ); (g\<^sup>\<star>; h) " using "7" "8" by auto
 have 10: "\<turnstile> f \<and>\<^sub>i \<not>\<^sub>i (g\<^sup>\<star>; h) \<supset>\<^sub>i (g \<and>\<^sub>i more); f \<and>\<^sub>i \<not>\<^sub>i ((g \<and>\<^sub>i more); (g\<^sup>\<star>; h)) " using "5" "9" by auto
 have 11: "\<turnstile> g \<and>\<^sub>i  more  \<supset>\<^sub>i  more " by auto
 from 10 11 show ?thesis using ChopContra by blast
qed


 
lemma DiamondAndEmptyEqvAndEmpty:
 "\<turnstile> \<diamond> f \<and>\<^sub>i empty \<equiv>\<^sub>i f \<and>\<^sub>i empty"
proof -
 have 1: "\<turnstile> \<diamond> f \<and>\<^sub>i empty \<supset>\<^sub>i f \<and>\<^sub>i empty " by auto
 have 2: "\<turnstile> f \<and>\<^sub>i empty \<supset>\<^sub>i \<diamond> f \<and>\<^sub>i empty" by auto
 from 1 2 show ?thesis using itl_prop(31) by blast
qed

lemma InitAndEmptyEqvAndEmpty:
 "\<turnstile> (init w) \<and>\<^sub>i empty \<equiv>\<^sub>i w \<and>\<^sub>i empty "
proof -
 have 1: "\<turnstile> (init w) \<and>\<^sub>i empty \<equiv>\<^sub>i (w \<and>\<^sub>i empty);true\<^sub>i \<and>\<^sub>i empty" 
      by auto
 have 2: "\<turnstile> (w \<and>\<^sub>i empty);true\<^sub>i \<and>\<^sub>i empty \<equiv>\<^sub>i (w \<and>\<^sub>i empty);(true\<^sub>i \<and>\<^sub>i empty)" 
      using ChopAndEmptyEqvEmptyChopEmpty by auto
 have 3: "\<turnstile> (w \<and>\<^sub>i empty);(true\<^sub>i \<and>\<^sub>i empty) \<equiv>\<^sub>i (w \<and>\<^sub>i empty);empty" 
      using RightChopEqvChop itl_prop(17) by blast
 have 4: "\<turnstile> (w \<and>\<^sub>i empty);empty \<equiv>\<^sub>i w \<and>\<^sub>i empty" 
      using ChopEmpty by blast
 from 1 2 3 4 show ?thesis by auto
qed

lemma InitAndNotBoxInitImpNotEmpty:
 "\<turnstile> init w \<and>\<^sub>i \<not>\<^sub>i( \<box> (init w)) \<supset>\<^sub>i \<not>\<^sub>i  empty" 
proof -
 have 1: "\<turnstile> (init w) \<and>\<^sub>i empty \<equiv>\<^sub>i w \<and>\<^sub>i empty " by (rule InitAndEmptyEqvAndEmpty)
 have 2: "\<turnstile> \<not>\<^sub>i( \<box> (init w)) \<and>\<^sub>i empty \<equiv>\<^sub>i \<diamond> \<not>\<^sub>i (init w) \<and>\<^sub>i empty" by auto
 have 3: "\<turnstile> \<diamond> \<not>\<^sub>i (init w) \<and>\<^sub>i empty \<equiv>\<^sub>i \<not>\<^sub>i (init w) \<and>\<^sub>i empty" by auto
 have 4: "\<turnstile> \<not>\<^sub>i (init w) \<equiv>\<^sub>i (init \<not>\<^sub>i w) " by auto
 have 5: "\<turnstile> \<not>\<^sub>i (init w) \<and>\<^sub>i empty \<equiv>\<^sub>i \<not>\<^sub>i w \<and>\<^sub>i empty" using "4" InitAndEmptyEqvAndEmpty by auto
 have 6: "\<turnstile> \<not>\<^sub>i( \<box> (init w)) \<and>\<^sub>i empty \<equiv>\<^sub>i \<not>\<^sub>i w \<and>\<^sub>i empty " using "2" "3" "5" prop03 by blast
 have 7: "\<turnstile> \<not>\<^sub>i(init w \<and>\<^sub>i \<not>\<^sub>i( \<box> (init w)) \<and>\<^sub>i empty) " using "1" "6" by auto  
 from 7 show ?thesis by auto
qed

lemma BoxImpTrueChopAndEmpty: 
 "\<turnstile> \<box> f \<supset>\<^sub>i true\<^sub>i;(f \<and>\<^sub>i empty) "
by auto

lemma BoxInitAndMoreImpBoxInitAndMoreAndFinInit:
 "\<turnstile> \<box>( init w) \<and>\<^sub>i  more  \<supset>\<^sub>i (\<box> (init w) \<and>\<^sub>i  more ) \<and>\<^sub>i  fin ( init w) "
proof -
 have 1: "\<turnstile> fin ( init w) \<equiv>\<^sub>i true\<^sub>i ; (init w \<and>\<^sub>i empty)" using FinEqvTrueChopAndEmpty by blast
 have 2: "\<turnstile> \<box>( init w) \<supset>\<^sub>i true\<^sub>i;(init w \<and>\<^sub>i empty) " by (rule BoxImpTrueChopAndEmpty)
 from 1 2 show ?thesis by auto
qed

lemma CSImpBox:
 assumes "\<turnstile> f \<supset>\<^sub>i  empty  \<or>\<^sub>i  (\<box> (init w) \<and>\<^sub>i  more ); f "
 shows   "\<turnstile> init w \<and>\<^sub>i f \<supset>\<^sub>i \<box> (init w) "
proof -
 have  1: "\<turnstile> f \<supset>\<^sub>i  empty  \<or>\<^sub>i  (\<box>( init w) \<and>\<^sub>i  more ); f " 
       using assms by auto
 have  2: "\<turnstile> init w \<and>\<^sub>i \<not>\<^sub>i( \<box> (init w)) \<supset>\<^sub>i \<not>\<^sub>i  empty " 
       by (rule InitAndNotBoxInitImpNotEmpty)
 have  3: "\<turnstile> init w \<and>\<^sub>i f \<and>\<^sub>i \<not>\<^sub>i( \<box> (init w)) \<supset>\<^sub>i (\<box> (init w) \<and>\<^sub>i  more ); f " 
       using "1" "2" by auto
 have  4: "\<turnstile> \<box> (init w) \<and>\<^sub>i  more  \<supset>\<^sub>i (\<box> (init w) \<and>\<^sub>i  more ) \<and>\<^sub>i  fin ( init w) " 
       by (rule BoxInitAndMoreImpBoxInitAndMoreAndFinInit)
 hence 5: "\<turnstile> (\<box>( init w) \<and>\<^sub>i  more ); f \<supset>\<^sub>i ((\<box> (init w) \<and>\<^sub>i  more ) \<and>\<^sub>i  fin ( init w) ); f " 
       by (rule LeftChopImpChop)
 have  6: "\<turnstile> ((\<box> (init w) \<and>\<^sub>i  more ) \<and>\<^sub>i  fin ( init w) ); f \<equiv>\<^sub>i
              (\<box>( init w) \<and>\<^sub>i  more ); (init w \<and>\<^sub>i f) " 
       by (rule AndFinChopEqvStateAndChop)
 have  7: "\<turnstile> \<not>\<^sub>i( \<box>( init w)) \<supset>\<^sub>i (\<box> (init w)) yields \<not>\<^sub>i( \<box> (init w)) " 
       by (rule NotBoxStateImpBoxYieldsNotBox)
 have  8: "\<turnstile> (\<box>( init w)) yields \<not>\<^sub>i (\<box> (init w)) \<supset>\<^sub>i 
             (\<box> (init w) \<and>\<^sub>i  more ) yields \<not>\<^sub>i( \<box>( init w)) " 
       by (rule AndYieldsA)
 have  9: "\<turnstile> (\<box>( init w) \<and>\<^sub>i  more ); (init w \<and>\<^sub>i f) \<and>\<^sub>i (\<box>( init w) \<and>\<^sub>i  more ) yields \<not>\<^sub>i( \<box> (init w))
             \<supset>\<^sub>i  
             (\<box> (init w) \<and>\<^sub>i  more ); ((init w \<and>\<^sub>i f) \<and>\<^sub>i \<not>\<^sub>i (\<box> (init w))) " 
       by (rule ChopAndYieldsImp)
 have 10: "\<turnstile> (init w \<and>\<^sub>i f) \<and>\<^sub>i \<not>\<^sub>i( \<box> (init w)) \<supset>\<^sub>i
             (\<box>( init w) \<and>\<^sub>i  more ); ((init w \<and>\<^sub>i f) \<and>\<^sub>i \<not>\<^sub>i( \<box> (init w))) " 
       using "3" "5" "6" "7" "8" "9" by auto
 have 11: "\<turnstile> (\<box>( init w) \<and>\<^sub>i  more ); ((init w \<and>\<^sub>i f) \<and>\<^sub>i \<not>\<^sub>i( \<box> (init w))) \<supset>\<^sub>i
              more ; ((init w \<and>\<^sub>i f) \<and>\<^sub>i \<not>\<^sub>i( \<box> (init w)) ) " 
       by (rule AndChopB)
 have 12: "\<turnstile> (init w \<and>\<^sub>i f) \<and>\<^sub>i \<not>\<^sub>i( \<box> (init w)) \<supset>\<^sub>i 
             more ; ((init w \<and>\<^sub>i f) \<and>\<^sub>i \<not>\<^sub>i( \<box> (init w)) ) " 
       using "10" "11" by auto
 from 12 show ?thesis using MoreChopContra by blast
qed

lemma BoxCSEqvBox:
 "\<turnstile>   init w \<and>\<^sub>i (\<box>( init w))\<^sup>\<star> \<equiv>\<^sub>i \<box> (init w) "
proof -
 have  1: "\<turnstile> (\<box> (init w))\<^sup>\<star> \<equiv>\<^sub>i  empty  \<or>\<^sub>i  (\<box> (init w) \<and>\<^sub>i  more ); (\<box> (init w))\<^sup>\<star> " 
       by (rule ChopstarEqv)
 hence 2: "\<turnstile> (\<box> (init w))\<^sup>\<star> \<supset>\<^sub>i  empty  \<or>\<^sub>i  (\<box> (init w) \<and>\<^sub>i  more ); (\<box> (init w))\<^sup>\<star> " 
       using itl_prop(31) by blast
 hence 3: "\<turnstile> init w \<and>\<^sub>i (\<box> (init w))\<^sup>\<star> \<supset>\<^sub>i \<box> (init w) " 
       by (rule CSImpBox)
 have 11: "\<turnstile> \<box> (init w) \<supset>\<^sub>i (init w) " 
       by auto
 have 12: "\<turnstile> \<box>( init w) \<supset>\<^sub>i (\<box> (init w))\<^sup>\<star> " 
       by (rule ImpCS)
 have 13: "\<turnstile> \<box> (init w) \<supset>\<^sub>i init w \<and>\<^sub>i (\<box> (init w))\<^sup>\<star> " 
       using "11" "12" using itl_prop(32) by blast
 from 3 13 show ?thesis using itl_prop(31) by blast
qed

lemma BoxStateAndCSEqvCS:
 "\<turnstile>   \<box>( init w) \<and>\<^sub>i f\<^sup>\<star> \<equiv>\<^sub>i init w \<and>\<^sub>i (\<box>( init w) \<and>\<^sub>i f)\<^sup>\<star> "
proof -
 have  1: "\<turnstile> \<box> (init w) \<supset>\<^sub>i init w " by auto
 have  2: "\<turnstile> f\<^sup>\<star> \<and>\<^sub>i  more  \<equiv>\<^sub>i (f \<and>\<^sub>i  more ); f\<^sup>\<star>" by (rule CSAndMoreEqvAndMoreChop)
 have  3: "\<turnstile> \<box>( init w) \<and>\<^sub>i ((f \<and>\<^sub>i  more ); f\<^sup>\<star>) \<equiv>\<^sub>i
            (\<box> (init w) \<and>\<^sub>i f \<and>\<^sub>i  more ); (\<box> (init w) \<and>\<^sub>i f\<^sup>\<star>) " by (rule BoxStateAndChopEqvChop)
 have  4: "\<turnstile> \<box> (init w) \<and>\<^sub>i f \<and>\<^sub>i  more  \<supset>\<^sub>i (\<box> (init w) \<and>\<^sub>i f) \<and>\<^sub>i  more " by auto
 hence 5: "\<turnstile> (\<box> (init w) \<and>\<^sub>i f \<and>\<^sub>i  more ); (\<box> (init w) \<and>\<^sub>i f\<^sup>\<star>) \<supset>\<^sub>i  
            ((\<box> (init w) \<and>\<^sub>i f) \<and>\<^sub>i  more ); (\<box> (init w) \<and>\<^sub>i f\<^sup>\<star>) " by (rule LeftChopImpChop)
 have  6: "\<turnstile> (\<box>( init w) \<and>\<^sub>i f\<^sup>\<star>) \<and>\<^sub>i  more  \<supset>\<^sub>i
            ((\<box> (init w) \<and>\<^sub>i f) \<and>\<^sub>i  more ); (\<box> (init w) \<and>\<^sub>i f\<^sup>\<star>) " using "2" "3" "5" by auto
 hence 7: "\<turnstile> \<box> (init w) \<and>\<^sub>i f\<^sup>\<star> \<supset>\<^sub>i (\<box> (init w) \<and>\<^sub>i f)\<^sup>\<star> " by (rule CSIntro)
 have 71: "\<turnstile> init w \<and>\<^sub>i \<box> (init w) \<and>\<^sub>i f\<^sup>\<star> \<supset>\<^sub>i init w \<and>\<^sub>i (\<box> (init w) \<and>\<^sub>i f)\<^sup>\<star>" using "7" prop12 by blast 
 have  8: "\<turnstile> \<box>( init w) \<and>\<^sub>i f\<^sup>\<star> \<supset>\<^sub>i init w \<and>\<^sub>i (\<box> (init w) \<and>\<^sub>i f)\<^sup>\<star> " using "1" "71"  prop37 by blast
 have 11: "\<turnstile> (\<box> (init w) \<and>\<^sub>i f)\<^sup>\<star> \<supset>\<^sub>i (\<box> (init w))\<^sup>\<star> " by (rule AndCSA)
 have 12: "\<turnstile> init w \<and>\<^sub>i (\<box> (init w))\<^sup>\<star> \<equiv>\<^sub>i \<box> (init w)" by (rule BoxCSEqvBox)
 have 13: "\<turnstile> (\<box> (init w) \<and>\<^sub>i f)\<^sup>\<star> \<supset>\<^sub>i f\<^sup>\<star> " by (rule AndCSB)
 have 14: "\<turnstile> init w \<and>\<^sub>i (\<box> (init w) \<and>\<^sub>i f)\<^sup>\<star> \<supset>\<^sub>i init w \<and>\<^sub>i (\<box> (init w))\<^sup>\<star> \<and>\<^sub>i f\<^sup>\<star> " using "11" "13" by auto
 have 15: "\<turnstile> init w \<and>\<^sub>i (\<box> (init w))\<^sup>\<star> \<and>\<^sub>i f\<^sup>\<star> \<supset>\<^sub>i \<box> (init w) \<and>\<^sub>i f\<^sup>\<star> " using "12" by auto
 have 16: "\<turnstile> init w \<and>\<^sub>i (\<box> (init w) \<and>\<^sub>i f)\<^sup>\<star> \<supset>\<^sub>i \<box> (init w) \<and>\<^sub>i f\<^sup>\<star> " using "14" "15" prop02 by blast
 from 8 16 show ?thesis using itl_prop(31) by blast
qed

lemma BaCSImpCS:
 "\<turnstile>    ba (f \<supset>\<^sub>i g) \<supset>\<^sub>i f\<^sup>\<star> \<supset>\<^sub>i g\<^sup>\<star> "
proof -
 have   1: "\<turnstile> f\<^sup>\<star> \<equiv>\<^sub>i  empty  \<or>\<^sub>i  (f \<and>\<^sub>i  more ); f\<^sup>\<star> " 
        by (rule ChopstarEqv)
 have   2: "\<turnstile> g\<^sup>\<star> \<equiv>\<^sub>i  empty  \<or>\<^sub>i  (g \<and>\<^sub>i  more ); g\<^sup>\<star> " 
        by (rule ChopstarEqv)
 have  21: "\<turnstile> \<not>\<^sub>i(g\<^sup>\<star>) \<equiv>\<^sub>i  \<not>\<^sub>iempty  \<and>\<^sub>i \<not>\<^sub>i( (g \<and>\<^sub>i  more ); g\<^sup>\<star>)" 
        using "2" prop20 by blast
 hence 22: "\<turnstile> \<not>\<^sub>i(g\<^sup>\<star>) \<equiv>\<^sub>i  more  \<and>\<^sub>i \<not>\<^sub>i( (g \<and>\<^sub>i  more ); g\<^sup>\<star>) " 
        by (meson NotCSImpMore itl_prop(31) itl_prop(32) prop18 NotEmptyEqvMore)
 have   3: "\<turnstile> f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i (g\<^sup>\<star>) \<supset>\<^sub>i
            (empty  \<or>\<^sub>i (f \<and>\<^sub>i  more ); f\<^sup>\<star>) \<and>\<^sub>i more \<and>\<^sub>i \<not>\<^sub>i ((g \<and>\<^sub>i  more ); g\<^sup>\<star>) " 
        using "1" "22" prop22 by blast
 have  31: "\<turnstile> (empty  \<or>\<^sub>i (f \<and>\<^sub>i  more ); f\<^sup>\<star>) \<and>\<^sub>i more \<equiv>\<^sub>i (f \<and>\<^sub>i  more ); f\<^sup>\<star> \<and>\<^sub>i more" 
        by auto
 have  32: "\<turnstile> f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i (g\<^sup>\<star>) \<supset>\<^sub>i (f \<and>\<^sub>i  more ); f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i ((g \<and>\<^sub>i  more ); g\<^sup>\<star>) " 
        using "3" "31" by auto
 have   4: "\<turnstile> (f \<supset>\<^sub>i g) \<supset>\<^sub>i (f \<and>\<^sub>i  more  \<supset>\<^sub>i g \<and>\<^sub>i  more ) " 
        by auto
 hence  5: "\<turnstile>  ba (f \<supset>\<^sub>i g) \<supset>\<^sub>i  ba (f \<and>\<^sub>i  more  \<supset>\<^sub>i g \<and>\<^sub>i  more ) " 
        by (rule BaImpBa)
 have   6: "\<turnstile>  ba (f \<and>\<^sub>i  more  \<supset>\<^sub>i g \<and>\<^sub>i  more ) \<supset>\<^sub>i
              (f \<and>\<^sub>i  more ); f\<^sup>\<star> \<supset>\<^sub>i  (g \<and>\<^sub>i  more ); f\<^sup>\<star> " 
        by (rule BaLeftChopImpChop)
 have   7: "\<turnstile>  ba (f \<supset>\<^sub>i g) \<and>\<^sub>i (f \<and>\<^sub>i more ); f\<^sup>\<star> \<supset>\<^sub>i (g \<and>\<^sub>i  more ); f\<^sup>\<star> " 
        using "5" "6" by auto
 have   8: "\<turnstile> (g \<and>\<^sub>i  more ); f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i ((g \<and>\<^sub>i  more ); g\<^sup>\<star>)
           \<supset>\<^sub>i  (g \<and>\<^sub>i  more ); (f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i (g\<^sup>\<star>)) " 
        by (rule ChopAndNotChopImp)
 have   9: "\<turnstile> (g \<and>\<^sub>i  more ); (f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i (g\<^sup>\<star>)) \<supset>\<^sub>i  more ; (f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i (g\<^sup>\<star>)) " 
        by (rule AndChopB)
 have  10: "\<turnstile>  ba (f \<supset>\<^sub>i g) \<supset>\<^sub>i more ; (f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i (g\<^sup>\<star>)) \<supset>\<^sub>i  
              more ; ( ba (f \<supset>\<^sub>i g) \<and>\<^sub>i f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i (g\<^sup>\<star>)) " 
        by (rule BaChopImpChopBa)
 have  11: "\<turnstile>  ba (f \<supset>\<^sub>i g) \<and>\<^sub>i f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i (g\<^sup>\<star>) \<supset>\<^sub>i  
              more ; ( ba (f \<supset>\<^sub>i g) \<and>\<^sub>i f\<^sup>\<star> \<and>\<^sub>i \<not>\<^sub>i (g\<^sup>\<star>)) " 
        using "32" "7" "8" "9" "10" by auto
 hence 12: "\<turnstile> \<not>\<^sub>i ( (ba (f \<supset>\<^sub>i g)) \<and>\<^sub>i (f\<^sup>\<star>) \<and>\<^sub>i (\<not>\<^sub>i (g\<^sup>\<star>))) " 
        using MoreChopLoop by blast
 from 12 show ?thesis using prop04 MP itl_prop(31) by blast
qed

lemma BaCSEqvCS:
 "\<turnstile>    ba (f \<equiv>\<^sub>i g) \<supset>\<^sub>i (f\<^sup>\<star> \<equiv>\<^sub>i g\<^sup>\<star>)"
proof -
 have 1: "\<turnstile> ba (f \<equiv>\<^sub>i g) \<equiv>\<^sub>i ba (f \<supset>\<^sub>i g) \<and>\<^sub>i ba (g \<supset>\<^sub>i f)  " by auto
 have 2: "\<turnstile> ba (f \<supset>\<^sub>i g) \<supset>\<^sub>i (f\<^sup>\<star> \<supset>\<^sub>i g\<^sup>\<star>)  " by (rule BaCSImpCS)
 have 3: "\<turnstile> ba (g \<supset>\<^sub>i f) \<supset>\<^sub>i (g\<^sup>\<star> \<supset>\<^sub>i f\<^sup>\<star>) " by (rule BaCSImpCS)
 have 4: "\<turnstile> ba (f \<equiv>\<^sub>i g) \<supset>\<^sub>i (f\<^sup>\<star> \<supset>\<^sub>i g\<^sup>\<star>) \<and>\<^sub>i  (g\<^sup>\<star> \<supset>\<^sub>i f\<^sup>\<star>) " using "1" "2" "3" by auto
 have 5: "\<turnstile> (f\<^sup>\<star> \<supset>\<^sub>i g\<^sup>\<star>) \<and>\<^sub>i (g\<^sup>\<star> \<supset>\<^sub>i f\<^sup>\<star>) \<equiv>\<^sub>i (f\<^sup>\<star> \<equiv>\<^sub>i g\<^sup>\<star>) " by auto
 from 4 5 show ?thesis by auto
qed

lemma BaAndCSImport:
 "\<turnstile>    ba  f \<and>\<^sub>i g\<^sup>\<star> \<supset>\<^sub>i (f \<and>\<^sub>i g)\<^sup>\<star> "
proof -
 have  1: "\<turnstile> f \<supset>\<^sub>i (g \<supset>\<^sub>i f \<and>\<^sub>i g) " by auto
 hence 2: "\<turnstile>  ba  f \<supset>\<^sub>i  ba  (g \<supset>\<^sub>i f \<and>\<^sub>i g)" by (rule BaImpBa)
 have  3: "\<turnstile>  ba  (g \<supset>\<^sub>i f \<and>\<^sub>i g) \<supset>\<^sub>i g\<^sup>\<star> \<supset>\<^sub>i (f \<and>\<^sub>i g)\<^sup>\<star> " by (rule BaCSImpCS)
 from 2 3 show ?thesis by auto
qed

lemma CSSkip:
 "\<turnstile> skip\<^sup>\<star>"
by (metis ChopPlusImpCS EmptyImpCS EmptyNextInducta next_d_def)

subsection \<open>Properties of While\<close>

lemma WhileEqvIf:
 "\<turnstile> while  (init w)  do  f \<equiv>\<^sub>i  if\<^sub>i  (init w)  then  (f; ( while  (init w)  do  f))  else   empty "
proof -
 have  1: "\<turnstile>  while  (init w)  do  f \<equiv>\<^sub>i ((init w) \<and>\<^sub>i f)\<^sup>\<star> \<and>\<^sub>i  fin \<not>\<^sub>i  (init w) " 
       by (simp add: while_d_def)
 have  2: "\<turnstile> (init w \<and>\<^sub>i f)\<^sup>\<star> \<equiv>\<^sub>i  empty  \<or>\<^sub>i  ((init w \<and>\<^sub>i f); (init w \<and>\<^sub>i f)\<^sup>\<star>) " 
       by (rule CSEqvOrChopCS)
 have 21: "\<turnstile> ((init w) \<and>\<^sub>i f)\<^sup>\<star> \<and>\<^sub>i  fin \<not>\<^sub>i  (init w) \<equiv>\<^sub>i
             (empty  \<or>\<^sub>i  ((init w \<and>\<^sub>i f); (init w \<and>\<^sub>i f)\<^sup>\<star>)) \<and>\<^sub>i  fin \<not>\<^sub>i  (init w)  " 
       using "2" prop06 by blast
 have 22: "\<turnstile> (empty  \<or>\<^sub>i  ((init w \<and>\<^sub>i f); (init w \<and>\<^sub>i f)\<^sup>\<star>)) \<and>\<^sub>i  fin \<not>\<^sub>i  (init w) \<equiv>\<^sub>i 
            ( empty \<and>\<^sub>i fin \<not>\<^sub>i(init w)) \<or>\<^sub>i ( ((init w \<and>\<^sub>i f); (init w \<and>\<^sub>i f)\<^sup>\<star>) \<and>\<^sub>i  fin \<not>\<^sub>i  (init w))  " 
       by auto
 have  3: "\<turnstile>  empty  \<and>\<^sub>i  fin \<not>\<^sub>i ( init w) \<equiv>\<^sub>i \<not>\<^sub>i ( init w) \<and>\<^sub>i  empty " 
       using AndFinEqvChopAndEmpty EmptyChop prop03 by blast
 have  4: "\<turnstile> (init w \<and>\<^sub>i f); (init w \<and>\<^sub>i f)\<^sup>\<star> \<equiv>\<^sub>i init w \<and>\<^sub>i (f; (init w \<and>\<^sub>i f)\<^sup>\<star>) " 
       by (rule StateAndChop)
 have 41: "\<turnstile> ((init w \<and>\<^sub>i f); (init w \<and>\<^sub>i f)\<^sup>\<star>) \<and>\<^sub>i  fin \<not>\<^sub>i  (init w) \<equiv>\<^sub>i
             init w \<and>\<^sub>i (f; (init w \<and>\<^sub>i f)\<^sup>\<star>) \<and>\<^sub>i  fin \<not>\<^sub>i  (init w) " 
       using "4" by auto
 have 42: "\<turnstile> init w \<and>\<^sub>i (f; (init w \<and>\<^sub>i f)\<^sup>\<star>) \<and>\<^sub>i  fin \<not>\<^sub>i  (init w) \<equiv>\<^sub>i 
             init w \<and>\<^sub>i (f; (init w \<and>\<^sub>i f)\<^sup>\<star>) \<and>\<^sub>i  fin   (init \<not>\<^sub>i w) " 
       by  (simp)
 have  5: "\<turnstile> (f; ((init w \<and>\<^sub>i f)\<^sup>\<star>)) \<and>\<^sub>i  (fin  ( init \<not>\<^sub>i w))
             \<equiv>\<^sub>i (f; ((init w \<and>\<^sub>i f)\<^sup>\<star> \<and>\<^sub>i  (fin  ( init \<not>\<^sub>i w)))) " 
       by (rule ChopAndFin)
 have 51: "\<turnstile> (f; ((init w \<and>\<^sub>i f)\<^sup>\<star> \<and>\<^sub>i  (fin  ( init \<not>\<^sub>i w)))) \<equiv>\<^sub>i 
             (f; ((init w \<and>\<^sub>i f)\<^sup>\<star> \<and>\<^sub>i  (fin \<not>\<^sub>i ( init  w))))  " 
       by (simp)
 have 52: "\<turnstile> init w \<and>\<^sub>i (f; (init w \<and>\<^sub>i f)\<^sup>\<star>) \<and>\<^sub>i  fin \<not>\<^sub>i  (init w) \<equiv>\<^sub>i
           init w \<and>\<^sub>i (f; ((init w \<and>\<^sub>i f)\<^sup>\<star> \<and>\<^sub>i  fin \<not>\<^sub>i ( init  w))) " 
       using "42" "5" "51" by auto 
 have  6: "\<turnstile> f; ((init w \<and>\<^sub>i f)\<^sup>\<star> \<and>\<^sub>i  fin \<not>\<^sub>i ( init w)) \<equiv>\<^sub>i f;  while  (init w)  do  f " 
       by (simp add: while_d_def)
 have 61: "\<turnstile> init w \<and>\<^sub>i (f; ((init w \<and>\<^sub>i f)\<^sup>\<star> \<and>\<^sub>i  fin \<not>\<^sub>i ( init  w))) \<equiv>\<^sub>i
            init w \<and>\<^sub>i  (f;  while  (init w)  do  f) " using "6" 
       by auto
 have 62: "\<turnstile> ( empty \<and>\<^sub>i  fin \<not>\<^sub>i  (init w)) \<or>\<^sub>i ( ((init w \<and>\<^sub>i f); (init w \<and>\<^sub>i f)\<^sup>\<star>) \<and>\<^sub>i  fin \<not>\<^sub>i  (init w))
            \<equiv>\<^sub>i (\<not>\<^sub>i ( init w) \<and>\<^sub>i  empty ) \<or>\<^sub>i  (init w \<and>\<^sub>i (f;  while  (init w)  do  f)) " 
       using "21" "22" "3" "4" "52" "61" by auto
 have  7: "\<turnstile>  while  (init w)  do  f     
           \<equiv>\<^sub>i (\<not>\<^sub>i ( init w) \<and>\<^sub>i  empty ) \<or>\<^sub>i  (init w \<and>\<^sub>i (f;  while  (init w)  do  f)) " 
       using "1" "21" "22" "62" prop03 by blast
 have 71: "\<turnstile> if\<^sub>i  (init w)  then  (f; ( while  (init w)  do  f))  else   empty \<equiv>\<^sub>i
              (\<not>\<^sub>i ( init w) \<and>\<^sub>i  empty ) \<or>\<^sub>i  (init w \<and>\<^sub>i (f;  while  (init w)  do  f)) " 
       by auto 
 from 7 71 show ?thesis using itl_prop(30) prop03 by blast
qed
 
lemma WhileChopEqvIf:
 "\<turnstile>   ( while ( init w) do f); g \<equiv>\<^sub>i  if\<^sub>i(init w) then (f; ((while ( init w) do f); g))  else  g "
proof -
 have  1: "\<turnstile>  while  (init w)  do  f \<equiv>\<^sub>i  
              if\<^sub>i  (init w)  then  (f; ( while  (init w)  do  f))  else   empty " 
       by (rule WhileEqvIf)
 hence 2: "\<turnstile> ( while  (init w)  do  f); g \<equiv>\<^sub>i  
              if\<^sub>i  (init w)  then  ((f;  while  (init w)  do  f); g)  else   (empty ; g)" 
       by (rule IfChopEqvRule)
 have  3: "\<turnstile>  empty ; g \<equiv>\<^sub>i g " 
       by (rule EmptyChop)
 have  4: "\<turnstile> if\<^sub>i  (init w)  then  ((f;  while  (init w)  do  f); g)  else   (empty ; g) \<equiv>\<^sub>i
            if\<^sub>i  (init w)  then  ((f;  while  (init w)  do  f); g)  else   g " 
       using "3" by (simp add: ifthenelse_d_def)
 have  5: "\<turnstile> ((f;  while  (init w)  do  f); g) \<equiv>\<^sub>i (f;  (while  (init w)  do  f; g)) " 
       by (rule ChopAssocB)
 have  6: "\<turnstile> if\<^sub>i  (init w)  then  ((f;  while  (init w)  do  f); g)  else   g \<equiv>\<^sub>i
            if\<^sub>i  (init w)  then  (f;  ((while  (init w)  do  f); g))  else   g " 
       using "5" by (simp add: ifthenelse_d_def)
 from 1 2 4 6 show ?thesis using prop02 prop03 by blast
qed

lemma WhileChopEqvIfRule:
 assumes "\<turnstile>  f \<equiv>\<^sub>i ( while  (init w)  do  g); h "
 shows   "\<turnstile> f \<equiv>\<^sub>i  if\<^sub>i  (init w)  then  (g; f)  else  h "
proof -
 have 1: "\<turnstile> f \<equiv>\<^sub>i ( while  (init w)  do  g); h " 
      using assms by auto
 have 2: "\<turnstile> ( while  (init w)  do  g); h \<equiv>\<^sub>i  
            if\<^sub>i  (init w)  then  (g; (( while  (init w)  do  g); h))  else  h " 
      by (rule WhileChopEqvIf)
 have 3: "\<turnstile> (g; f) \<equiv>\<^sub>i (g; (( while  (init w)  do  g); h)) " 
      using "1" by (rule RightChopEqvChop)
 have 4: "\<turnstile> (g; (( while  (init w)  do  g); h)) \<equiv>\<^sub>i (g; f) " 
      using "3" by auto
 have 5: "\<turnstile> if\<^sub>i  (init w)  then  (g; (( while  (init w)  do  g); h))  else  h \<equiv>\<^sub>i
           if\<^sub>i  (init w)  then  (g; f)  else  h " 
      using "4" by (simp add: ifthenelse_d_def)
 from 1 2 5 show ?thesis using prop03 by blast
qed

lemma WhileImpFin:
 "\<turnstile>    while  (init w)  do  f \<supset>\<^sub>i  fin \<not>\<^sub>i ( init w) "
proof -
 have 1: "\<turnstile> (init w \<and>\<^sub>i f)\<^sup>\<star> \<and>\<^sub>i  fin \<not>\<^sub>i ( init w) \<supset>\<^sub>i  fin \<not>\<^sub>i ( init w)" by auto
 from 1 show ?thesis by (simp add: while_d_def)
qed

lemma WhileEqvEmptyOrChopWhile:
 "\<turnstile>  while (init w) do f \<equiv>\<^sub>i (\<not>\<^sub>i (init w) \<and>\<^sub>i empty) \<or>\<^sub>i (init w \<and>\<^sub>i (f \<and>\<^sub>i more );while (init w) do f) "
proof -
 have  1: "\<turnstile> (init w \<and>\<^sub>i f)\<^sup>\<star>  \<equiv>\<^sub>i  empty  \<or>\<^sub>i  ((init w \<and>\<^sub>i f)\<and>\<^sub>i  more ); (init w \<and>\<^sub>i f)\<^sup>\<star> " 
       by (rule ChopstarEqv)
 have  2: "\<turnstile> (init w \<and>\<^sub>i f) \<and>\<^sub>i  more  \<equiv>\<^sub>i init w \<and>\<^sub>i (f \<and>\<^sub>i  more ) " 
       by auto
 hence 3: "\<turnstile> ((init w \<and>\<^sub>i f)\<and>\<^sub>i  more ); (init w \<and>\<^sub>i f)\<^sup>\<star>  \<equiv>\<^sub>i (init w \<and>\<^sub>i f \<and>\<^sub>i  more ); (init w \<and>\<^sub>i f)\<^sup>\<star> " 
       by (rule LeftChopEqvChop)
 have  4: "\<turnstile> (init w \<and>\<^sub>i f)\<^sup>\<star>  \<equiv>\<^sub>i  empty  \<or>\<^sub>i  (init w \<and>\<^sub>i f \<and>\<^sub>i  more ); (init w \<and>\<^sub>i f)\<^sup>\<star> "  
       using "1" "3" by (meson EmptyImpCS itl_prop(31) prop30 prop19 prop02 prop14)
 have  5: "\<turnstile> (init w \<and>\<^sub>i f)\<^sup>\<star> \<and>\<^sub>i  fin \<not>\<^sub>i ( init w)  \<equiv>\<^sub>i  
             ( empty  \<and>\<^sub>i  fin  \<not>\<^sub>i (init w)) \<or>\<^sub>i  
             ((init w \<and>\<^sub>i f \<and>\<^sub>i  more ); (init w \<and>\<^sub>i f)\<^sup>\<star>\<and>\<^sub>i fin \<not>\<^sub>i ( init w)) " 
       using "1" prop23 using "4" by blast
 have  6: "\<turnstile>  empty  \<and>\<^sub>i  fin  \<not>\<^sub>i ( init w)\<equiv>\<^sub>i \<not>\<^sub>i ( init w) \<and>\<^sub>i  empty " 
       using AndFinEqvChopAndEmpty EmptyChop prop03 by blast
 have  7: "\<turnstile> (init w \<and>\<^sub>i f \<and>\<^sub>i  more ); (init w \<and>\<^sub>i f)\<^sup>\<star> \<equiv>\<^sub>i init w \<and>\<^sub>i (f \<and>\<^sub>i  more ); (init w \<and>\<^sub>i f)\<^sup>\<star> " 
       by (rule StateAndChop)
 have  8: "\<turnstile> ((f \<and>\<^sub>i  more ); (init w \<and>\<^sub>i f)\<^sup>\<star>) \<and>\<^sub>i  fin  ( init \<not>\<^sub>i w)  \<equiv>\<^sub>i  
            (f \<and>\<^sub>i  more ); ((init w \<and>\<^sub>i f)\<^sup>\<star> \<and>\<^sub>i  fin (  init \<not>\<^sub>i w))  " 
       by (rule ChopAndFin)
 have 81: "\<turnstile> fin (  init \<not>\<^sub>i w) \<equiv>\<^sub>i fin \<not>\<^sub>i ( init w) " 
       using FinEqvFin Initprop(2) itl_prop(30) by blast
 have 82: "\<turnstile> (f \<and>\<^sub>i  more ); (init w \<and>\<^sub>i f)\<^sup>\<star> \<and>\<^sub>i  fin \<not>\<^sub>i ( init w) \<equiv>\<^sub>i  
            (f \<and>\<^sub>i  more ); ((init w \<and>\<^sub>i f)\<^sup>\<star> \<and>\<^sub>i  fin \<not>\<^sub>i ( init w)) " 
       using "8" "81" by auto
 have  9: "\<turnstile> (init w \<and>\<^sub>i f)\<^sup>\<star> \<and>\<^sub>i  fin \<not>\<^sub>i ( init w)  \<equiv>\<^sub>i  
            (\<not>\<^sub>i ( init w) \<and>\<^sub>i  empty ) \<or>\<^sub>i  
                (init w \<and>\<^sub>i (f \<and>\<^sub>i  more ); ((init w \<and>\<^sub>i f)\<^sup>\<star> \<and>\<^sub>i  fin \<not>\<^sub>i ( init w))) " 
       using "5" "6" "7" "82" prop24 by blast
 from 9 show ?thesis by (metis while_d_def)
qed

lemma WhileIntro:
 assumes "\<turnstile>   \<not>\<^sub>i ( init w) \<and>\<^sub>i f \<supset>\<^sub>i  empty "
         "\<turnstile> init w \<and>\<^sub>i f \<supset>\<^sub>i (g \<and>\<^sub>i  more ); f"
 shows   "\<turnstile> f\<supset>\<^sub>i  while ( init w) do  g "
proof -
 have  1: "\<turnstile>   \<not>\<^sub>i ( init w) \<and>\<^sub>i f \<supset>\<^sub>i  empty " 
       using assms by blast
 have  2: "\<turnstile> init w \<and>\<^sub>i f \<supset>\<^sub>i (g \<and>\<^sub>i  more ); f " 
       using assms by blast
 have  3: "\<turnstile>  while ( init w)  do  g \<equiv>\<^sub>i  
           (\<not>\<^sub>i  (init w) \<and>\<^sub>i  empty ) \<or>\<^sub>i  (init w \<and>\<^sub>i (g \<and>\<^sub>i  more ); while  (init w)  do  g) " 
       by (rule WhileEqvEmptyOrChopWhile)
 hence 31: "\<turnstile> \<not>\<^sub>i ( while  (init w)  do  g) \<equiv>\<^sub>i 
           \<not>\<^sub>i( (\<not>\<^sub>i  (init w) \<and>\<^sub>i  empty ) \<or>\<^sub>i  (init w \<and>\<^sub>i (g \<and>\<^sub>i  more );  while  (init w)  do  g))" 
       using itl_prop(33) by blast
 hence 32: "\<turnstile> f \<and>\<^sub>i \<not>\<^sub>i ( while  (init w)  do  g) \<equiv>\<^sub>i 
              f \<and>\<^sub>i \<not>\<^sub>i( (\<not>\<^sub>i(init w) \<and>\<^sub>i empty) \<or>\<^sub>i  (init w \<and>\<^sub>i (g \<and>\<^sub>i  more );  while  (init w)  do  g)) "  
       using prop05 by blast
 have  33: "\<turnstile> f \<and>\<^sub>i \<not>\<^sub>i( (\<not>\<^sub>i(init w) \<and>\<^sub>i empty) \<or>\<^sub>i (init w \<and>\<^sub>i (g \<and>\<^sub>i more);  while  (init w)  do  g)) \<equiv>\<^sub>i
              f \<and>\<^sub>i \<not>\<^sub>i(\<not>\<^sub>i(init w) \<and>\<^sub>i  empty ) \<and>\<^sub>i \<not>\<^sub>i(init w \<and>\<^sub>i (g \<and>\<^sub>i  more );  while  (init w)  do  g) " 
       by auto
 have  34: "\<turnstile> f \<and>\<^sub>i \<not>\<^sub>i(\<not>\<^sub>i(init w) \<and>\<^sub>i empty) \<and>\<^sub>i \<not>\<^sub>i(init w \<and>\<^sub>i (g \<and>\<^sub>i  more );  while  (init w)  do  g) \<equiv>\<^sub>i
               f \<and>\<^sub>i  (  (init w) \<or>\<^sub>i more ) \<and>\<^sub>i (\<not>\<^sub>i(init w) \<or>\<^sub>i \<not>\<^sub>i((g \<and>\<^sub>i more); while (init w)  do  g)) "
       by auto 
 have  35: "\<turnstile> f \<and>\<^sub>i ((init w) \<or>\<^sub>i more) \<and>\<^sub>i (\<not>\<^sub>i(init w) \<or>\<^sub>i \<not>\<^sub>i((g \<and>\<^sub>i more);while (init w) do g)) \<equiv>\<^sub>i
              (f \<and>\<^sub>i (init w) \<and>\<^sub>i \<not>\<^sub>i ((g \<and>\<^sub>i  more );  while ( init w) do  g)) \<or>\<^sub>i
              (f \<and>\<^sub>i (init w) \<and>\<^sub>i \<not>\<^sub>i(init w)) \<or>\<^sub>i 
              (f \<and>\<^sub>i more \<and>\<^sub>i \<not>\<^sub>i ((g \<and>\<^sub>i  more );  while ( init w) do  g)) \<or>\<^sub>i
              (f \<and>\<^sub>i more \<and>\<^sub>i \<not>\<^sub>i(init w)) "  
       by auto
 have  36: "\<turnstile> \<not>\<^sub>i(f \<and>\<^sub>i (init w) \<and>\<^sub>i \<not>\<^sub>i(init w)) " 
       by auto
 have  37: "\<turnstile> \<not>\<^sub>i(f \<and>\<^sub>i more \<and>\<^sub>i \<not>\<^sub>i(init w))" 
       using "1" by auto
 have  38: "\<turnstile> (f \<and>\<^sub>i more \<and>\<^sub>i \<not>\<^sub>i ((g \<and>\<^sub>i  more );  while ( init w) do  g)) \<supset>\<^sub>i 
             ((g \<and>\<^sub>i  more ); f \<and>\<^sub>i \<not>\<^sub>i ((g \<and>\<^sub>i  more );  while ( init w) do  g)) " 
       using "1" "2" by auto
 have  39: "\<turnstile> (f \<and>\<^sub>i (init w) \<and>\<^sub>i \<not>\<^sub>i ((g \<and>\<^sub>i  more );  while ( init w) do  g)) \<supset>\<^sub>i 
             ((g \<and>\<^sub>i  more ); f  \<and>\<^sub>i \<not>\<^sub>i ((g \<and>\<^sub>i  more );  while ( init w) do  g))" 
       using "2" by auto
 have  40: "\<turnstile> ((f \<and>\<^sub>i (init w) \<and>\<^sub>i \<not>\<^sub>i ((g \<and>\<^sub>i  more );  while ( init w) do  g)) \<or>\<^sub>i
              (f \<and>\<^sub>i (init w) \<and>\<^sub>i \<not>\<^sub>i(init w)) \<or>\<^sub>i 
              (f \<and>\<^sub>i more \<and>\<^sub>i \<not>\<^sub>i ((g \<and>\<^sub>i  more );  while ( init w) do  g)) \<or>\<^sub>i
              (f \<and>\<^sub>i more \<and>\<^sub>i \<not>\<^sub>i(init w))) \<supset>\<^sub>i
              (g \<and>\<^sub>i  more ); f \<and>\<^sub>i \<not>\<^sub>i ((g \<and>\<^sub>i  more );  while ( init w) do  g) " 
        using "39" "38" "37" "38" by auto
 have   4: "\<turnstile> f \<and>\<^sub>i \<not>\<^sub>i ( while  (init w)  do  g) \<supset>\<^sub>i  
              (g \<and>\<^sub>i  more ); f \<and>\<^sub>i \<not>\<^sub>i ((g \<and>\<^sub>i  more );  while ( init w) do  g) " 
        using "32" "33" "34" "35" "40" by auto
 have   5: "\<turnstile> g \<and>\<^sub>i  more  \<supset>\<^sub>i  more " 
        by auto
 from 4 5 show ?thesis using ChopContra by blast
qed

lemma WhileElim:
 assumes "\<turnstile>  \<not>\<^sub>i ( init w) \<and>\<^sub>i  empty  \<supset>\<^sub>i g "
         "\<turnstile> init w \<and>\<^sub>i (f \<and>\<^sub>i  more ); g \<supset>\<^sub>i g"
 shows   "\<turnstile>  while  (init w) do  f \<supset>\<^sub>i g "
proof -
 have   1: "\<turnstile>  while ( init w)  do  f \<equiv>\<^sub>i  
             (\<not>\<^sub>i ( init w) \<and>\<^sub>i  empty ) \<or>\<^sub>i  (init w \<and>\<^sub>i (f \<and>\<^sub>i  more );  while ( init w)  do  f) " 
        by (rule WhileEqvEmptyOrChopWhile)
 hence 11: "\<turnstile> (while ( init w)  do  f) \<and>\<^sub>i \<not>\<^sub>i g \<equiv>\<^sub>i
             ((\<not>\<^sub>i(init w) \<and>\<^sub>i empty) \<or>\<^sub>i  (init w \<and>\<^sub>i (f \<and>\<^sub>i more);while (init w) do f)) \<and>\<^sub>i \<not>\<^sub>i g " 
        using prop06 by blast 
 have   2: "\<turnstile> \<not>\<^sub>i ( init w) \<and>\<^sub>i  empty  \<supset>\<^sub>i g " 
        using assms by blast
 hence 21: "\<turnstile> \<not>\<^sub>i g \<supset>\<^sub>i \<not>\<^sub>i(\<not>\<^sub>i ( init w) \<and>\<^sub>i  empty)" 
        by auto
 have  22: "\<turnstile>  ((\<not>\<^sub>i (init w) \<and>\<^sub>i empty) \<or>\<^sub>i (init w \<and>\<^sub>i (f \<and>\<^sub>i more);while (init w) do f)) \<and>\<^sub>i \<not>\<^sub>i  g \<supset>\<^sub>i
              (init w \<and>\<^sub>i (f \<and>\<^sub>i  more );  while ( init w)  do  f)  " 
        using "21" by auto
 have  23: "\<turnstile> (while ( init w)  do  f) \<and>\<^sub>i \<not>\<^sub>i g \<supset>\<^sub>i
              (init w \<and>\<^sub>i (f \<and>\<^sub>i  more );  while ( init w)  do  f) \<and>\<^sub>i \<not>\<^sub>i  g" 
        using "11" "21" by auto
 have   3: "\<turnstile> (init w) \<and>\<^sub>i ((f \<and>\<^sub>i  more ); g) \<supset>\<^sub>i g " 
        using assms by blast
 hence 31: "\<turnstile> \<not>\<^sub>i g \<supset>\<^sub>i \<not>\<^sub>i((init w) \<and>\<^sub>i ((f \<and>\<^sub>i  more ); g))" 
        using prop27 by blast 
 have  32: "\<turnstile> (init w \<and>\<^sub>i (f \<and>\<^sub>i  more );  while ( init w)  do  f) \<and>\<^sub>i \<not>\<^sub>i g \<supset>\<^sub>i 
             (((f \<and>\<^sub>i  more );  (while  (init w)  do  f)) \<and>\<^sub>i \<not>\<^sub>i ((f \<and>\<^sub>i  more ); g)) \<and>\<^sub>i \<not>\<^sub>ig" 
        using "31" by auto
 have   4: "\<turnstile>  (while  (init w)  do  f) \<and>\<^sub>i \<not>\<^sub>i  g \<supset>\<^sub>i  
              ((f \<and>\<^sub>i  more );  (while  (init w)  do  f)) \<and>\<^sub>i \<not>\<^sub>i ((f \<and>\<^sub>i  more ); g) " 
        using "23" "32" by auto 
 have   5: "\<turnstile> f \<and>\<^sub>i  more  \<supset>\<^sub>i  more " 
        by auto
 from 4 5 show ?thesis using ChopContra by blast
qed

lemma BaWhileImpWhile:
 "\<turnstile>    ba (f \<supset>\<^sub>i g) \<supset>\<^sub>i ( while  (init w)  do  f) \<supset>\<^sub>i ( while  (init w)  do  g)"
proof -
 have  1: "\<turnstile> (f \<supset>\<^sub>i g) \<supset>\<^sub>i ((init w \<and>\<^sub>i f)  \<supset>\<^sub>i (init w \<and>\<^sub>i g)) " 
       by auto
 hence 2: "\<turnstile>  ba (f \<supset>\<^sub>i g) \<supset>\<^sub>i  ba ((init w \<and>\<^sub>i f)  \<supset>\<^sub>i (init w\<and>\<^sub>i g)) " 
       by (rule BaImpBa)
 have  3: " \<turnstile>  ba ((init w \<and>\<^sub>i f)  \<supset>\<^sub>i (init w\<and>\<^sub>i g)) \<supset>\<^sub>i ((init w\<and>\<^sub>i f)\<^sup>\<star> \<supset>\<^sub>i (init w\<and>\<^sub>i g)\<^sup>\<star>) " 
       by (rule BaCSImpCS)
 have  4: "\<turnstile>  ba (f \<supset>\<^sub>i g) \<supset>\<^sub>i ((init w\<and>\<^sub>i f)\<^sup>\<star>\<and>\<^sub>i fin \<not>\<^sub>i ( init w) \<supset>\<^sub>i (init w\<and>\<^sub>i g)\<^sup>\<star>\<and>\<^sub>i fin \<not>\<^sub>i (init w)) " 
       using "2" "3" by auto
 from 4 show ?thesis by (simp add: while_d_def)
qed

lemma WhileImpWhile:
 assumes "\<turnstile>   f\<supset>\<^sub>i g "
 shows "\<turnstile> ( while  (init w)  do  f) \<supset>\<^sub>i ( while  (init w)  do  g) "
proof -
 have  1: "\<turnstile> f \<supset>\<^sub>i g " 
       using assms by auto
 hence 2: "\<turnstile>  ba (f \<supset>\<^sub>i g) " 
       by (rule BaGen)
 have  3: "\<turnstile>  ba (f \<supset>\<^sub>i g) \<supset>\<^sub>i ( while  (init w)  do  f) \<supset>\<^sub>i ( while  (init w)  do  g) " 
       by (rule BaWhileImpWhile)
 from 2 3 show ?thesis using MP by blast
qed

subsection \<open>Properties of Halt\<close>

lemma WnextAndMoreEqvNext:
 "\<turnstile> wnext f \<and>\<^sub>i more \<equiv>\<^sub>i \<circle> f"
by auto

lemma BoxStateAndEmptyEqvStateAndEmpty:
 "\<turnstile> \<box>(empty \<equiv>\<^sub>i (init w)) \<and>\<^sub>i empty \<equiv>\<^sub>i (init w) \<and>\<^sub>i empty "
apply simp_all
by auto

lemma BoxEmptyEqvIStateqvEmptyAndStateOrNotStateNext:
 "\<turnstile> \<box>(empty \<equiv>\<^sub>i (init w)) \<equiv>\<^sub>i (empty \<and>\<^sub>i init w) \<or>\<^sub>i (\<not>\<^sub>i(init w) \<and>\<^sub>i \<circle>(\<box>(empty \<equiv>\<^sub>i (init w)))) "
proof -
 have  1: "\<turnstile> \<box>(empty \<equiv>\<^sub>i (init w)) \<equiv>\<^sub>i 
            (\<box>(empty \<equiv>\<^sub>i (init w)) \<and>\<^sub>i empty) \<or>\<^sub>i (\<box>(empty \<equiv>\<^sub>i (init w)) \<and>\<^sub>i more)" 
       by auto
 have  2: "\<turnstile> \<box>(empty \<equiv>\<^sub>i (init w)) \<and>\<^sub>i empty \<equiv>\<^sub>i (init w) \<and>\<^sub>i empty " 
       using BoxStateAndEmptyEqvStateAndEmpty by blast
 have  3: "\<turnstile> \<box>(empty \<equiv>\<^sub>i (init w))  \<equiv>\<^sub>i (empty \<equiv>\<^sub>i (init w)) \<and>\<^sub>i wnext(\<box>(empty \<equiv>\<^sub>i (init w))) " 
       using BoxEqvAndWnextBox by blast
 hence 4: "\<turnstile> \<box>(empty \<equiv>\<^sub>i (init w)) \<and>\<^sub>i more  \<equiv>\<^sub>i 
               (empty \<equiv>\<^sub>i (init w)) \<and>\<^sub>i wnext(\<box>(empty \<equiv>\<^sub>i (init w))) \<and>\<^sub>i more " 
       by auto
 have  5: "\<turnstile> (empty \<equiv>\<^sub>i (init w)) \<and>\<^sub>i more \<equiv>\<^sub>i \<not>\<^sub>i(init w) \<and>\<^sub>i more " 
       by auto
 have  6: "\<turnstile> wnext(\<box>(empty \<equiv>\<^sub>i (init w))) \<and>\<^sub>i more \<equiv>\<^sub>i \<circle>(\<box>(empty \<equiv>\<^sub>i (init w)))  " 
       using WnextAndMoreEqvNext by auto 
 from 1 2 4 5 6 show ?thesis by auto
qed

lemma HaltStateEqvIfStateThenEmptyElseNext:
 "\<turnstile>  halt(  init w) \<equiv>\<^sub>i  if\<^sub>i  (init w)  then   empty   else ( \<circle>( halt ( init w))) "
proof -
 have 1: "\<turnstile> halt(  init w) \<equiv>\<^sub>i \<box>(empty \<equiv>\<^sub>i (init w))" 
      by (simp add: halt_d_def)
 have 2: "\<turnstile> \<box>(empty \<equiv>\<^sub>i (init w)) \<equiv>\<^sub>i 
             (empty \<and>\<^sub>i init w) \<or>\<^sub>i (\<not>\<^sub>i(init w) \<and>\<^sub>i \<circle>(\<box>(empty \<equiv>\<^sub>i (init w))))  " 
      by (rule BoxEmptyEqvIStateqvEmptyAndStateOrNotStateNext)
 have 21: "\<turnstile> (empty \<and>\<^sub>i init w) \<or>\<^sub>i (\<not>\<^sub>i(init w) \<and>\<^sub>i \<circle>(\<box>(empty \<equiv>\<^sub>i (init w)))) \<equiv>\<^sub>i
             (init w \<and>\<^sub>i empty) \<or>\<^sub>i (\<not>\<^sub>i(init w) \<and>\<^sub>i \<circle>(\<box>(empty \<equiv>\<^sub>i (init w))))" 
      by auto
 have 22: "\<turnstile> \<circle>(halt ( init w)) \<equiv>\<^sub>i \<circle>(\<box>(empty \<equiv>\<^sub>i (init w)))" 
      using NextEqvNext using "1" by blast
 have 3: "\<turnstile> if\<^sub>i  (init w)  then   empty   else ( \<circle>( halt ( init w))) \<equiv>\<^sub>i
            (init w \<and>\<^sub>i empty ) \<or>\<^sub>i (\<not>\<^sub>i(init w) \<and>\<^sub>i \<circle>( halt ( init w))) " 
      by (simp add: ifthenelse_d_def)
 from 1 2 21 22 3 show ?thesis by (simp add: halt_d_def)
qed

lemma HaltChopEqv:
 "\<turnstile>  ((halt ( init w)) ; f) \<equiv>\<^sub>i  (if\<^sub>i  (init w)  then ( f ) else (\<circle>( (halt ( init w)); f))) "
proof -
 have  1: "\<turnstile>  halt(init w) \<equiv>\<^sub>i  
             (if\<^sub>i  (init w)  then   empty   else ( \<circle>( halt ( init w)))) " 
       by (rule HaltStateEqvIfStateThenEmptyElseNext)
 hence 2: "\<turnstile>  ((halt(init w));f) \<equiv>\<^sub>i  
             (if\<^sub>i  (init w)  then   (empty;f)   else ( \<circle>( halt ( init w));f)) " 
       by (rule IfChopEqvRule)
 have  3: "\<turnstile>  empty ; f \<equiv>\<^sub>i f " 
       by (rule EmptyChop)
 have  4: "\<turnstile> (\<circle> (halt ( init w))); f \<equiv>\<^sub>i \<circle>( halt ( init w); f)" 
       by (rule NextChop)
 from 2 3 4 show ?thesis using prop07 prop03 by blast
qed

lemma AndHaltChopImp:
 "\<turnstile>   init w \<and>\<^sub>i ( halt ( init w); f) \<supset>\<^sub>i f "
proof -
 have 1: "\<turnstile> halt ( init w); f \<equiv>\<^sub>i  if\<^sub>i  (init w)  then  f  else ( \<circle>( halt ( init w); f)) " 
      by (rule HaltChopEqv)
 have 2: "\<turnstile> init w \<and>\<^sub>i if\<^sub>i  (init w)  then  f  else ( \<circle>( halt ( init w); f)) \<supset>\<^sub>i f" 
      by (simp add: ifthenelse_d_def and_d_def) 
 from 1 2 show ?thesis by auto
qed
 
lemma NotAndHaltChopImpNext:
 "\<turnstile>  \<not>\<^sub>i ( init w) \<and>\<^sub>i ( halt  (init w); f) \<supset>\<^sub>i \<circle>( halt ( init w); f) "
proof -
 have 1: "\<turnstile>  halt ( init w); f \<equiv>\<^sub>i  if\<^sub>i  (init w)  then  f  else ( \<circle>( halt ( init w); f)) " 
      by (rule HaltChopEqv)
 have 2: "\<turnstile> \<not>\<^sub>i ( init w) \<and>\<^sub>i if\<^sub>i  (init w)  then  f  else ( \<circle>( halt ( init w); f)) \<supset>\<^sub>i 
            \<circle>( halt ( init w); f)"  
      by auto 
from 1 2 show ?thesis by auto
qed

lemma NotAndHaltChopImpSkipYields:
 "\<turnstile>  \<not>\<^sub>i ( init w) \<and>\<^sub>i ( halt ( init w); f) \<supset>\<^sub>i  skip  yields ( halt  (init w); f) "
proof -
 have 1: "\<turnstile> \<not>\<^sub>i ( init w) \<and>\<^sub>i ( halt ( init w); f) \<supset>\<^sub>i \<circle>( halt ( init w); f) " 
      by (rule NotAndHaltChopImpNext)
 have 2: "\<turnstile> \<circle>( halt ( init w); f) \<supset>\<^sub>i  skip  yields ( halt ( init w); f) " 
      by (rule NextImpSkipYields)
 from 1 2 show ?thesis by auto
qed

lemma TrueChopAndEmptyEqvChopAndEmpty:
 " \<turnstile> (true\<^sub>i;(f \<and>\<^sub>i empty)) \<and>\<^sub>i g \<equiv>\<^sub>i (g;(f\<and>\<^sub>i empty))  "
by force

lemma WprevEqvEmptyOrPrev:
 "\<turnstile> wprev f \<equiv>\<^sub>i empty \<or>\<^sub>i prev f"
by  auto


lemma NotChopSkipEqvMoreAndNotChopSkip:
 "\<turnstile> (\<not>\<^sub>i f);skip \<equiv>\<^sub>i more \<and>\<^sub>i \<not>\<^sub>i(f;skip)"
using WprevEqvEmptyOrPrev 
by (metis (full_types) and_d_def empty_d_def 
          itl_prop(30) itl_prop(33) itl_prop(4) prev_d_def prop03 prop28 wprev_d_def)

(* changed proof *)
lemma HaltChopImpNotHaltChopNot:
 "\<turnstile>    halt ( init w); f \<supset>\<^sub>i \<not>\<^sub>i ( halt ( init w); \<not>\<^sub>i  f) "
proof -
 have   1: "\<turnstile>  halt  (init w); f \<equiv>\<^sub>i  if\<^sub>i  (init w)  then  f  else ( \<circle>( halt ( init w); f)) " 
        by (rule HaltChopEqv)
 have   2: "\<turnstile> if\<^sub>i  (init w)  then  f  else ( \<circle>( halt ( init w); f)) \<supset>\<^sub>i 
              ( ((init w) \<supset>\<^sub>i f) \<and>\<^sub>i ( \<not>\<^sub>i(init w) \<supset>\<^sub>i ( \<circle>( halt ( init w); f))))"  
        by (rule prop11) 
 have   3: "\<turnstile>  halt  (init w); \<not>\<^sub>if \<equiv>\<^sub>i  
               if\<^sub>i  (init w)  then  \<not>\<^sub>if  else ( \<circle>( halt ( init w); \<not>\<^sub>if)) " 
        by (rule HaltChopEqv)
 have   4: "\<turnstile> if\<^sub>i  (init w)  then  \<not>\<^sub>if  else ( \<circle>( halt ( init w); \<not>\<^sub>if)) \<supset>\<^sub>i 
              ( ((init w) \<supset>\<^sub>i \<not>\<^sub>if) \<and>\<^sub>i ( \<not>\<^sub>i(init w) \<supset>\<^sub>i ( \<circle>( halt ( init w); \<not>\<^sub>if))))"  
        by (rule prop11)
 have   5: "\<turnstile> halt  (init w); f \<and>\<^sub>i halt  (init w); \<not>\<^sub>if \<supset>\<^sub>i
            ( ((init w) \<supset>\<^sub>i f) \<and>\<^sub>i ( \<not>\<^sub>i(init w) \<supset>\<^sub>i ( \<circle>( halt ( init w); f)))) \<and>\<^sub>i
            ( ((init w) \<supset>\<^sub>i \<not>\<^sub>if) \<and>\<^sub>i ( \<not>\<^sub>i(init w) \<supset>\<^sub>i ( \<circle>( halt ( init w); \<not>\<^sub>if)))) " 
        using "1" "2" "3" "4" by auto
 have   6: "\<turnstile> ( ((init w) \<supset>\<^sub>i f) \<and>\<^sub>i ( \<not>\<^sub>i(init w) \<supset>\<^sub>i ( \<circle>( halt ( init w); f)))) \<and>\<^sub>i
            ( ((init w) \<supset>\<^sub>i \<not>\<^sub>if) \<and>\<^sub>i ( \<not>\<^sub>i(init w) \<supset>\<^sub>i ( \<circle>( halt ( init w); \<not>\<^sub>if)))) \<supset>\<^sub>i
            ( \<circle>( halt ( init w); f)) \<and>\<^sub>i ( \<circle>( halt ( init w); \<not>\<^sub>if)) " 
        by auto
 have   7: "\<turnstile> halt  (init w); f \<and>\<^sub>i halt  (init w); \<not>\<^sub>if \<supset>\<^sub>i
            ( \<circle>( halt ( init w); f)) \<and>\<^sub>i ( \<circle>( halt ( init w); \<not>\<^sub>if)) " 
        using "5" "6" prop02 by blast
 have   8: "\<turnstile> ( \<circle>( halt ( init w); f)) \<and>\<^sub>i ( \<circle>( halt ( init w); \<not>\<^sub>if)) \<equiv>\<^sub>i
             \<circle> (halt ( init w); f \<and>\<^sub>i halt ( init w); \<not>\<^sub>if) " 
        by auto
 have   9: "\<turnstile> halt  (init w); f \<and>\<^sub>i halt  (init w); \<not>\<^sub>if \<supset>\<^sub>i 
              \<circle> (halt ( init w); f \<and>\<^sub>i halt ( init w); \<not>\<^sub>if) " 
        using "7" "8" itl_prop(31) prop02 by blast
 hence 10: "\<turnstile> \<not>\<^sub>i(halt  (init w); f \<and>\<^sub>i halt  (init w); \<not>\<^sub>if) " 
        using NextLoop by blast
 from 10 show ?thesis by auto
qed

lemma HaltChopImpHaltYields:
 "\<turnstile>    halt ( init w); f \<supset>\<^sub>i ( halt ( init w)) yields  f "
proof -
 have 1: "\<turnstile>  halt ( init w); f \<supset>\<^sub>i \<not>\<^sub>i ( halt ( init w); \<not>\<^sub>i  f) " by (rule HaltChopImpNotHaltChopNot)
 from 1 show ?thesis by (simp add: yields_d_def)
qed

lemma HaltChopAnd:
 "\<turnstile> ( halt (init w)); f \<and>\<^sub>i ( halt  (init w)); g \<supset>\<^sub>i ( halt ( init w)); (f \<and>\<^sub>i g) "
proof -
 have  1: "\<turnstile> ( halt (init w)); g \<supset>\<^sub>i ( halt (init w))  yields  g " by (rule HaltChopImpHaltYields)
 hence 2: "\<turnstile> ( halt  (init w)); f \<and>\<^sub>i ( halt  (init w)); g \<supset>\<^sub>i
             ( halt  (init w)); f \<and>\<^sub>i ( halt  (init w)) yields  g " by auto
 have  3: "\<turnstile> ( halt  (init w)); f \<and>\<^sub>i ( halt  (init w)) yields  g\<supset>\<^sub>i
             ( halt  (init w)); (f \<and>\<^sub>i g) " by (rule ChopAndYieldsImp)
 from 2 3 show ?thesis by auto
qed

lemma HaltAndChopAndHaltChopImpHaltAndChopAnd:
 "\<turnstile>   ( halt  (init w) \<and>\<^sub>i f); f1 \<and>\<^sub>i ( halt  (init w); g) \<supset>\<^sub>i ( halt ( init w) \<and>\<^sub>i f); (f1 \<and>\<^sub>i g) "
proof -
 have   1: "\<turnstile> f1 \<supset>\<^sub>i \<not>\<^sub>i  g \<or>\<^sub>i  (f1 \<and>\<^sub>i g) " 
        by auto
 hence  2: "\<turnstile> ( halt  (init w) \<and>\<^sub>i f); f1 \<supset>\<^sub>i  
             ( halt  (init w) \<and>\<^sub>i f); \<not>\<^sub>i  g \<or>\<^sub>i  (( halt  (init w) \<and>\<^sub>i f); (f1 \<and>\<^sub>i g)) " 
        by (rule ChopOrImpRule)
 have   3: "\<turnstile> ( halt  (init w) \<and>\<^sub>i f); \<not>\<^sub>i  g \<supset>\<^sub>i  halt  (init w); \<not>\<^sub>i  g " 
        by (rule AndChopA)
 have  31: "\<turnstile> ( halt  (init w) \<and>\<^sub>i f); f1 \<supset>\<^sub>i 
             halt  (init w); \<not>\<^sub>i  g \<or>\<^sub>i  (( halt  (init w) \<and>\<^sub>i f); (f1 \<and>\<^sub>i g)) " 
        using "2""3" by auto
 have   4: "\<turnstile>  halt  (init w); g \<supset>\<^sub>i \<not>\<^sub>i ( halt  (init w); \<not>\<^sub>i  g) " 
        by (rule HaltChopImpNotHaltChopNot)
 hence 41: "\<turnstile> ( halt  (init w); \<not>\<^sub>i  g) \<supset>\<^sub>i \<not>\<^sub>i(halt  (init w); g)" 
        by auto
 have  42: "\<turnstile> ( halt  (init w) \<and>\<^sub>i f); f1 \<supset>\<^sub>i 
             \<not>\<^sub>i( halt  (init w); g) \<or>\<^sub>i  (( halt  (init w) \<and>\<^sub>i f); (f1 \<and>\<^sub>i g)) " 
        using "31" "41" by auto
 from 42 show ?thesis by auto
qed

lemma HaltImpBoxYields:
 "\<turnstile>   ( halt  (init w)); f \<supset>\<^sub>i (\<box>\<not>\<^sub>i ( init w))  yields  (( halt  (init w)); f)"
proof -
 have   1: "\<turnstile> (\<box> \<not>\<^sub>i  (init w)); \<not>\<^sub>i ( halt  (init w); f) \<supset>\<^sub>i  di  (\<box> \<not>\<^sub>i  (init w)) " 
        by (rule ChopImpDi)
 have   2: "\<turnstile> \<box> \<not>\<^sub>i  (init w) \<supset>\<^sub>i \<not>\<^sub>i  (init w) " 
        by (rule BoxElim)
 hence  3: "\<turnstile>  di (\<box> \<not>\<^sub>i  (init w)) \<supset>\<^sub>i  di \<not>\<^sub>i  (init w) " 
        by (rule DiImpDi)
 have   4: "\<turnstile>  di   (init \<not>\<^sub>i w) \<equiv>\<^sub>i   (init \<not>\<^sub>iw) " 
        by (rule DiState)
 have  41: "\<turnstile>  (init \<not>\<^sub>i w) \<equiv>\<^sub>i  \<not>\<^sub>i (init w)" 
        by auto
 have  42: "\<turnstile>  di  \<not>\<^sub>i (init  w) \<equiv>\<^sub>i   \<not>\<^sub>i(init w) " 
        using "4" "41" by auto
 have   5: "\<turnstile> ((\<box>\<not>\<^sub>i  (init w)); \<not>\<^sub>i ( halt  (init w); f)) \<supset>\<^sub>i \<not>\<^sub>i ( init w) " 
        using "1" "2" "42" using "3" itl_prop(31) prop02 by blast
 hence 51: "\<turnstile> ( halt  (init w); f) \<and>\<^sub>i ((\<box>\<not>\<^sub>i  (init w)); \<not>\<^sub>i ( halt  (init w); f)) \<supset>\<^sub>i
            ( halt  (init w); f) \<and>\<^sub>i \<not>\<^sub>i ( init w) " 
        using prop12 by blast
 have   6: "\<turnstile>  halt  (init w); f \<equiv>\<^sub>i if\<^sub>i (init w)  then  f  else  (\<circle>( halt  (init w); f)) " 
        by (rule HaltChopEqv)
 hence 61: "\<turnstile> halt  (init w); f \<and>\<^sub>i \<not>\<^sub>i ( init w) \<equiv>\<^sub>i 
              (if\<^sub>i (init w)  then  f  else  (\<circle>( halt  (init w); f))) \<and>\<^sub>i \<not>\<^sub>i ( init w) " 
        using "6" by auto
 have  62: "\<turnstile> (if\<^sub>i (init w)  then  f  else  (\<circle>( halt  (init w); f))) \<and>\<^sub>i 
              \<not>\<^sub>i ( init w) \<supset>\<^sub>i (\<circle>( halt  (init w); f)) "  
        by auto 
 have  63: "\<turnstile> halt  (init w); f \<and>\<^sub>i \<not>\<^sub>i ( init w) \<supset>\<^sub>i (\<circle>( halt  (init w); f))  " 
        using "61" "62" by auto
 have   7: "\<turnstile> ( halt  (init w); f) \<and>\<^sub>i (\<box>\<not>\<^sub>i  (init w)); \<not>\<^sub>i ( halt  (init w); f) \<supset>\<^sub>i
             \<circle>(( halt  (init w)); f) " 
        using "51" "63" using prop02 by blast
 have   8: "\<turnstile> \<box> \<not>\<^sub>i (init w) \<supset>\<^sub>i  empty  \<or>\<^sub>i  \<circle>(\<box>\<not>\<^sub>i(  init w)) " 
        by auto
 hence  9: "\<turnstile> ((\<box> \<not>\<^sub>i ( init w)); \<not>\<^sub>i ( halt  (init w); f)) \<supset>\<^sub>i  
            \<not>\<^sub>i ( halt  (init w); f) \<or>\<^sub>i  \<circle>((\<box>\<not>\<^sub>i  (init w)); \<not>\<^sub>i ( halt  (init w); f)) " 
        by (rule EmptyOrNextChopImpRule)
 hence 10: "\<turnstile> (( halt  (init w)); f) \<and>\<^sub>i (\<box> \<not>\<^sub>i  (init w)); \<not>\<^sub>i ( halt  (init w); f) \<supset>\<^sub>i  
            \<circle>((\<box>\<not>\<^sub>i  (init w)); \<not>\<^sub>i ( halt  (init w); f)) " 
        using prop13 by blast
 have  11: "\<turnstile> ( halt  (init w)); f \<and>\<^sub>i (\<box> \<not>\<^sub>i  (init w)); \<not>\<^sub>i ( halt  (init w); f) \<supset>\<^sub>i 
           \<circle>(( halt  (init w)); f) \<and>\<^sub>i \<circle>((\<box>\<not>\<^sub>i  (init w)); \<not>\<^sub>i ( halt  (init w); f)) " 
        using "7" "10" by auto
 have  12: "\<turnstile> \<circle>(( halt  (init w)); f) \<and>\<^sub>i \<circle>((\<box>\<not>\<^sub>i  (init w)); \<not>\<^sub>i ( halt  (init w); f))
           \<supset>\<^sub>i  \<circle>((( halt (init w)); f) \<and>\<^sub>i ((\<box>\<not>\<^sub>i  (init w)); \<not>\<^sub>i ( halt  (init w); f))) " 
        by auto
 have  13: "\<turnstile> ( halt  (init w)); f \<and>\<^sub>i (\<box> \<not>\<^sub>i  (init w)); \<not>\<^sub>i ( halt  (init w); f) \<supset>\<^sub>i  
             \<circle>((( halt  (init w)); f) \<and>\<^sub>i ((\<box>\<not>\<^sub>i  (init w)); \<not>\<^sub>i ( halt  (init w); f))) " 
        using "11" "12" by auto
 hence 14: "\<turnstile> \<not>\<^sub>i (( halt  (init w)); f \<and>\<^sub>i (\<box> \<not>\<^sub>i  (init w)); \<not>\<^sub>i ( halt  (init w); f)) " 
        using NextLoop by blast
 hence 15: "\<turnstile> ( halt  (init w)); f \<supset>\<^sub>i \<not>\<^sub>i ((\<box> \<not>\<^sub>i  (init w)); \<not>\<^sub>i ( halt  (init w); f)) " 
        by auto
 from 15 show ?thesis by (simp add: yields_d_def)
qed

subsection \<open>Properties of Groups of chops\<close>

lemma NestedChopImpChop:
 assumes "\<turnstile>   init w \<and>\<^sub>i f \<supset>\<^sub>i g; (init w1 \<and>\<^sub>i f1) "
         "\<turnstile> init w1 \<and>\<^sub>i f1 \<supset>\<^sub>i g1; (init w2 \<and>\<^sub>i f2) "
 shows   "\<turnstile> init w \<and>\<^sub>i f \<supset>\<^sub>i g; (g1; (init w2 \<and>\<^sub>i f2)) "
proof -
 have  1: "\<turnstile> init w \<and>\<^sub>i f \<supset>\<^sub>i g; (init w1 \<and>\<^sub>i f1) " using assms(1) by auto
 have  2: "\<turnstile> init w1 \<and>\<^sub>i f1 \<supset>\<^sub>i g1; (init w2 \<and>\<^sub>i f2) " using assms(2) by auto
 hence 3: "\<turnstile> g; (init w1 \<and>\<^sub>i f1) \<supset>\<^sub>i g; (g1; (init w2 \<and>\<^sub>i f2)) " by (rule RightChopImpChop)
 from 1 3 show ?thesis by auto
qed
 

subsection \<open>Properties of Time Reversal\<close>

lemma RNot:
 "\<turnstile> (\<not>\<^sub>if)\<^sup>r \<equiv>\<^sub>i \<not>\<^sub>i f\<^sup>r"
by simp

lemma RRNot:
 "\<turnstile> (\<not>\<^sub>i(f\<^sup>r))\<^sup>r \<equiv>\<^sub>i \<not>\<^sub>if"
by (metis EqvReverseReverse not_d_def rev_d.simps(1) rev_d.simps(3))

lemma RTrue:
 "\<turnstile> (true\<^sub>i)\<^sup>r \<equiv>\<^sub>i true\<^sub>i"
by (simp add: not_d_def true_d_def) 

lemma ROr:
 "\<turnstile> (f \<or>\<^sub>i g)\<^sup>r \<equiv>\<^sub>i f\<^sup>r \<or>\<^sub>i g\<^sup>r"
by (simp add: not_d_def or_d_def)

lemma RROr:
 "\<turnstile> (f\<^sup>r \<or>\<^sub>i g\<^sup>r)\<^sup>r \<equiv>\<^sub>i f \<or>\<^sub>i g"
proof -
 have 1: "\<turnstile> (f\<^sup>r \<or>\<^sub>i g\<^sup>r)\<^sup>r \<equiv>\<^sub>i (f\<^sup>r)\<^sup>r \<or>\<^sub>i (g\<^sup>r)\<^sup>r" using ROr by blast
 have 2: "\<turnstile> (f\<^sup>r)\<^sup>r \<or>\<^sub>i (g\<^sup>r)\<^sup>r \<equiv>\<^sub>i f \<or>\<^sub>i g" using EqvReverseReverse by auto
 from 1 2 show ?thesis by auto
qed

lemma RAnd:
 " \<turnstile> (f \<and>\<^sub>i g)\<^sup>r \<equiv>\<^sub>i f\<^sup>r \<and>\<^sub>i g\<^sup>r"
by (simp add: and_d_def not_d_def or_d_def)

lemma RRAnd:
 " \<turnstile> (f\<^sup>r \<and>\<^sub>i g\<^sup>r)\<^sup>r \<equiv>\<^sub>i f \<and>\<^sub>i g"
proof -
 have 1: "\<turnstile> (f\<^sup>r \<and>\<^sub>i g\<^sup>r)\<^sup>r \<equiv>\<^sub>i (f\<^sup>r)\<^sup>r \<and>\<^sub>i (g\<^sup>r)\<^sup>r" using RAnd by blast
 have 2: "\<turnstile> (f\<^sup>r)\<^sup>r \<and>\<^sub>i (g\<^sup>r)\<^sup>r \<equiv>\<^sub>i f \<and>\<^sub>i g" using EqvReverseReverse by auto
 from 1 2 show ?thesis by auto
qed

lemma REqvRule:
 assumes "\<turnstile> f \<equiv>\<^sub>i g"
 shows   "\<turnstile> f\<^sup>r \<equiv>\<^sub>i g\<^sup>r"
using assms 
by (metis ReverseEqv itl_prop(31) rev_d.simps(3))

lemma RImpRule:
 assumes "\<turnstile> f \<supset>\<^sub>i g"
 shows   "\<turnstile> f\<^sup>r \<supset>\<^sub>i g\<^sup>r"
using assms 
by (metis ReverseEqv rev_d.simps(3))

lemma RNextEqvPrev:
 "\<turnstile> (\<circle> f)\<^sup>r \<equiv>\<^sub>i prev (f\<^sup>r)"
by (simp add: next_d_def prev_d_def)

lemma RRNextEqvPrev:
 "\<turnstile> (\<circle> (f\<^sup>r))\<^sup>r \<equiv>\<^sub>i prev (f)"
proof -
 have 1: "\<turnstile> (\<circle> (f\<^sup>r))\<^sup>r \<equiv>\<^sub>i prev ((f\<^sup>r)\<^sup>r)" using RNextEqvPrev by blast
 have 2: "\<turnstile> prev ((f\<^sup>r)\<^sup>r) \<equiv>\<^sub>i prev f" using EqvReverseReverse by auto
 from 1 2 show ?thesis by auto
qed

lemma RWNextEqvWPrev:
 "\<turnstile> (wnext f)\<^sup>r \<equiv>\<^sub>i wprev(f\<^sup>r)"
by (simp add: next_d_def not_d_def prev_d_def wnext_d_def wprev_d_def)

lemma RRWNextEqvWPrev:
 "\<turnstile> (wnext (f\<^sup>r))\<^sup>r \<equiv>\<^sub>i wprev(f)"
proof -
 have 1: "\<turnstile> (wnext (f\<^sup>r))\<^sup>r \<equiv>\<^sub>i wprev ((f\<^sup>r)\<^sup>r)" using RWNextEqvWPrev by blast
 have 2: "\<turnstile> wprev ((f\<^sup>r)\<^sup>r) \<equiv>\<^sub>i wprev f" using EqvReverseReverse by auto
 from 1 2 show ?thesis by auto
qed

lemma RPrevEqvNext:
"\<turnstile> (prev f)\<^sup>r \<equiv>\<^sub>i \<circle> (f\<^sup>r)"
by (simp add: next_d_def prev_d_def)

lemma RRPrevEqvNext:
"\<turnstile> (prev (f\<^sup>r))\<^sup>r \<equiv>\<^sub>i \<circle> (f)"
proof -
 have 1: "\<turnstile> (prev (f\<^sup>r))\<^sup>r \<equiv>\<^sub>i \<circle> ((f\<^sup>r)\<^sup>r)" using RPrevEqvNext by blast
 have 2: "\<turnstile> \<circle> ((f\<^sup>r)\<^sup>r) \<equiv>\<^sub>i \<circle> f" using EqvReverseReverse by auto
 from 1 2 show ?thesis by auto
qed

lemma RWPrevEqvWNext:
 "\<turnstile> (wprev f)\<^sup>r \<equiv>\<^sub>i wnext(f\<^sup>r)"
by (simp add: next_d_def not_d_def prev_d_def wnext_d_def wprev_d_def)

lemma RRWPrevEqvWNext:
 "\<turnstile> (wprev (f\<^sup>r))\<^sup>r \<equiv>\<^sub>i wnext(f)"
proof -
 have 1: "\<turnstile> (wprev (f\<^sup>r))\<^sup>r \<equiv>\<^sub>i wnext ((f\<^sup>r)\<^sup>r)" using RWPrevEqvWNext by blast
 have 2: "\<turnstile> wnext ((f\<^sup>r)\<^sup>r) \<equiv>\<^sub>i wnext f" using EqvReverseReverse by auto
 from 1 2 show ?thesis by auto
qed

lemma RDiamondEqvDi:
 "\<turnstile> (\<diamond>f)\<^sup>r \<equiv>\<^sub>i di (f\<^sup>r)"
by (metis RTrue RightChopEqvChop di_d_def rev_d.simps(5) sometimes_d_def) 

lemma RRDiamondEqvDi:
 "\<turnstile> (\<diamond>(f\<^sup>r))\<^sup>r \<equiv>\<^sub>i di (f)"
proof -
 have 1: "\<turnstile> (\<diamond> (f\<^sup>r))\<^sup>r \<equiv>\<^sub>i di ((f\<^sup>r)\<^sup>r)" using RDiamondEqvDi by blast
 have 2: "\<turnstile> di ((f\<^sup>r)\<^sup>r) \<equiv>\<^sub>i di f" using EqvReverseReverse by auto
 from 1 2 show ?thesis by auto
qed

lemma RBoxEqvBi:
 "\<turnstile> (\<box> f)\<^sup>r \<equiv>\<^sub>i bi (f\<^sup>r)"
using RDiamondEqvDi 
by (simp add: always_d_def not_d_def sometimes_d_def true_d_def)

lemma RRBoxEqvBi:
 "\<turnstile> (\<box> (f\<^sup>r))\<^sup>r \<equiv>\<^sub>i bi (f)"
proof -
 have 1: "\<turnstile> (\<box> (f\<^sup>r))\<^sup>r \<equiv>\<^sub>i bi ((f\<^sup>r)\<^sup>r)" using RBoxEqvBi by blast
 have 2: "\<turnstile> bi ((f\<^sup>r)\<^sup>r) \<equiv>\<^sub>i bi f" using EqvReverseReverse by auto
 from 1 2 show ?thesis by auto
qed

lemma RDiEqvDiamond:
 "\<turnstile> (di f)\<^sup>r \<equiv>\<^sub>i \<diamond> (f\<^sup>r)"
by (metis RTrue LeftChopEqvChop di_d_def rev_d.simps(5) sometimes_d_def) 

lemma RRDiEqvDiamond:
 "\<turnstile> (di (f\<^sup>r))\<^sup>r \<equiv>\<^sub>i \<diamond> (f)"
proof -
 have 1: "\<turnstile> (di (f\<^sup>r))\<^sup>r \<equiv>\<^sub>i \<diamond> ((f\<^sup>r)\<^sup>r)" using RDiEqvDiamond by blast
 have 2: "\<turnstile> \<diamond> ((f\<^sup>r)\<^sup>r) \<equiv>\<^sub>i \<diamond> f" using EqvReverseReverse by auto
 from 1 2 show ?thesis by auto
qed

lemma RBiEqvBox:
 "\<turnstile> (bi f)\<^sup>r \<equiv>\<^sub>i \<box> (f\<^sup>r)"
using RDiEqvDiamond 
by (simp add: bi_d_def di_d_def not_d_def true_d_def)

lemma RRBiEqvBox:
 "\<turnstile> (bi (f\<^sup>r))\<^sup>r \<equiv>\<^sub>i \<box> (f)"
proof -
 have 1: "\<turnstile> (bi (f\<^sup>r))\<^sup>r \<equiv>\<^sub>i \<box> ((f\<^sup>r)\<^sup>r)" using RBiEqvBox by blast
 have 2: "\<turnstile> \<box> ((f\<^sup>r)\<^sup>r) \<equiv>\<^sub>i \<box> f" using EqvReverseReverse by auto
 from 1 2 show ?thesis by auto
qed

lemma RDaEqvDa:
 "\<turnstile> (da f)\<^sup>r \<equiv>\<^sub>i da(f\<^sup>r)"
by (metis ChopAssoc da_d_def itl_prop(30) not_d_def rev_d.simps(1) 
          rev_d.simps(3) rev_d.simps(5) true_d_def)

lemma RRDaEqvDa:
 "\<turnstile> (da (f\<^sup>r))\<^sup>r \<equiv>\<^sub>i da(f)"
proof -
 have 1: "\<turnstile> (da (f\<^sup>r))\<^sup>r \<equiv>\<^sub>i da ((f\<^sup>r)\<^sup>r)" using RDaEqvDa by blast
 have 2: "\<turnstile> da ((f\<^sup>r)\<^sup>r) \<equiv>\<^sub>i da f" using EqvReverseReverse by auto
 from 1 2 show ?thesis by auto
qed

lemma RBaEqvBa:
 "\<turnstile> (ba f)\<^sup>r \<equiv>\<^sub>i ba(f\<^sup>r)"
using RDaEqvDa 
by (metis ba_d_def itl_prop(33) not_d_def rev_d.simps(1) rev_d.simps(3))

lemma RRBaEqvBa:
 "\<turnstile> (ba (f\<^sup>r))\<^sup>r \<equiv>\<^sub>i ba(f)"
proof -
 have 1: "\<turnstile> (ba (f\<^sup>r))\<^sup>r \<equiv>\<^sub>i ba ((f\<^sup>r)\<^sup>r)" using RBaEqvBa by blast
 have 2: "\<turnstile> ba ((f\<^sup>r)\<^sup>r) \<equiv>\<^sub>i ba f" using EqvReverseReverse by auto
 from 1 2 show ?thesis by auto
qed

lemma ChopCsImpCSChop:
 "\<turnstile> f;f\<^sup>\<star> \<supset>\<^sub>i f\<^sup>\<star>;f"
by (meson CSChopEqvChopOrRule CSChopEqvOrChopPlusChop ChopAssocB 
          ChopPlusElimWithoutMore EmptyYields prop19 prop21 prop28)

lemma CSChopImpChopCS:
 "\<turnstile> f\<^sup>\<star>;f \<supset>\<^sub>i f;f\<^sup>\<star>"
proof -
 have 1: "\<turnstile> (f\<^sup>r);(f\<^sup>r)\<^sup>\<star> \<supset>\<^sub>i (f\<^sup>r)\<^sup>\<star>;(f\<^sup>r)" using ChopCsImpCSChop by blast
 hence 2: "\<turnstile> ((f\<^sup>r);(f\<^sup>r)\<^sup>\<star> \<supset>\<^sub>i (f\<^sup>r)\<^sup>\<star>;(f\<^sup>r) )\<^sup>r " using ReverseEqv by blast
 have 3: "\<turnstile> ((f\<^sup>r);(f\<^sup>r)\<^sup>\<star> \<supset>\<^sub>i (f\<^sup>r)\<^sup>\<star>;(f\<^sup>r) )\<^sup>r \<equiv>\<^sub>i ( ((f\<^sup>r);(f\<^sup>r)\<^sup>\<star>)\<^sup>r \<supset>\<^sub>i ((f\<^sup>r)\<^sup>\<star>;(f\<^sup>r))\<^sup>r)" by simp
 have 4: "\<turnstile> ((f\<^sup>r);(f\<^sup>r)\<^sup>\<star>)\<^sup>r \<equiv>\<^sub>i ((f\<^sup>r)\<^sup>\<star> )\<^sup>r; (f\<^sup>r)\<^sup>r" by simp
 have 5: "\<turnstile> ((f\<^sup>r)\<^sup>\<star> )\<^sup>r; (f\<^sup>r)\<^sup>r \<equiv>\<^sub>i ((f\<^sup>r)\<^sup>r)\<^sup>\<star>;(f\<^sup>r)\<^sup>r" by simp
 have 6: "\<turnstile> (f\<^sup>r)\<^sup>r \<equiv>\<^sub>i f" using EqvReverseReverse itl_prop(30) by blast
 have 7: "\<turnstile> ((f\<^sup>r)\<^sup>r)\<^sup>\<star>;(f\<^sup>r)\<^sup>r \<equiv>\<^sub>i f\<^sup>\<star>;f" using "6" CSEqvCS ChopEqvChop by blast
 have 8: "\<turnstile> ((f\<^sup>r);(f\<^sup>r)\<^sup>\<star>)\<^sup>r \<equiv>\<^sub>i f\<^sup>\<star>;f" using "7" "5" by auto
 have 9: "\<turnstile> ((f\<^sup>r)\<^sup>\<star>;(f\<^sup>r))\<^sup>r \<equiv>\<^sub>i (f\<^sup>r)\<^sup>r;((f\<^sup>r)\<^sup>\<star>)\<^sup>r" by simp
 have 10: "\<turnstile> (f\<^sup>r)\<^sup>r;((f\<^sup>r)\<^sup>\<star>)\<^sup>r \<equiv>\<^sub>i (f\<^sup>r)\<^sup>r; ((f\<^sup>r)\<^sup>r)\<^sup>\<star>" by simp
 have 11: "\<turnstile> (f\<^sup>r)\<^sup>r; ((f\<^sup>r)\<^sup>r)\<^sup>\<star> \<equiv>\<^sub>i f;f\<^sup>\<star>" using "6" ChopPlusEqvChopPlus by blast
 have 12: "\<turnstile> ((f\<^sup>r);(f\<^sup>r)\<^sup>\<star>)\<^sup>r \<equiv>\<^sub>i f;f\<^sup>\<star>" using "9" "10" "11" 
 by (metis "2" ChopCsImpCSChop itl_prop(31) prop03 rev_d.simps(3) rev_d.simps(5) rev_d.simps(6)) 
 from 2 3 8 12 show ?thesis by auto
qed

lemma CSChopEqvChopCS:
 "\<turnstile> f;f\<^sup>\<star> \<equiv>\<^sub>i f\<^sup>\<star>;f"
using ChopCsImpCSChop CSChopImpChopCS using itl_prop(31) by blast

lemma TrueChopSkipEqvSkipChopTrue:
 "\<turnstile>  true\<^sub>i;skip \<equiv>\<^sub>i skip;true\<^sub>i"
proof -
 have 1: "\<turnstile>skip;skip\<^sup>\<star> \<equiv>\<^sub>i skip\<^sup>\<star>;skip " using CSChopEqvChopCS by blast
 have 2: "\<turnstile> skip\<^sup>\<star> \<equiv>\<^sub>i true\<^sub>i" using CSSkip by simp
 have 3: "\<turnstile> skip;skip\<^sup>\<star> \<equiv>\<^sub>i skip;true\<^sub>i" using "2" using RightChopEqvChop by blast
 have 4: "\<turnstile> skip\<^sup>\<star>;skip \<equiv>\<^sub>i true\<^sub>i;skip" using "2" using LeftChopEqvChop by blast
 from 1 3 4 show ?thesis using itl_prop(30) prop03 by blast
qed

lemma RMoreEqvMore:
 " \<turnstile> more\<^sup>r \<equiv>\<^sub>i more"
by (metis TrueChopSkipEqvSkipChopTrue more_d_def next_d_def not_d_def rev_d.simps(1) 
          rev_d.simps(3) rev_d.simps(4) rev_d.simps(5) true_d_def)

lemma REmptyEqvEmpty:
 "\<turnstile> empty\<^sup>r \<equiv>\<^sub>i empty "
by (metis RMoreEqvMore empty_d_def not_d_def prop01 rev_d.simps(1) rev_d.simps(3))

lemma RInitEqvFin:
 "\<turnstile> (init f)\<^sup>r \<equiv>\<^sub>i fin(f\<^sup>r)"
proof -
 have 1: "\<turnstile> (init f)\<^sup>r \<equiv>\<^sub>i ((f \<and>\<^sub>i empty);true\<^sub>i)\<^sup>r" 
      by (metis AndChopCommute REqvRule init_d_def)
 have 2: "\<turnstile> ((f \<and>\<^sub>i empty);true\<^sub>i)\<^sup>r \<equiv>\<^sub>i (true\<^sub>i;(f \<and>\<^sub>i empty)\<^sup>r)" 
      using RTrue by auto
 have 3: "\<turnstile> true\<^sub>i;(f \<and>\<^sub>i empty)\<^sup>r \<equiv>\<^sub>i true\<^sub>i;(f\<^sup>r \<and>\<^sub>i empty)" 
      by (meson ChopEqvChop RAnd REmptyEqvEmpty RTrue itl_prop(30) prop05 prop21)
 have 4: "\<turnstile> true\<^sub>i;(f\<^sup>r \<and>\<^sub>i empty) \<equiv>\<^sub>i fin(f\<^sup>r)"  
      using FinEqvTrueChopAndEmpty itl_prop(30) by blast
from 1 2 3 4 show ?thesis by auto
qed

lemma RRInitEqvFin:
 "\<turnstile> (init (f\<^sup>r))\<^sup>r \<equiv>\<^sub>i fin(f)"
proof -
 have 1: "\<turnstile> (init (f\<^sup>r))\<^sup>r \<equiv>\<^sub>i fin ((f\<^sup>r)\<^sup>r)" using RInitEqvFin by blast
 have 2: "\<turnstile> fin ((f\<^sup>r)\<^sup>r) \<equiv>\<^sub>i fin f" using EqvReverseReverse by auto
 from 1 2 show ?thesis by auto
qed

lemma RFinEqvInit:
 "\<turnstile> (fin f)\<^sup>r \<equiv>\<^sub>i init (f\<^sup>r)"
proof -
 have 1: "\<turnstile> fin f \<equiv>\<^sub>i true\<^sub>i;(f \<and>\<^sub>i empty)" 
      using FinEqvTrueChopAndEmpty by auto
 have 2: "\<turnstile> (fin f)\<^sup>r \<equiv>\<^sub>i (true\<^sub>i;(f \<and>\<^sub>i empty))\<^sup>r" 
      using "1" REqvRule by blast
 have 3: "\<turnstile> (true\<^sub>i;(f \<and>\<^sub>i empty))\<^sup>r \<equiv>\<^sub>i (f\<and>\<^sub>i empty)\<^sup>r;true\<^sub>i" 
      using RTrue by auto
 have 4: "\<turnstile> (f\<and>\<^sub>i empty)\<^sup>r;true\<^sub>i \<equiv>\<^sub>i (f\<^sup>r \<and>\<^sub>i empty);true\<^sub>i" 
      using LeftChopEqvChop RAnd REmptyEqvEmpty prop03 prop05 by blast
 have 5: "\<turnstile> (f\<^sup>r \<and>\<^sub>i empty);true\<^sub>i \<equiv>\<^sub>i init(f\<^sup>r)" 
      by auto
 from 1 2 3 4 5 show ?thesis by auto
qed

lemma RRFinEqvInit:
 "\<turnstile> (fin (f\<^sup>r))\<^sup>r \<equiv>\<^sub>i init (f)"
proof -
 have 1: "\<turnstile> (fin (f\<^sup>r))\<^sup>r \<equiv>\<^sub>i init ((f\<^sup>r)\<^sup>r)" using RFinEqvInit by blast
 have 2: "\<turnstile> init ((f\<^sup>r)\<^sup>r) \<equiv>\<^sub>i init f" using EqvReverseReverse by auto
 from 1 2 show ?thesis by auto
qed

lemma NextDiamondEqvDiamondNext:
 " \<turnstile> \<circle>( \<diamond> f) \<equiv>\<^sub>i \<diamond>(\<circle>f)"
proof -
 have  1: "\<turnstile> true\<^sub>i;skip \<equiv>\<^sub>i skip;true\<^sub>i  " by (rule TrueChopSkipEqvSkipChopTrue)
 hence 2: "\<turnstile> (true\<^sub>i;skip);f \<equiv>\<^sub>i( skip;true\<^sub>i);f " using LeftChopEqvChop by blast
 have  3: "\<turnstile> (true\<^sub>i;skip);f \<equiv>\<^sub>i  true\<^sub>i;(skip;f)  " using ChopAssoc itl_prop(30) by blast
 have  4: "\<turnstile> ( skip;true\<^sub>i);f \<equiv>\<^sub>i  skip;(true\<^sub>i;f) " using ChopAssoc itl_prop(30) by blast
 from 2 3 4 show ?thesis by (simp add: next_d_def)
qed

lemma WeakNextBoxInduct:
 assumes "\<turnstile> wnext  (\<box> f) \<supset>\<^sub>i f "
 shows   "\<turnstile> f "
proof -
 have   1: "\<turnstile> wnext  (\<box> f) \<supset>\<^sub>i f" using assms by blast
 hence  2: "\<turnstile> \<not>\<^sub>i f  \<supset>\<^sub>i \<not>\<^sub>i ( wnext  (\<box> f))  "  using prop27 by blast   
 hence  3: "\<turnstile> \<not>\<^sub>i f  \<supset>\<^sub>i   \<circle>  (\<not>\<^sub>i (\<box> f)) "  by simp 
 have   4: "\<turnstile> \<not>\<^sub>i (\<box> f) \<equiv>\<^sub>i   (\<diamond> \<not>\<^sub>i f) " by auto
 hence  5: "\<turnstile> \<circle>(\<not>\<^sub>i (\<box> f)) \<equiv>\<^sub>i  \<circle> (\<diamond> \<not>\<^sub>i f) " by auto
 have   6: "\<turnstile> \<not>\<^sub>i f  \<supset>\<^sub>i   \<circle>  (\<diamond> \<not>\<^sub>i f) " using "3" "5" by auto
 have   7: "\<turnstile> \<circle>  (\<diamond> \<not>\<^sub>i f) \<equiv>\<^sub>i \<diamond>( \<circle> \<not>\<^sub>i f)  " using NextDiamondEqvDiamondNext by blast
 have   8: "\<turnstile> \<not>\<^sub>i f  \<supset>\<^sub>i   \<diamond>( \<circle> \<not>\<^sub>i f)  " using "6" "7" by auto
 have   9: "\<turnstile> \<diamond>(\<not>\<^sub>i f)  \<supset>\<^sub>i   \<diamond>(\<diamond>( \<circle> \<not>\<^sub>i f)) " using "8" DiamondImpDiamond by blast
 have  10: "\<turnstile> \<diamond>(\<diamond>( \<circle> \<not>\<^sub>i f)) \<equiv>\<^sub>i \<diamond>( \<circle> \<not>\<^sub>i f) " using DiamondDiamondEqvDiamond by blast
 have  11: "\<turnstile> \<diamond>(\<not>\<^sub>i f)  \<supset>\<^sub>i \<diamond>( \<circle> \<not>\<^sub>i f) " using "9" "10" by auto
 have  12: "\<turnstile> \<diamond>(\<not>\<^sub>i f)  \<supset>\<^sub>i \<circle> (\<diamond> \<not>\<^sub>i f) " using "7" "11" by auto
 hence 13: "\<turnstile> \<not>\<^sub>i( \<diamond>(\<not>\<^sub>i f))"  using NextLoop by blast
 hence 14: "\<turnstile> \<box>f " by (simp add: always_d_def)
 have  15: "\<turnstile> \<box>f \<supset>\<^sub>i f " using BoxElim by blast
 from 14 15 show ?thesis using MP by blast 
qed

end


